
/******************************************************************************
* MODULE     : tft_test.cpp
* DESCRIPTION: Test TFT
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "tft.hpp"
using namespace mmx;

#define K modular<FFTP>

template<typename C> C*
make (nat n) {
  C* a= mmx_new<C> (n);
  for (nat i=0; i<n; i++)
    a[i]= 0;
  return a;
}

template<typename C> C*
make (nat s, nat n) {
  C* a= make<C> (n);
  for (nat i=0; i<s; i++)
    a[i]= (i+1)*(i+1);
  return a;
}

nat
up_2 (nat i) {
  return 1 << (log_2 (i-1) + 1);
}

int
main () {
  K* a= make<K> (11, 16);
  a[6]= 0;
  a[7]= 0;
  print (a, 16); cout << "\n";
  tft_tree<K> t= tft_segment (a, 16);
  print (t); cout << "\n";
  fft (a, 16, false);
  print (a, 16); cout << "\n";  
  tft (t);
  print (t); cout << "\n";
  itft (t);
  print (t); cout << "\n";
  mmx_delete<K> (a, 16);

  for (nat s=2; s < 100000; s= (nat) (((double) s) * 1.5)) {
    K* a= make<K> (s, up_2 (s));
    K* b= make<K> (s, up_2 (s));

    /*
    tft_tree<K> t= tft_segment (a, up_2 (s));
    print (t); cout << "\n";
    tft (t);
    print (t); cout << "\n";
    itft (t);
    print (t); cout << "\n";
    */

    //print (a, s);
    tft (a, s, log_2 (s-1) + 1);
    //print (a, s);
    itft (a, s, log_2 (s-1) + 1);
    //print (a, s);
    for (nat j=0; j<s; ++j)
      assert (a[j] == b[j]);
    cout << "TFT on " << s << " entries works\n";
    mmx_delete<K> (a, up_2 (s));
    mmx_delete<K> (b, up_2 (s));
  }

  return 0;
}
