
/******************************************************************************
* MODULE     : matrix_glue.hpp
* DESCRIPTION: Glue for matrices
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_MATRIX_GLUE_HPP
#define __MMX_MATRIX_GLUE_HPP
#include "dense_matrix.hpp"
#include "dense_pivot.hpp"
#include "category_meta.hpp"
#include "category_lift_glue.hpp"
#include "tuple.hpp"
#include "row_tuple.hpp"
#include "integer.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Matrix matrix<C>

TMPL row_tuple<C>
make_row (const tuple<C>& t) {
  return row_tuple<C> (cons (GEN_ROW, cdr (*t)));
}

template<typename Cond, typename C> void
define_standard_row () {
  define_type<always,row_tuple<C> > (gen ("Row", type_name<C> ()));
  define<always> (GEN_ROW, make_row<C>);
  define_from_iterator<always,C> (GEN_ROW);
}

TMPL Matrix
make_matrix (const tuple<row_tuple<C> >& t) {
  nat i, j, rows= N(t), cols= N(t[0]);
  C dummy;
  Matrix r (dummy, rows, cols);
  for (i=0; i<rows; i++) {
    ASSERT (N(t[i]) == cols, "unequal row lengths");
    for (j=0; j<cols; j++)
      r(i,j)= t[i][j];
  }
  return r;
}

TMPL Matrix
make_matrix_row (const tuple<C>& t) {
  nat j, rows= 1, cols= N(t);
  C dummy;
  Matrix r (dummy, rows, cols);
  for (j=0; j<cols; j++)
    r(0,j)= t[j];
  return r;
}

TMPL
class matrix_element_rep: public alias_rep<C> {
  alias<Matrix > a;
  nat row, col;
public:
  inline matrix_element_rep (const alias<Matrix >& a2, nat row2, nat col2):
    a (a2), row (row2), col (col2) {}
  C get () const { return read (get_alias (a), row, col); }
  C& open () const { return open_alias (a) (row, col); }
  void close () const { close_alias (a); }
};

TMPL alias<C>
matrix_element (const alias<Matrix >& a, const integer& i, const integer& j) {
  return new matrix_element_rep<C> (a, as_int (i), as_int (j));
}

TMPL inline integer matrix_size (const Matrix& m) { return N(m); }
TMPL inline integer matrix_rows (const Matrix& m) { return rows (m); }
TMPL inline integer matrix_cols (const Matrix& m) { return cols (m); }

TMPL inline integer matrix_rank (const Matrix& m) { return rank (m); }
TMPL inline Matrix matrix_triangulate (const Matrix& m) {
  return triangulate (m); }
TMPL inline Matrix matrix_ker (const Matrix& m) { return kernel (m); }
TMPL inline Matrix matrix_im (const Matrix& m) { return image (m); }
TMPL inline C matrix_det (const Matrix& m) { return det (m); }
TMPL inline Matrix matrix_invert (const Matrix& m) { return invert (m); }
TMPL inline Matrix matrix_hjoin (const Matrix& m1, const Matrix& m2) {
  return horizontal_join (m1, m2); }
TMPL inline Matrix matrix_vjoin (const Matrix& m1, const Matrix& m2) {
  return vertical_join (m1, m2); }
TMPL inline Matrix matrix_krylov (const Matrix& m1, const Matrix& m2) {
  return krylov (m1, m2); }

template<typename Hyp, typename C> void
define_matrix () {
  typedef both<has<C,ring_cat>,Hyp> Cond;

  define_standard_row<Cond,C> ();
  define_type<Cond,Matrix > (gen ("Matrix", type_name<C> ()));
  define<Cond> ("matrix", make_matrix<C>);
  define<Cond> ("matrix", make_matrix_row<C>);
  define_upgrader<Cond,Matrix,C> ();
  define_from_iterator<Cond,C> ("matrix");

  define<Cond> (GEN_SIZE, matrix_size<C>);
  define<Cond> ("rows", matrix_rows<C>);
  define<Cond> ("columns", matrix_cols<C>);
  define<Cond> (GEN_ACCESS, matrix_element<C>);

  define_ring<Cond,Matrix > ();
  lift_scalar_ring_ops<Cond,Matrix,C> ();

  define<Cond> ("rank", matrix_rank<C>);
  define<Cond> ("triangulate", matrix_triangulate<C>);
  define<Cond> ("ker", matrix_ker<C>);
  define<Cond> ("im", matrix_im<C>);
  define<Cond> ("det", matrix_det<C>);
  define<Cond> ("invert", matrix_invert<C>);
  define<Cond> ("horizontal_join", matrix_hjoin<C>);
  define<Cond> ("vertical_join", matrix_vjoin<C>);
  define<Cond> ("krylov", matrix_krylov<C>);
}

#undef TMPL
#undef Matrix
} // namespace mmx
#endif // __MMX_MATRIX_GLUE_HPP
