
/******************************************************************************
* MODULE     : mould.hpp
* DESCRIPTION: Moulds
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_MOULD_HPP
#define __MMX_MOULD_HPP
#include "string.hpp"
#include "list.hpp"
#include "table.hpp"
#include "series.hpp"
namespace mmx {
#define TMPL_DEF template<typename C, typename X, typename V=naive_multiply>
#define TMPL template<typename C, typename X, typename V>
#define Mould mould<C,X,V>
#define Mould_rep mould_rep<C,X,V>
#define M list<X>
TMPL class mould;
TMPL generic flatten (const Mould& f, const generic& z);

/******************************************************************************
* Moulds
******************************************************************************/

TMPL_DEF
class mould_rep REP_STRUCT {
public:                 // should be protected
  table<C,M>     t;     // coefficients
  table<bool,X>* deps;  // dependencies

  inline mould_rep (): t (C(0)), deps (NULL) {}
  inline virtual ~mould_rep () { if (deps != NULL) delete deps; }
  virtual generic expression () const = 0;
  inline Mould me () const;

  virtual C compute (const M& x) = 0;
  virtual table<bool,X> compute_deps () = 0;
  friend class Mould;
};

TMPL_DEF
class mould {
INDIRECT_PROTO_3 (mould, mould_rep, C, X, V)
public:
  mould ();
  template<typename T> mould (const T& c);
  template<typename T> mould (const T& c, nat deg);
  template<typename T> mould (const T& c, const M& x);
  mould (C (*coeffs) (const M& x), const string& name= "explicit");
  C operator [] (const M& x) const;
};
INDIRECT_IMPL_3 (mould, mould_rep, typename C, C, typename X, X, typename V, V)

STYPE_TO_TYPE(TMPL,scalar_type,Mould,C);

/******************************************************************************
* Routines on moulds
******************************************************************************/

/*
TMPL nat
hash (const Mould& s) {
  return 0;
}
*/

TMPL inline Mould
Mould_rep::me () const {
  return Mould (this, true);
}

TMPL C
Mould::operator [] (const M& x) const {
  if (!rep->t->contains (x))
    rep->t[x]= rep->compute (x);
  return read (rep->t, x);
}

TMPL table<bool,X>
dependencies (const Mould& f) {
  Mould_rep* rep= inside (f);
  if (rep->deps == NULL)
    rep->deps= new table<bool,X> (rep->compute_deps ());
  return *rep->deps;
}

/******************************************************************************
* Routines for the underlying monoid
******************************************************************************/

template<typename X> nat
weight (const X& x) {
  return 1;
}

template<typename X> nat
weight (const list<X>& l) {
  if (nil (l)) return 0;
  return weight (car (l)) + weight (cdr (l));
}

nat
weight (const nat& n) {
  return n;
}

template<typename X> bool
can_list (const table<bool,X>& t) {
  return !I(t);
}

bool
can_list (const table<bool,nat>& t) {
  (void) t; return true;
}

template<typename X> list<X>
filter_lower (const list<X>& l, nat wt) {
  if (nil (l)) return l;
  if (weight (car (l)) > wt) return filter_lower (cdr (l), wt);
  return cons (car (l), filter_lower (cdr (l), wt));
}

template<typename X> list<X>
list_until_weight (const table<bool,X>& t, nat wt) {
  return filter_lower (list<X> (entries (t)), wt);
}

list<nat>
list_until_weight (const table<bool,nat>& t, nat wt) {
  if (I (t)) {
    list<nat> l;
    for (nat n=wt; n!=0; n--)
      l= cons (n, l);
    return l;
  }
  return filter_lower (list<nat> (entries (t)), wt);
}

template<typename X> list<M >
homogeneous (const M& prefix, const list<X>& l, nat wt) {
  if (wt == 0) return list<M > (prefix);
  else if (nil (l)) return list<M > ();
  else {
    list<M > r;
    for (list<X> it= l; !nil (it); it= cdr (it))
      if (weight (car (it)) <= wt)
	r= r * homogeneous (prefix * M (car (it)), l,
			    wt - weight (car (it)));
    return r;
  }
}

template<typename X> list<M >
homogeneous (const table<bool,X>& t, nat wt) {
  return homogeneous (M (), list_until_weight (t, wt), wt);
}

template<typename X> list<M >
up_till_weight (const table<bool,X>& t, nat wt) {
  if (wt == 0) return homogeneous (t, wt);
  else return up_till_weight (t, wt-1) * homogeneous (t, wt);
}

/******************************************************************************
* Printing moulds
******************************************************************************/

TMPL generic
flatten (const Mould& f, const list<M >& l) {
  if (nil (l)) return "...";
  return flatten (f[car (l)]) * flatten (car (l)) + flatten (f, cdr (l));
}

TMPL generic
flatten (const Mould& f, nat wt) {
  table<bool,X> t= dependencies (f);
  if (!can_list (t)) return f->expression ();
  return flatten (f, up_till_weight (t, wt));
}

TMPL generic
flatten (const Mould& f) {
  return flatten (f, 6);
}

PRINT_SUGAR(TMPL,Mould)

/******************************************************************************
* Explicit moulds
******************************************************************************/

TMPL
class elementary_mould_rep: public Mould_rep {
  C   c;
  nat deg;
public:
  elementary_mould_rep (const C& c2, nat deg2): c (c2), deg (deg2) {}
  generic expression () const {
    return flatten (c) * pow (generic ("I"), flatten (deg)); }
  C compute (const M& x) {
    if (N(x) == deg) return c; else return C (0); }
  table<bool,X> compute_deps () {
    return table<bool,X> (deg != 0); }
};

TMPL
class term_mould_rep: public Mould_rep {
  C c;
  M x;
public:
  term_mould_rep (const C& c2, const M& x2): c (c2), x (x2) {}
  generic expression () const {
    return flatten (c) * flatten (x); }
  C compute (const M& x2) {
    if (x == x2) return c; else return C (0); }
  table<bool,X> compute_deps () {
    table<bool,X> t (false);
    for (M y= x; !nil (y); y= cdr (y))
      t [car (y)]= true;
    return t; }
};

TMPL
Mould::mould () {
  rep= new elementary_mould_rep<C,X,V> (C(0), 0);
}

TMPL template<typename T>
Mould::mould (const T& c) {
  rep= new elementary_mould_rep<C,X,V> (C(c), 0);
}

TMPL template<typename T>
Mould::mould (const T& c, nat deg) {
  rep= new elementary_mould_rep<C,X,V> (C(c), deg);
}

TMPL template<typename T>
Mould::mould (const T& c, const M& x) {
  rep= new term_mould_rep<C,X,V> (C(c), x);
}

/******************************************************************************
* Abstract operations on moulds
******************************************************************************/

template<typename Op, typename C, typename X, typename V>
class unary_mould_rep : public Mould_rep {
protected:
  Mould f;
public:
  inline unary_mould_rep (const Mould& f2): f(f2) {}
  generic expression () const {
    return Op::op (flatten (f)); }
  virtual C compute (const M& x) {
    return Op::op (f[x]); }
  table<bool,X> compute_deps () {
    return dependencies (f); }
};

template<typename Op, typename C, typename X, typename V> inline Mould
unary_mould (const Mould& f) {
  return (Mould_rep*) new unary_mould_rep<Op,C,X,V> (f);
}

template<typename Op, typename C, typename X, typename V>
class binary_mould_rep : public Mould_rep {
protected:
  Mould f, g;
public:
  inline binary_mould_rep (const Mould& f2, const Mould& g2):
    f(f2), g (g2) {}
  generic expression () const {
    return Op::op (flatten (f), flatten (g)); }
  virtual C compute (const M& x) {
    return Op::op (f[x], g[x]); }
  table<bool,X> compute_deps () {
    return dependencies (f) | dependencies (g); }
};

template<typename Op, typename C, typename X, typename V> inline Mould
binary_mould (const Mould& f, const Mould& g) {
  return (Mould_rep*) new binary_mould_rep<Op,C,X,V> (f, g);
}

template<typename Op, typename C, typename X, typename V, typename E>
class extrary_mould_rep : public Mould_rep {
protected:
  Mould f;
  E e;
public:
  inline extrary_mould_rep (const Mould& f2, const E& e2):
    f(f2), e (e2) {}
  generic expression () const {
    return Op::op (flatten (f), flatten (e)); }
  virtual C compute (const M& x) {
    return Op::op (f[x], e); }
  table<bool,X> compute_deps () {
    return dependencies (f); }
};

template<typename Op, typename C, typename X, typename V, typename E>
inline Mould
extrary_mould (const Mould& f, const E& e) {
  return (Mould_rep*) new extrary_mould_rep<Op,C,X,V,E> (f, e);
}

/******************************************************************************
* Coefficient-wise operations
******************************************************************************/

TMPL Mould
operator + (const Mould& f, const Mould& g) {
  return binary_mould<add_op> (f, g);
}

TMPL Mould
operator - (const Mould& f) {
  return unary_mould<neg_op> (f);
}

TMPL Mould
operator - (const Mould& f, const Mould& g) {
  return binary_mould<sub_op> (f, g);
}

TMPL Mould
operator * (const Mould& f, const C& c) {
  return extrary_mould<rmul_op> (f, c);
}

TMPL Mould
operator * (const C& c, const Mould& f) {
  return extrary_mould<lmul_op> (f, c);
}

TMPL Mould
operator / (const Mould& f, const C& c) {
  return extrary_mould<rdiv_op> (f, c);
}

TMPL Mould
quo (const Mould& f, const C& c) {
  return extrary_mould<rquo_op> (f, c);
}

TMPL Mould
rem (const Mould& f, const C& c) {
  return extrary_mould<rrem_op> (f, c);
}

/******************************************************************************
* Other linear time operations
******************************************************************************/

TMPL
class lres_mould_rep: public mould_rep<C,X,V> {
  Mould f;
  M     x;
public:
  inline lres_mould_rep (const Mould& f2, const M& x2):
    f (f2), x (x2) {}
  generic expression () const {
    return lres (flatten (f), flatten (x)); }
  C compute (const M& x2) {
    return f[x * x2]; }
  table<bool,X> compute_deps () {
    return dependencies (f); }
};

TMPL Mould
lres (const Mould& f, const M& x) {
  return (Mould_rep*) new lres_mould_rep<C,X,V> (f, x);
}

TMPL
class rres_mould_rep: public mould_rep<C,X,V> {
  Mould f;
  M     x;
public:
  inline rres_mould_rep (const Mould& f2, const M& x2):
    f (f2), x (x2) {}
  generic expression () const {
    return rres (flatten (f), flatten (x)); }
  C compute (const M& x2) {
    return f[x2 * x]; }
  table<bool,X> compute_deps () {
    return dependencies (f); }
};

TMPL Mould
rres (const Mould& f, const M& x) {
  return (Mould_rep*) new rres_mould_rep<C,X,V> (f, x);
}

TMPL
class hom_part_mould_rep: public mould_rep<C,X,V> {
  Mould f;
  nat   wt;  
public:
  inline hom_part_mould_rep (const Mould& f2, nat wt2):
    f (f2), wt (wt2) {}
  generic expression () const {
    return access (flatten (f), flatten (wt)); }
  C compute (const M& x) {
    if (weight (x) == wt) return f[x];
    else return 0; }
  table<bool,X> compute_deps () {
    return dependencies (f); }
};

TMPL Mould
hom_part (const Mould& f, nat wt) {
  return (Mould_rep*) new hom_part_mould_rep<C,X,V> (f, wt);
}

/******************************************************************************
* Multiplication
******************************************************************************/

TMPL
class mul_mould_rep: public binary_mould_rep<mul_op,C,X,V> {
public:
  inline mul_mould_rep (const Mould& f, const Mould& g):
    binary_mould_rep<mul_op,C,X,V> (f, g) {}
  C compute (const M& x) {
    M x1, x2= x;
    C acc (0);
    for (; true; x1= x1 * M (car (x2)), x2= cdr (x2)) {
      acc += this->f[x1] * this->g[x2];
      if (nil (x2)) break;
    }
    return acc;
  }
};

TMPL Mould
operator * (const Mould& f, const Mould& g) {
  return (Mould_rep*) new mul_mould_rep<C,X,V> (f, g);
}

ARITH_INT_SUGAR(TMPL,Mould,C);

/******************************************************************************
* Composition
******************************************************************************/

TMPL
class compose_series_mould_rep: public mould_rep<C,X,V> {
  series<C> f;
  Mould     g;
  nat       deg;
  Mould     sum;
  Mould     prod;
public:
  inline compose_series_mould_rep (const series<C>& f2, const Mould& g2):
    f (f2), g (g2), deg (0), sum (C (0)), prod (C (1)) {}
  generic expression () const {
    return flatten (f, flatten (g)); }
  C compute (const M& x) {
    if (N(x) >= deg) {
      for (; deg <= N(x); deg++) {
	sum = sum + f[deg] * prod;
	prod= g * prod;
      }
    }
    return sum[x];
  }
  table<bool,X> compute_deps () {
    return dependencies (g); }
};

TMPL Mould
compose (const series<C>& f, const Mould& g) {
  ASSERT (g [M ()] == C(0), "composition should preserve the origin");
  return (Mould_rep*) new compose_series_mould_rep<C,X,V> (f, g);
}

TMPL
class compose_mould_rep: public mould_rep<C,X,V> {
  mould<C,nat,V> f;
  Mould          g;
public:
  inline compose_mould_rep (const mould<C,nat,V>& f2, const Mould& g2):
    f (f2), g (g2) {}
  generic expression () const {
    return compose (flatten (f), flatten (g)); }
  list<list<nat> > decompositions (list<nat> prefix, nat n) {
    if (n == 0) return list<list<nat> > (prefix);
    list<list<nat> > l;
    for (nat i=1; i<=n; i++)
      l= l * decompositions (prefix * list<nat> (i), n-i);
    return l;
  }
  C compute (const M& x) {
    //cout << "-----------\n";
    //cout << "x= " << x << "\n";
    C sum= 0;
    list<list<nat> > l= decompositions (list<nat> (), N(x));
    for (; !nil (l); l= cdr (l)) {
      list<nat> it= car (l);
      M y= copy (x);
      C prod= 1;
      list<nat> mon;
      //list<M >  aux;
      for (; !nil (it); it= cdr (it)) {
	M w= range (y, 0, car (it));
	prod= prod * g[w];
	mon = mon * list<nat> (weight (w));
	//aux = aux * list<M  > (w);
	y= range (y, car (it), N(y));
      }
      //cout << mon << ", " << aux << " -> " << f[mon] << ", " << prod << "\n";
      sum= sum + f[mon] * prod;
    }
    //cout << "sum= " << sum << "\n";
    return sum;
  }
  table<bool,X> compute_deps () {
    return dependencies (g); }
};

TMPL Mould
compose (const mould<C,nat,V>& f, const Mould& g) {
  ASSERT (g [M ()] == C(0), "composition should preserve the origin");
  return (Mould_rep*) new compose_mould_rep<C,X,V> (f, g);
}

/******************************************************************************
* Elementary functions
******************************************************************************/

TMPL Mould
invert (const Mould& f) {
  C c= f[M()];
  series<C> z (C(1), 1);
  series<C> s= 1 / (series<C> (c) + z);
  return compose (s, f - Mould (c));
}

TMPL Mould
operator / (const Mould& f, const Mould& g) {
  return f * invert (g);
}

TMPL Mould
sqrt (const Mould& f) {
  C c= f[M()];
  series<C> z (C(1), 1);
  series<C> s= sqrt (series<C> (1) + z / series<C> (c));
  return sqrt (c) * compose (s, f - Mould (c));
}

TMPL Mould
exp (const Mould& f) {
  C c= f[M()];
  series<C> z (C(1), 1);
  series<C> s= exp (z);
  return exp (c) * compose (s, f - Mould (c));
}

TMPL Mould
log (const Mould& f) {
  C c= f[M()];
  series<C> z (C(1), 1);
  series<C> s= log (series<C> (1) + z / series<C> (c));
  return Mould (log (c)) + compose (s, f - Mould (c));
}

#undef TMPL_DEF
#undef TMPL
#undef Mould
#undef Mould_rep
#undef M
} // namespace mmx
#endif // __MMX_MOULD_HPP
