
/******************************************************************************
* MODULE     : mould_test.cc
* DESCRIPTION: Testing moulds
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rational.hpp"
#include "averages.hpp"
#include "dense_vector.hpp"
//#include "symbolic.hpp"
//#include "mould_symbolic.hpp"
using namespace mmx;

#define C rational
//#define C symbolic<rational>
#define X string
#define V naive_multiply
#define Mould mould<C,X,V>
#define MOULD mould<C,nat,V>

int
main () {
  list<string> lm (string ("-"));
  list<string> rm (string ("+"));
  Mould l= l_mould<C,X,V> ();
  Mould r= r_mould<C,X,V> ();
  Mould I= l+r;
  Mould mul= mul_mould<C,X,V> ();
  Mould mur= mur_mould<C,X,V> ();
  Mould lur= lur_mould<C,X,V> ();
  Mould rul= rul_mould<C,X,V> ();
  Mould dur= lur - 1;
  Mould dul= rul - 1;
  Mould mun= mun_mould<C,X,V> ();
  Mould dun= dun_mould<C,X,V> ();
  Mould mon= mon_mould<C,X,V> ();
  Mould dom= dom_mould<C,X,V> ();

  cout << "mul\t= " << mul << "\n";
  cout << "mur\t= " << mur << "\n";
  cout << "lur\t= " << lur << "\n";
  cout << "rul\t= " << rul << "\n";
  cout << "dul\t= " << dul << "\n";
  cout << "dur\t= " << dur << "\n";
  /*
  cout << "dun:dul\t= " << decompose (dun, dul) << "\n";
  cout << "dul:dun\t= " << decompose (dul, dun) << "\n";
  cout << "dom:dul\t= " << decompose (dom, dul) << "\n";
  cout << "dul:dom\t= " << decompose (dul, dom) << "\n";
  cout << "dom:dun\t= " << decompose (dom, dun) << "\n";
  cout << "dun:dom\t= " << decompose (dun, dom) << "\n";
  */

  cout << "l\t= " << l << "\n";
  cout << "r\t= " << r << "\n";
  cout << "I\t= " << I << "\n";
  cout << "I+l\t= " << I + l << "\n";
  cout << "l-r\t= " << l - r << "\n";
  cout << "I I\t= " << I * I << "\n";
  cout << "exp I\t= " << exp (I) << "\n";
  cout << "\n";

  cout << "mul\t= " << mul << "\n";
  cout << "mur\t= " << mur << "\n";
  cout << "lur\t= " << lur << "\n";
  cout << "rul\t= " << rul << "\n";
  cout << "mun\t= " << mun << "\n";
  cout << "dun\t= " << dun << "\n";
  cout << "mon\t= " << mon << "\n";
  cout << "dom\t= " << dom << "\n";
  cout << "\n";

  cout << "mun :/ (r-l)\t= " << rres (mun, rm) - rres (mun, lm) << "\n";
  cout << "mon :/ (r-l)\t= " << rres (mon, rm) - rres (mon, lm) << "\n";
  cout << "\n";

  MOULD Z= MOULD (C(1), (nat) 1);
  MOULD T= MOULD (C(1), list<nat> (1, 2));
  cout << "Z\t= " << Z << "\n";
  cout << "T\t= " << T << "\n";
  cout << "exp Z\t= " << exp (Z) << "\n";
  cout << "Z o dom\t= " << compose (Z, dom) << "\n";
  cout << "T o dom\t= " << compose (T, dom) << "\n";
  cout << "\n";

  /*
  MOULD M= symbolic_mould<C,nat,V> ("M");
  cout << "M\t= " << M << "\n";
  cout << "exp M\t= " << exp (M) << "\n";
  cout << "\n";

  cout << "Z\t\t\t= " << Z << "\n";
  cout << "M o Z\t\t\t= " << compose (M, Z) << "\n";
  cout << "solve (M o Z = T, M)\t= "
       << solve_mould (compose (M, Z) - compose (T, Z), M) << "\n";
  cout << "decompose (dom, dun)\t= " << decompose (dom, dun) << "\n";

  Mould xun= (rres (dun, rm) - rres (dun, lm)) / 2;
  Mould zun= xun / mul;
  cout << "xun\t\t\t= " << xun << "\n";
  cout << "xun*(r-l)-dun\t\t= " << xun*(r-l)-dun << "\n";
  cout << "zun\t\t\t= " << zun << "\n";
  cout << "decompose (zun, dun)\t= " << decompose (zun, dun) << "\n";

  Mould xom= (rres (dom, rm) - rres (dom, lm)) / 2;
  Mould zom= (xom / mul - xom / mur) / 2;
  cout << "dom\t\t\t= " << dom << "\n";
  cout << "dom 1,2 \t\t\t= "
       << compose (MOULD (C(1), list<nat> (1, 2)), dom) << "\n";
  cout << "dom 2,1 \t\t\t= "
       << compose (MOULD (C(1), list<nat> (2, 1)), dom) << "\n";
  cout << "xom\t\t\t= " << xom << "\n";
  cout << "zom\t\t\t= " << zom << "\n";
  cout << "decompose (zom, dom)\t= " << decompose (zom, dom) << "\n";

  mould<C,nat,V> sol[10];
  for (nat i=1; i<=8; i++) {
    Mould eq= hom_part (compose (M, dom) * mul - xom, i);
    sol[i]= solve_mould (eq, M);
    //cout << i << " >>> " << sol[i] << "\n";
    (void) sol[i][list<nat> (i)];
    cout << "\n";
    list<nat> dummy;
    list<list<nat> > l= up_till_weight (table<bool,nat> (true), i);
    vector<rational> v (rational (0), (nat) 10);
    vector<nat> ww ((nat) 10, (nat) 10);
    vector<list<nat> > wv (dummy, 10);
    for (; !nil (l); l= cdr (l)) {
      C c= sol[i][car (l)];
      rational x= abs (as<rational> (*c));
      nat d= N (car (l));
      if (x > v[d]) {
	v (d)= x;
	ww(d)= weight (car (l));
	wv(d)= car (l);
      }
      else if (x == v[d] && weight (car (l)) < ww[d]) {
	ww(d)= weight (car (l));
	wv(d)= car (l);
      }
      //cout << car (l) << ", " << d << ", " << x << ", " << v[d] << "\n";
    }
    for (nat j=1; j<=i; j++)
      cout << "    " << j << ": " << v[j] << ", " << wv[j] << "\n";
    cout << "\n";
  }
  */

  return 0;
}
