
/******************************************************************************
* MODULE     : polynomial_glue.hpp
* DESCRIPTION: Glue for polynomials
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_POLYNOMIAL_GLUE_HPP
#define __MMX_POLYNOMIAL_GLUE_HPP
#include "dense_polynomial.hpp"
#include "category_meta.hpp"
#include "category_lift_glue.hpp"
#include "tuple.hpp"
#include "integer.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Polynomial polynomial<C>
#define Vector vector<C>

TMPL Polynomial
make_polynomial (const tuple<C>& t) {
  nat i, n= N (t);
  C* coeffs= mmx_new<C> (n);
  for (i=0; i<n; i++)
    coeffs[n-1-i]= t[i];
  return Polynomial (coeffs, n);
}

TMPL C polynomial_access (const Polynomial& v, const integer& i) {
  return v[as_int (i)]; }
TMPL integer polynomial_size (const Polynomial& p) { return N(p); }
TMPL integer polynomial_deg (const Polynomial& p) { return deg (p); }
TMPL Polynomial polynomial_big_add (const tuple<polynomial<C> >& t) {
  return big_add (as_array (t)); }
TMPL Polynomial polynomial_annulator (const Vector& v) {
  return annulator (v); }
TMPL C polynomial_eval (const Polynomial& p, const C& x) {
  return eval (p, x); }
TMPL Vector polynomial_eval_vector (const Polynomial& p, const Vector& v) {
  return eval (p, v); }
TMPL Polynomial polynomial_shift (const Polynomial& p, const C& c) {
  return shift (p, c); }

template<typename Cond, typename C> void
define_polynomial () {
  define_type<Cond,Polynomial > (gen ("Polynomial", type_name<C> ()));
  define<Cond> ("polynomial", make_polynomial<C>);
  define_upgrader<Cond,Polynomial,C> ();
  define_from_iterator<Cond,C> ("polynomial");
  define_downgrader<Cond,iterator<C>,Polynomial >
    (iterate<C>, PENALTY_HOMOMORPHISM);

  define<Cond> (GEN_SIZE, polynomial_size<C>);
  define<Cond> ("deg", polynomial_deg<C>);
  define<Cond> (GEN_ACCESS, polynomial_access<C>);

  lift_ring_ops<Cond,Polynomial,C> ();
  lift_scalar_field_ops<Cond,Polynomial,C> ();
  define<Cond> (GEN_PLUS, polynomial_big_add<C>);
  define_binary<Cond,Polynomial,quo_op> ();
  define_binary<Cond,Polynomial,rem_op> ();  
  lift_factorial_ops<Cond,Polynomial,C> ();
  lift_ordering_ops<Cond,Polynomial,C> ();

  define<Cond> ("annulator", polynomial_annulator<C>);
  define<Cond> ("eval", polynomial_eval<C>);
  define<Cond> ("eval", polynomial_eval_vector<C>);
  define<Cond> ("shift", polynomial_shift<C>);
}

#undef TMPL
#undef Polynomial
#undef Vector
} // namespace mmx
#endif // __MMX_POLYNOMIAL_GLUE_HPP
