
/******************************************************************************
* MODULE     : series_glue.hpp
* DESCRIPTION: Glue for seriess
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_SERIES_GLUE_HPP
#define __MMX_SERIES_GLUE_HPP
#include "linear_series.hpp"
#include "relaxed_series.hpp"
#include "category_meta.hpp"
#include "category_lift_glue.hpp"
#include "tuple.hpp"
#include "integer.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Series series<C>

TMPL Series
make_series (const tuple<C>& t) {
  return Series (iterate (t));
}

TMPL inline C
series_access (const Series& v, const integer& i) {
  return v[as_int (i)];
}

TMPL inline integer series_deg (const Series& v) {
  return deg (v); }

template<typename Hyp, typename C> void
define_series () {
  typedef both<has<C,ring_cat>,Hyp> Cond;
  typedef both<different<C,generic>,Cond> Specialized;

  define_type<Cond,Series > (gen ("Series", type_name<C> ()));
  define<Cond> ("series", make_series<C>);
  define_from_iterator<Cond,C> ("series");
  define_upgrader<Cond,Series,C> ();
  define_downgrader<Cond,iterator<C>,Series >
    (iterate<C>, PENALTY_HOMOMORPHISM);

  define<Cond> (GEN_ACCESS, series_access<C>);

  lift_field_ops<Cond,Series,C> ();
  lift_scalar_ops<Cond,Series,C> ();
  lift_ordering_ops<Cond,Series,C> ();
  define_elementary<Cond,Series> ();

  define_binary<Cond,Series,compose_op> ();
}

#undef TMPL
#undef Series
} // namespace mmx
#endif // __MMX_SERIES_GLUE_HPP
