
/******************************************************************************
* MODULE     : transseries_glue.hpp
* DESCRIPTION: Glue for transseries
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_TRANSSERIES_GLUE_HPP
#define __MMX_TRANSSERIES_GLUE_HPP
#include "transseries.hpp"
#include "linear_series.hpp"
#include "category_meta.hpp"
#include "category_lift_glue.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Transseries transseries<C>

TMPL Transseries transvariable (const generic& name) {
  return Transseries (name, true); }
TMPL int transseries_sign (const Transseries& f) {
  return sign (f); }
TMPL C transseries_dominant_coefficient (const Transseries& f) {
  return dominant_coefficient (f); }
TMPL Transseries transseries_infinite_part (const Transseries& f) {
  return infinite_part (f); }
TMPL Transseries transseries_bounded_part (const Transseries& f) {
  return bounded_part (f); }
TMPL Transseries transseries_constant_part (const Transseries& f) {
  return constant_part (f); }
TMPL Transseries transseries_infinitesimal_part (const Transseries& f) {
  return infinitesimal_part (f); }

template<typename Hyp, typename C> void
define_transseries () {
  typedef both<has<C,ring_cat>,Hyp> Cond;
  typedef both<different<C,generic>,Cond> Specialized;

  define_type<Cond,Transseries > (gen ("Transseries", type_name<C> ()));
  define<Cond> ("transvariable", transvariable<C>);
  define_upgrader<Cond,Transseries,C> ();

  lift_field_ops<Cond,Transseries,C> ();
  //lift_scalar_ops<Cond,Transseries,C> ();
  lift_ordering_ops<Cond,Transseries,C> ();
  //define_elementary<Cond,Transseries> ();
  define_unary<Cond,Transseries,log_op> ();
  define_unary<Cond,Transseries,exp_op> ();
  define_binary<Cond,Transseries,pow_op> ();

  define<Cond> ("sign", transseries_sign<C>);
  define<Cond> ("dominant_coefficient", transseries_dominant_coefficient<C>);
  define<Cond> ("infinite_part", transseries_infinite_part<C>);
  define<Cond> ("bounded_part", transseries_bounded_part<C>);
  define<Cond> ("constant_part", transseries_constant_part<C>);
  define<Cond> ("infinitesimal_part", transseries_infinitesimal_part<C>);
}

#undef TMPL
#undef Transseries
} // namespace mmx
#endif // __MMX_TRANSSERIES_GLUE_HPP
