
/******************************************************************************
* MODULE     : laurent_series.hpp
* DESCRIPTION: Laurent series (both univariate and multivariate)
* COPYRIGHT  : (C) 2007  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_LAURENT_SERIES_HPP
#define __MML_LAURENT_SERIES_HPP
#include "mv_series.hpp"
namespace mmx {
#define TMPL_DEF template<typename C, typename V= naive_multiply>
#define TMPL template<typename C, typename V>
#define Monomial monomial<int,generic>
#define Stair list<Monomial >
#define Series series<C,V>
#define Variable typename Series::variable_type
#define Laurent_series laurent_series<C,V>
#define Laurent_series_rep laurent_series_rep<C,V>
#define Set table<bool,Variable >
TMPL class laurent_series_rep;
TMPL class laurent_series;

/******************************************************************************
* Laurent_series
******************************************************************************/

TMPL_DEF
class laurent_series_rep REP_STRUCT {
public:
  Series   f;
  Monomial m;
  inline laurent_series_rep ():
    f (0), m (1) {}
  inline laurent_series_rep (const Series& f2):
    f (f2), m (1) {}
  inline laurent_series_rep (const Series& f2, const Monomial& m2):
    f (f2), m (m2) {}
  inline laurent_series_rep (const Monomial& m2):
    f (1), m (m2) {}
  inline ~laurent_series_rep () {}
};

TMPL_DEF
class laurent_series {
INDIRECT_PROTO_2 (laurent_series, laurent_series_rep, C, V)
public:
  inline laurent_series ():
    rep (new Laurent_series_rep ()) {}
  template<typename T> inline laurent_series (const T& c):
    rep (new Laurent_series_rep (Series (c))) {}
  inline laurent_series (const Monomial& m):
    rep (new Laurent_series_rep (m)) {}
  inline laurent_series (const Series& f, const Monomial& m):
    rep (new Laurent_series_rep (f, m)) {}
  inline C operator [] (const Monomial& m) const {
    return rep->f [m / rep->m]; }
};
INDIRECT_IMPL_2 (laurent_series, laurent_series_rep,
		 typename C, C, typename V, V)

TMPL inline nat hash (const Laurent_series& f) {
  // FIXME: find something better to return here
  (void) f; return 0; }
TMPL inline bool
operator == (const Laurent_series& f, const Laurent_series& g) {
  Monomial m= gcd (f->m, g->m);
  return f->f * (g->m / m) == g->f * (f->m / m); }
TMPL inline bool
operator != (const Laurent_series& f, const Laurent_series& g) {
  return !(f == g); }
TMPL inline nat soft_hash (const Laurent_series& f) {
  return soft_hash (f->f) ^ soft_hash (f->m); }
TMPL inline bool eq (const Laurent_series& f, const Laurent_series& g) {
  return eq (f->f, g->f) && eq (f->m, g->m); }
TMPL inline bool neq (const Laurent_series& f, const Laurent_series& g) {
  return !eq (f, g); }

TMPL inline generic flatten (const Laurent_series& f) {
  return flatten (f->f) * flatten (f->m); }
PRINT_SUGAR(TMPL,Laurent_series)

/******************************************************************************
* Support related routines
******************************************************************************/

TMPL inline Set variables (const Laurent_series& f) {
  return variables (f->f) | variables (f->m); }
TMPL inline int val (const Laurent_series& f, const Variable& v) {
  return val (f->f, v) * f->m[v]; }
TMPL inline Monomial monomial_val (const Laurent_series f) {
  return monomial_val (f->f) * f->m; }

TMPL Laurent_series project (const Laurent_series& f, const Stair& l) {
  return Laurent_series (project (f->f, stair_mul (1/f->m, l)), f->m); }
TMPL Laurent_series head (const Laurent_series& f, const Stair& l) {
  return Laurent_series (head (f->f, stair_mul (1/f->m, l)), f->m); }
TMPL Laurent_series tail (const Laurent_series& f, const Stair& l) {
  return Laurent_series (tail (f->f, stair_mul (1/f->m, l)), f->m); }
TMPL Stair dominant_monomials (const Laurent_series& f) {
  return stair_mul (f->m, dominant_monomials (f->f)); }
TMPL inline Laurent_series
normalize (const Laurent_series& f, const Monomial& dom_m) {
  return Laurent_series (shift_exp (f->f, f->m / dom_m), dom_m); }

/******************************************************************************
* Basic arithmetic
******************************************************************************/

TMPL Laurent_series
operator + (const Laurent_series& f, const Laurent_series& g) {
  if (f->m == g->m) return Laurent_series (f->f + g->f, f->m);
  else {
    Monomial m= gcd (f->m, g->m);
    return Laurent_series (f->f * (f->m / m) + g->f * (g->m / m), m);
  }
}

TMPL Laurent_series
operator + (const Laurent_series& f, const C& c) {
  return Laurent_series (f->f + Laurent_series (c));
}

TMPL Laurent_series
operator + (const C& c, const Laurent_series& f) {
  return Laurent_series (Laurent_series (c) + f->f);
}

TMPL Laurent_series
operator - (const Laurent_series& f) {
  return Laurent_series (-f->f, f->m);
}

TMPL Laurent_series
operator - (const Laurent_series& f, const Laurent_series& g) {
  if (f->m == g->m) return Laurent_series (f->f - g->f, f->m);
  else {
    Monomial m= gcd (f->m, g->m);
    return Laurent_series (f->f * (f->m / m) - g->f * (g->m / m), m);
  }
}

TMPL Laurent_series
operator - (const Laurent_series& f, const C& c) {
  return Laurent_series (f->f - Laurent_series (c));
}

TMPL Laurent_series
operator - (const C& c, const Laurent_series& f) {
  return Laurent_series (Laurent_series (c) - f->f);
}

TMPL Laurent_series
operator * (const Monomial& m, const Laurent_series& f) {
  return Laurent_series (f->f, m * f->m);
}

TMPL Laurent_series
operator * (const Laurent_series& f, const Monomial& m) {
  return Laurent_series (f->f, f->m * m);
}

TMPL Laurent_series
operator * (const C& c, const Laurent_series& f) {
  return Laurent_series (c * f->f, f->m);
}

TMPL Laurent_series
operator * (const Laurent_series& f, const C& c) {
  return Laurent_series (c * f->f, f->m);
}

TMPL Laurent_series
operator * (const Laurent_series& f, const Laurent_series& g) {
  return Laurent_series (f->f * g->f, f->m * g->m);
}

TMPL Laurent_series
operator / (const Laurent_series& f, const Monomial& m) {
  return Laurent_series (f->f, f->m / m);
}

TMPL Laurent_series
operator / (const Monomial& m, const Laurent_series& f) {
  Monomial v= monomial_val (f);
  if (v != Monomial (1)) return (m/v) / (f/v);
  return Laurent_series (C(1) / f->f, m / f->m);
}

TMPL Laurent_series
operator / (const Laurent_series& f, const C& c) {
  return Laurent_series (f->f / c, f->m);
}

TMPL Laurent_series
operator / (const C& c, const Laurent_series& f) {
  Monomial v= monomial_val (f);
  if (v != Monomial (1)) return (c / (f/v)) / v;
  return Laurent_series (c / f->f, Monomial (1) / f->m);
}

TMPL Laurent_series
operator / (const Laurent_series& f, const Laurent_series& g) {
  Monomial v= monomial_val (g);
  if (v != Monomial (1)) return (f/v) / (g/v);
  return Laurent_series (f->f / g->f, f->m);
}

ARITH_INT_SUGAR(TMPL,Laurent_series,C);

#undef TMPL_DEF
#undef TMPL
#undef Monomial
#undef Stair
#undef Series
#undef Variable
#undef Laurent_series
#undef Laurent_series_rep
#undef Set
} // namespace mmx
#endif // __MML_LAURENT_SERIES_HPP
