
/******************************************************************************
* MODULE     : mv_series.hpp
* DESCRIPTION: Multivariate power series
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_MV_SERIES_HPP
#define __MMX_MV_SERIES_HPP
#include "series.hpp"
#include "table.hpp"
#include "list.hpp"
namespace mmx {
#define TMPL_DEF template<typename C, typename V>
#define TMPL template<typename C, typename V>
#define Variant multivariate<V>
#define Monomial monomial<int,generic>
#define Stair list<Monomial >
#define Series series<C,Variant >
#define Series_rep series_rep<C,Variant >
#define Variable generic
#define Series_0 C
#define Series_1 typename Series::series_1_type
#define Series_n typename Series::series_n_type
#define Set table<bool,Variable >
TMPL class series;

/******************************************************************************
* Multivariate series traits
******************************************************************************/

template<typename V>
class multivariate {
public:
  typedef naive_multiply series_1_multiply;
  typedef naive_multiply series_n_multiply;
};

/******************************************************************************
* Multivariate series
******************************************************************************/

template<typename C, typename V, typename X>
class series_cache_rep {
public:
  Set variables;
  X   expansion;
  inline series_cache_rep (const Set& v, const X& x):
    variables (v), expansion (x) {}
};

TMPL_DEF
class Series_rep REP_STRUCT {
private:
  union {
    series_cache_rep<C,V,C>* cache_0;
    series_cache_rep<C,V,Series_1>* cache_1;
    series_cache_rep<C,V,table<Series_n,Variable> >* cache_n;
  };

public:
  inline series_rep (): cache_0 (NULL) {}
  inline virtual ~series_rep () {
    if (cache_0 != NULL) {
      if (N (cache_0->variables) == 0) delete cache_0;
      else if (N (cache_0->variables) == 1) delete cache_1;
      else delete cache_n;
    }
  }
  inline void init_cache () {
    if (cache_0 == NULL) {
      Set vars= Variables ();
      if (N (vars) == 0)
	cache_0= new series_cache_rep<C,V,C> (vars, Expand_0 ());
      else if (N (vars) == 1)
	cache_1= new series_cache_rep<C,V,Series_1> (vars, Expand_1 ());
      else
	cache_n= new series_cache_rep<C,V,table<Series_n,Variable> >
	  (vars, table<Series_n,Variable> (0));
    }
  }
  inline Series me () const;
  virtual generic expression () const = 0;

public:
  virtual Set Variables () const = 0;
  virtual Series_0 Expand_0 () const = 0;
  virtual Series_1 Expand_1 () const = 0;
  virtual Series_n Expand_n (const Variable& v) const = 0;

  inline Set cached_variables () {
    init_cache ();
    return cache_0->variables;
  }
  inline Series_0 cached_expand_0 () {
    init_cache ();
    return cache_0->expansion;
  }
  inline Series_1 cached_expand_1 () {
    init_cache ();
    if (N (cache_0->variables) == 0) return Series_1 (cache_0->expansion);
    else return cache_1->expansion;
  }
  inline Series_n cached_expand_n (const Variable& v) {
    init_cache ();
    if (cache_0->variables->contains (v)) {
      if (N (cache_0->variables) > 1) {
	if (!cache_n->expansion->contains (v)) {
	  Series_n val= Expand_n (v);
	  cache_n->expansion [v]= val;
	}
	return read (cache_n->expansion, v);
      }
      else return Series_n (cache_1->expansion);
    }
    else return Series_n (this->me ());
  }

  friend class Series;
};

TMPL_DEF
class Series {
INDIRECT_PROTO_2 (series, series_rep, C, Variant)
public:
  typedef series<C, typename Variant::series_1_multiply>
    series_1_type;
  typedef series<Series, typename Variant::series_n_multiply>
    series_n_type;

public:
  series ();
  template<typename T> series (const T& c);
  series (const Variable& v);
  series (const Monomial& m);
  C operator [] (const Monomial& m) const;

  inline Set cached_variables () const {
    return rep->cached_variables (); }
  inline Series_0 cached_expand_0 () const {
    return rep->cached_expand_0 (); }
  inline Series_1 cached_expand_1 () const {
    return rep->cached_expand_1 (); }
  inline Series_n cached_expand_n (const Variable& v) const {
    return rep->cached_expand_n (v); }
};
INDIRECT_IMPL_2 (series, series_rep, typename C, C, typename V, Variant)

TMPL inline Series Series_rep::me () const { return Series (this, true); }

/******************************************************************************
* User interface
******************************************************************************/

template<typename Op, typename C, typename V> nat
unary_hash (const Series& s) {
  // TODO: implement decent hash function
  (void) s;
  return 0;
}

TMPL inline nat
hash (const Series& s) {
  return unary_hash<hash_op> (s);
}

TMPL Set variables (const Series& f) {
  return f.cached_variables (); }
TMPL Series_0 expand_0 (const Series& f) {
  return f.cached_expand_0 (); }
TMPL Series_1 expand_1 (const Series& f) {
  return f.cached_expand_1 (); }
TMPL Series_n expand_n (const Series& f, const Variable& v) {
  return f.cached_expand_n (v); }

TMPL generic
expand (const Series& f, int n) {
  if (n == 0) return 0;
  Set s= variables (f);
  if (N(s) == 0) return flatten (expand_0 (f));
  Variable v= *entries (s);
  generic sum= 0;
  if (N(s) == 1) {
    Series_1 g= expand_1 (f);
    for (int i=0; i<n; i++) {
      generic coeff= flatten (g[i]);
      sum= sum + coeff * pow (v, generic (i));
    }
  }
  else {
    Series_n g= expand_n (f, v);
    for (int i=0; i<n; i++) {
      generic coeff= expand (g[i], n-i);
      sum= sum + coeff * pow (v, generic (i));
    }
  }
  return sum;
}

TMPL C
Series::operator [] (const Monomial& m) const {
  Set s= variables (*this);
  if (N(s) == 0) {
    if (m == Monomial (1)) return expand_0 (*this);
    else return 0;
  }
  else {
    Variable v= *entries (s);
    Monomial n= m / Monomial (v, m[v]);
    if (N(s) == 1) {
      if (n == Monomial (1)) return expand_1 (*this) [m[v]];
      else return 0;
    }
    else return expand_n (*this, v) [m[v]] [n];
  }
}

TMPL generic
flatten (const Series& f) {
  if (get_series_formula_output ()) return f->expression ();
  return expand (f, 5);
}

PRINT_SUGAR(TMPL,Series)

/******************************************************************************
* Explicit series
******************************************************************************/

TMPL
class constant_series_rep: public Series_rep {
  C c;
public:
  constant_series_rep (const C& c2): c (c2) {}
  generic expression () const { return flatten (c); }
  Set Variables () const { return Set (false); }
  Series_0 Expand_0 () const { return c; }
  Series_1 Expand_1 () const { return c; }
  Series_n Expand_n (const Variable& v) const {
    (void) v;
    return Series_n (this->me ()); }
};

TMPL
Series::series () {
  rep= new constant_series_rep<C,V> (C (0));
}

TMPL template<typename T>
Series::series (const T& c) {
  rep= new constant_series_rep<C,V> (C (c));
}

TMPL
class variable_series_rep: public Series_rep {
  Variable v;
public:
  variable_series_rep (const Variable& v2): v (v2) {}
  generic expression () const { return flatten (v); }
  Set Variables () const { Set s (false); s[v]= true; return s; }
  Series_0 Expand_0 () const { return 0; }
  Series_1 Expand_1 () const { return Series_1 (1, 1); }
  Series_n Expand_n (const Variable& w) const {
    if (v == w) return Series_n (Series (1), 1);
    else return Series_n (this->me ()); }
};

TMPL
Series::series (const Variable& v) {
  rep= new variable_series_rep<C,V> (v);
}

TMPL
class monomial_series_rep: public Series_rep {
  Monomial m;
public:
  monomial_series_rep (const Monomial& m2): m (m2) {}
  generic expression () const { return flatten (m); }
  Set Variables () const {
    Set s (false);
    for (iterator<Variable > it= entries (*m); busy (it); ++it)
      if (m[*it] != 0) s[*it]= true;
    return s; }
  Series_0 Expand_0 () const {
    return Series_0 (m == Monomial (1)? 1: 0); }
  Series_1 Expand_1 () const {
    Variable v= *entries (*m);
    return Series_1 (1, m[v]); }
  Series_n Expand_n (const Variable& v) const {
    if (m[v] != 0) return Series_n (Series (m / Monomial (v, m[v])), m[v]);
    else return Series_n (this->me ()); }
};

TMPL
Series::series (const Monomial& m) {
  rep= new monomial_series_rep<C,V> (m);
}

/******************************************************************************
* Abstract functions on multivariate series
******************************************************************************/

template<typename Op,typename C,typename V>
class unary_mv_series_rep : public Series_rep {
protected:
  Series f;
public:
  unary_mv_series_rep (const Series& f2): f(f2) {}
  generic expression () const {
    return Op::op (flatten (f)); }
  Set Variables () const {
    return variables (f); }
  Series_0 Expand_0 () const {
    return Op::op (expand_0 (f)); }
  Series_1 Expand_1 () const {
    return Op::op (expand_1 (f)); }
  Series_n Expand_n (const Variable& v) const {
    return Op::op (expand_n (f, v)); }
};

template<typename Op,typename C,typename V> Series
unary_mv_series (const Series& f) {
  return (Series_rep*) new unary_mv_series_rep<Op,C,V> (f);
}

template<typename Op,typename C,typename V>
class binary_mv_series_rep : public Series_rep {
protected:
  Series f, g;
public:
  binary_mv_series_rep (const Series& f2, const Series& g2):
    f(f2), g (g2) {}
  generic expression () const {
    return Op::op (flatten (f), flatten (g)); }
  Set Variables () const {
    return variables (f) | variables (g); }
  Series_0 Expand_0 () const {
    return Op::op (expand_0 (f), expand_0 (g)); }
  Series_1 Expand_1 () const {
    return Op::op (expand_1 (f), expand_1 (g)); }
  Series_n Expand_n (const Variable& v) const {
    return Op::op (expand_n (f, v), expand_n (g, v)); }
};

template<typename Op,typename C,typename V> Series
binary_mv_series (const Series& f, const Series& g) {
  return (Series_rep*) new binary_mv_series_rep<Op,C,V> (f, g);
}

template<typename Op,typename C,typename V,typename X>
class extrary_mv_series_rep : public Series_rep {
protected:
  Series f;
  X x;
public:
  extrary_mv_series_rep (const Series& f2, const X& x2):
    f(f2), x (x2) {}
  generic expression () const {
    return Op::op (flatten (f), flatten (x)); }
  Set Variables () const {
    return variables (f); }
  Series_0 Expand_0 () const {
    return Op::op (expand_0 (f), x); }
  Series_1 Expand_1 () const {
    return Op::op (expand_1 (f), x); }
  Series_n Expand_n (const Variable& v) const {
    return Op::op (expand_n (f, v), Series (x)); }
};

template<typename Op,typename C,typename V,typename X> Series
extrary_mv_series (const Series& f, const X& x) {
  return (Series_rep*) new extrary_mv_series_rep<Op,C,V,X> (f, x);
}

/******************************************************************************
* Concrete functions on multivariate series
******************************************************************************/

TMPL Series inline operator - (const Series& f) {
  return unary_mv_series<neg_op> (f); }
TMPL Series inline operator + (const Series& f, const Series& g) {
  return binary_mv_series<add_op> (f, g); }
TMPL Series inline operator - (const Series& f, const Series& g) {
  return binary_mv_series<sub_op> (f, g); }
TMPL Series inline operator * (const Series& f, const Series& g) {
  return binary_mv_series<mul_op> (f, g); }
TMPL Series inline operator / (const Series& f, const Series& g) {
  return binary_mv_series<div_op> (f, g); }
TMPL Series inline operator * (const Series& f, const C& sc) {
  return extrary_mv_series<rmul_op> (f, sc); }
TMPL Series inline operator * (const C& sc, const Series& f) {
  return extrary_mv_series<lmul_op> (f, sc); }
TMPL Series inline operator / (const Series& f, const C& sc) {
  return extrary_mv_series<rdiv_op> (f, sc); }

TMPL Series inline sqrt (const Series& f) {
  return unary_mv_series<sqrt_op> (f); }
TMPL Series inline exp (const Series& f) {
  return unary_mv_series<exp_op> (f); }
TMPL Series inline log (const Series& f) {
  return unary_mv_series<log_op> (f); }
TMPL Series inline cos (const Series& f) {
  return unary_mv_series<cos_op> (f); }
TMPL Series inline sin (const Series& f) {
  return unary_mv_series<sin_op> (f); }
TMPL Series inline tan (const Series& f) {
  return unary_mv_series<tan_op> (f); }
TMPL Series inline acos (const Series& f) {
  return unary_mv_series<acos_op> (f); }
TMPL Series inline asin (const Series& f) {
  return unary_mv_series<asin_op> (f); }
TMPL Series inline atan (const Series& f) {
  return unary_mv_series<atan_op> (f); }

INV_TRIGO_SUGAR(TMPL,Series)
HYPER_SUGAR(TMPL,Series)
INV_HYPER_SUGAR(TMPL,Series)
ARG_HYPER_SUGAR(TMPL,Series)

/******************************************************************************
* Multiplication with monomials
******************************************************************************/

TMPL Series shift_exp (const Series& f, const Monomial& m);

struct shift_exp_op {
  template<typename C,typename S> static inline C
  op (const C& x, const S& y) { return shift_exp (x, y); }
};

TMPL
class mv_shift_series_rep: public Series_rep {
protected:
  Series f;
  Monomial m;
public:
  mv_shift_series_rep (const Series& f2, const Monomial& m2):
    f(f2), m (m2) {}
  generic expression () const {
    return flatten (f) * flatten (m); }
  Set Variables () const {
    Set s= variables (f);
    for (iterator<Variable > it= entries (*m); busy (it); ++it)
      if (m[*it] != 0) s[*it]= true;
    return s; }
  Series_0 Expand_0 () const {
    return f[Monomial(1) / m]; }
  Series_1 Expand_1 () const {
    Set s= variables (f);
    if (N (s) == 0) {
      if (m <<= Monomial (1)) return expand_0 (f) * expand_1 (Series (m));
      else return (0);
    }
    Variable v= *entries (s);
    return shift_exp (expand_1 (f), m[v]); }
  Series_n Expand_n (const Variable& v) const {
    Monomial vi (v, m[v]);
    Series_n g1= expand_n (f, v);
    Series_n g2= extrary_series<shift_exp_op> (g1, m / vi);
    return shift_exp (g2, m[v]); }
};

TMPL Series
shift_exp (const Series& f, const Monomial& m) {
  return (Series_rep*) new mv_shift_series_rep<C,V> (f, m);
}

TMPL Series
operator * (const Series& f, const Monomial& m) {
  if (!(m <<= Monomial (1))) cout << "Times " << f << ", " << m << "\n";
  ASSERT (m <<= Monomial(1), "bounded monomial expected");
  return shift_exp (f, m);
}

TMPL Series
operator * (const Monomial& m, const Series& f) {
  if (!(m <<= Monomial (1))) cout << "Times " << f << ", " << m << "\n";
  ASSERT (m <<= Monomial(1), "bounded monomial expected");
  return shift_exp (f, m);
}

/******************************************************************************
* Differentiation and integration
******************************************************************************/

TMPL
class mv_derive_series_rep : public Series_rep {
  Series f;
  Variable v;
public:
  mv_derive_series_rep (const Series& f2, const Variable& v2):
    f (f2), v (v2) {}
  generic expression () const { return derive (flatten (f), flatten (v)); }
  Set Variables () const { return variables (f); }
  Series_0 Expand_0 () const { ERROR ("invalid situation (Expand_0)"); }
  Series_1 Expand_1 () const { return derive (expand_1 (f)); }
  Series_n Expand_n (const Variable& w) const {
    if (v == w) return derive (expand_n (f, v));
    else return derive_coeffs (expand_n (f, w), v);
  }
};

TMPL Series
derive (const Series& f, const Variable& v) {
  return (Series_rep*) new mv_derive_series_rep<C,V> (f, v);
}

TMPL Series
xderive (const Series& f, const Variable& v) {
  return Series (v) * derive (f);
}

TMPL
class mv_integrate_series_rep : public Series_rep {
  Series f;
  Variable v;
public:
  mv_integrate_series_rep (const Series& f2, const Variable& v2):
    f (f2), v (v2) {}
  generic expression () const {
    return integrate (flatten (f), flatten (v)); }
  Set Variables () const { Set s(false); s[v]= true; return variables (f)|s; }
  Series_0 Expand_0 () const { return 0; }
  Series_1 Expand_1 () const { return integrate (expand_1 (f)); }
  Series_n Expand_n (const Variable& w) const {
    if (v == w) return integrate (expand_n (f, v));
    else return integrate_coeffs (expand_n (f, w), v);
  }
};

TMPL Series
integrate (const Series& f, const Variable& v) {
  return (Series_rep*) new mv_integrate_series_rep<C,V> (f, v);
}

/******************************************************************************
* Equality
******************************************************************************/

TMPL bool
zero_coefficients (const Series& f, nat order= 8) {
  Set s= variables (f);
  if (N(s) == 0) return expand_0 (f) == 0;
  else if (N(s) == 1) {
    Series_1 g= expand_1 (f);
    for (nat i=0; i<order; i++)
      if (g[i] != 0) return false;
    return true;
  }
  else {
    Variable v= *entries (s);
    Series_n g= expand_n (f, v);
    for (nat i=0; i<order; i++)
      if (!zero_coefficients (g[i], order-i))
	return false;
    return true;
  }
}

TMPL bool operator == (const Series& f, const Series& g) {
  return zero_coefficients (f-g); }
TMPL bool operator != (const Series& f, const Series& g) {
  return !(f == g); }
EQUAL_INT_SUGAR(TMPL,Series,Series);

/******************************************************************************
* Restrictions of the support of a series
******************************************************************************/

TMPL Series
project (const Series& f, const Stair& l) {
  if (nil (l)) return 0;
  else return f[car (l)] * Series (car (l)) + project (f, cdr (l));
}

TMPL Series
head (const Series& f, const Stair& l) {
  if (nil (l)) return f;
  if (car (l) >>= Monomial (1)) return 0;
  Variable v= *entries (*car (l));
  int n= max (max_power_in (l, v), 0);
  Series sum= 0;
  Series t= f;
  Series_n xf= expand_n (f, v);
  for (int i=0; i<n; i++) {
    Monomial vi (v, i);
    sum += head (xf[i], project_up_to (l, v, i)) * vi;
    t -= xf[i] * vi;
  }
  return sum + head (t, project_up_to (l, v, n));
}

TMPL Series
tail (const Series& f, const Stair& l) {
  if (nil (l)) return 0;
  if (car (l) >>= Monomial (1)) return f;
  Variable v= *entries (*car (l));
  int n= max (max_power_in (l, v), 0);
  Series sum= 0;
  Series t= f;
  Series_n xf= expand_n (f, v);
  for (int i=0; i<n; i++) {
    Monomial vi (v, i);
    sum += tail (xf[i], project_up_to (l, v, i)) * vi;
    t -= xf[i] * vi;
  }
  return sum + tail (t, project_up_to (l, v, n));
}

/******************************************************************************
* Finding the dominant monomials of a series
******************************************************************************/

TMPL Stair
dominant_monomials (const Series& f) {
  if (f == 0) return Stair ();
  Set s= variables (f);
  if (N(s) == 0) return Stair (Monomial (1));
  Variable v= *entries (s);
  Stair l;
  Series r= f;
  for (int i=0; true; i++) {
    Series_n xr= expand_n (r, v);
    Stair li= dominant_monomials (xr[i]);
    if (!nil (li)) {
      l= stair_append (stair_mul (Monomial (v, i), li), l);
      r= head (f, l);
      if (r == 0) return l;
    }
  }
}

TMPL int
val (const Series& f, const Variable& v) {
  return val (expand_n (f, v));
}

TMPL Monomial
monomial_val (const Series& f) {
  Set s= variables (f);
  if (N(s) == 0) {
    ASSERT (expand_0 (f) != 0, "non zero series expected");
    return Monomial (1);
  }
  Monomial m (1);
  for (iterator<Variable > it= entries (s); busy (it); ++it)
    m= m * Monomial (*it, val (f, *it));
  return m;
}

#undef TMPL_DEF
#undef TMPL
#undef Variant
#undef Monomial
#undef Stair
#undef Series
#undef Series_rep
#undef Variable
#undef Series_0
#undef Series_1
#undef Series_n
#undef Set
} // namespace mmx
#endif // __MMX_MV_SERIES_HPP
