
/******************************************************************************
* MODULE     : transseries.hpp
* DESCRIPTION: Grid-based transseries
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_TRANSSERIES_HPP
#define __MML_TRANSSERIES_HPP
#include "laurent_series.hpp"
#include "list.hpp"
#include "wrap.hpp"
namespace mmx {
#define TMPL_DEF template<typename C, typename V= naive_multiply>
#define TMPL template<typename C, typename V>
#define Series laurent_series<C,multivariate<V> >
#define Monomial monomial<int,generic>
#define Variable generic
#define Stair list<Monomial >
#define Transbasis transbasis<C,V>
#define Transbasis_rep transbasis_rep<C,V>
#define Transmonomial transmonomial<C,V>
#define Transmonomial_rep transmonomial_rep<C,V>
#define Transseries transseries<C,V>
#define Transseries_rep transseries_rep<C,V>
#define Supermonomial monomial<C,generic>
TMPL class transbasis_rep;
TMPL class transbasis;
TMPL class transmonomial_rep;
TMPL class transmonomial;
TMPL class transseries_rep;
TMPL class transseries;

/******************************************************************************
* Transbases
******************************************************************************/

TMPL_DEF
class transbasis_rep REP_STRUCT {
public:
  list<Variable > b_coord;
  table<Supermonomial,Variable > z_coord;
  table<Series,Variable > log_table;
  inline transbasis_rep ();
  inline ~transbasis_rep ();
  Monomial encode (const Transmonomial& m) const;
  Transmonomial decode (const Monomial& m) const;
};

TMPL_DEF
class transbasis {
INDIRECT_PROTO_2 (transbasis, transbasis_rep, C, V)
public:
  inline transbasis (): rep (new Transbasis_rep ()) {}
};
INDIRECT_IMPL_2 (transbasis, transbasis_rep, typename C, C, typename V, V)

TMPL
class global_transbasis {
public:
  static Transbasis B;
};

/******************************************************************************
* Transmonomials
******************************************************************************/

TMPL_DEF
class transmonomial_rep REP_STRUCT {
public:
  Supermonomial m;
  Transbasis B;
  inline transmonomial_rep (const Supermonomial& m2):
    m (m2), B (global_transbasis<C,V>::B) {}
  inline ~transmonomial_rep () {}
};

TMPL_DEF
class transmonomial {
INDIRECT_PROTO_2 (transmonomial, transmonomial_rep, C, V)
public:
  inline Supermonomial operator * () const { return rep->m; }
  inline transmonomial ():
    rep (new Transmonomial_rep (Supermonomial (1))) {}
  template<typename T> inline transmonomial (const T& c):
    rep (new Transmonomial_rep (Supermonomial (c))) {}
};
INDIRECT_IMPL_2 (transmonomial, transmonomial_rep, typename C, C, typename V, V)

WRAP_INDIRECT_IMPL(TMPL inline,Transmonomial)
TMPL generic flatten (const Transmonomial& m) { return flatten (*m); }
PRINT_SUGAR(TMPL,Transmonomial)

/******************************************************************************
* Transseries
******************************************************************************/

TMPL_DEF
class transseries_rep REP_STRUCT {
public:
  Series f;
  Transbasis B;
  inline transseries_rep (const Series& f2):
    f (f2), B (global_transbasis<C,V>::B) {}
  inline ~transseries_rep () {}
};

TMPL_DEF
class transseries {
INDIRECT_PROTO_2 (transseries, transseries_rep, C, V)
public:
  inline Series operator * () const { return rep->f; }
  inline transseries ():
    rep (new Transseries_rep ()) {}
  template<typename T> inline transseries (const T& c):
    rep (new Transseries_rep (Series (c))) {}
  inline transseries (const Series& f):
    rep (new Transseries_rep (f)) {}
  transseries (const Variable& x, bool large);
};
INDIRECT_IMPL_2 (transseries, transseries_rep, typename C, C, typename V, V)

WRAP_INDIRECT_IMPL(TMPL inline,Transseries)
EQUAL_INT_SUGAR(TMPL,Transseries,Transseries)

/******************************************************************************
* Routines on transbases
******************************************************************************/

TMPL inline bool operator << (const Transmonomial& m1, const int& i);
TMPL Transmonomial operator / (const int& c, const Transmonomial& m1);

TMPL inline Transbasis_rep::transbasis_rep () {}
TMPL inline Transbasis_rep::~transbasis_rep () {}
TMPL Transbasis global_transbasis<C,V>::B= Transbasis ();

TMPL Transmonomial
Transbasis_rep::decode (const Monomial& m) const {
  return Transmonomial (compose (z_coord, m));
}

TMPL Monomial
Transbasis_rep::encode (const Transmonomial& m) const {
  if (m << 1) {
    Transbasis_rep* B= const_cast<Transbasis_rep*> (this);
    Variable v= as_variable (m->m);
    B->z_coord [v]= m->m;
    return Monomial (v, 1);
  }
  else if (m == 1) return 1;
  else return 1 / encode (1/m);
}

/******************************************************************************
* Routines on transmonomials
******************************************************************************/

TMPL Transmonomial
operator * (const Transmonomial& m1, const Transmonomial& m2) {
  return Transmonomial (m1->m * m2->m); }
TMPL Transmonomial
operator / (const Transmonomial& m1, const Transmonomial& m2) {
  return Transmonomial (m1->m / m2->m); }
TMPL Transmonomial
operator / (const int& c, const Transmonomial& m1) {
  return Transmonomial (c / m1->m); }
TMPL Transmonomial
pow (const Transmonomial& m1, const C& c) {
  return Transmonomial (pow (m1->m, c)); }

TMPL int
magnitude (const Transmonomial& m) {
  // cout << "Infinitesimal? " << enc_m << "\n";
  list<Variable > l= m->B->b_coord;
  while (!nil (l)) {
    C p= m->m[car (l)];
    // cout << "  Exponent " << car (l) << " -> " << p << "\n";
    if (p > 0) return 1;
    if (p < 0) return -1;
    l= cdr (l);
  }
  ASSERT (m == Transmonomial (1), "transmonomial not in scale");
  return 0;
}

TMPL inline bool
operator << (const Transmonomial& m1, const Transmonomial& m2) {
  return magnitude (m1/m2) < 0; }
TMPL inline bool
operator <<= (const Transmonomial& m1, const Transmonomial& m2) {
  return magnitude (m1/m2) <= 0; }
TMPL inline bool
operator >> (const Transmonomial& m1, const Transmonomial& m2) {
  return magnitude (m1/m2) > 0; }
TMPL inline bool
operator >>= (const Transmonomial& m1, const Transmonomial& m2) {
  return magnitude (m1/m2) >= 0; }

TMPL inline bool operator == (const Transmonomial& m1, const int& i) {
  return m1 == Transmonomial (i); }
TMPL inline bool operator != (const Transmonomial& m1, const int& i) {
  return m1 == Transmonomial (i); }
TMPL inline bool operator << (const Transmonomial& m1, const int& i) {
  return m1 << Transmonomial (i); }
TMPL inline bool operator <<= (const Transmonomial& m1, const int& i) {
  return m1 <<= Transmonomial (i); }
TMPL inline bool operator >> (const Transmonomial& m1, const int& i) {
  return m1 >> Transmonomial (i); }
TMPL inline bool operator >>= (const Transmonomial& m1, const int& i) {
  return m1 >>= Transmonomial (i); }

/******************************************************************************
* Constructor
******************************************************************************/

TMPL Series
as_indeterminate (const Variable& x, bool large) {
  if (large) return Series (Monomial (1/x, -1));
  else return Series (Monomial (x, 1));
}

template<typename C> bool
contains (const list<C>& l, const C& x) {
  if (nil (l)) return false;
  else if (car (l) == x) return true;
  else return contains (cdr (l), x);
}

TMPL
Transseries::transseries (const Variable& x, bool large):
  rep (new Transseries_rep (as_indeterminate<C,V> (x, large)))
{
  Transbasis_rep* B= inside (rep->B);
  Variable v= (large? x: 1/x);
  if (!contains (B->b_coord, v)) {
    B->b_coord= cons (v, B->b_coord);
    B->z_coord [1/v]= Supermonomial (v, -1);
    //cout << "B= " << B->b_coord << "\n";
  }
}

/******************************************************************************
* Dominant stair
******************************************************************************/

TMPL Monomial
extract_largest (const Transbasis& B, Stair& l) {
  Monomial m= car (l);
  l= cdr (l);
  if (nil (l)) return m;
  else {
    Monomial n= extract_largest (B, l);
    if (B->decode (m / n) << 1) {
      l= cons (m, l);
      return n;
    }
    else {
      l= cons (n, l);
      return m;
    }
  }
}

TMPL Stair
extract_dominant (const Transbasis& B, Stair& l) {
  Stair r;
  while (!nil (l)) {
    Monomial m= extract_largest (B, l);
    if ((!nil (r)) && (B->decode (m / car (r)) << 1)) {
      l= cons (m, l);
      return r;
    }
    r= cons (m, r);
  }
  return r;
}

TMPL Stair
dominant_stair (const Transseries& f) {
  Stair l= dominant_monomials (f->f);
  return extract_dominant (f->B, l);
}

/******************************************************************************
* Regularization
******************************************************************************/

TMPL Transseries
regularize (const Transseries& ts) {
  // cout << "Regularize " << ts << "\n";
  Transbasis_rep* B= inside (ts->B);
  Series f= ts->f;
  Stair l= dominant_monomials (f);
  if (nil (l)) return 0;
  if (nil (cdr (l))) return Transseries (normalize (f, car (l)));
  Monomial m= extract_largest (ts->B, l);
  Series r= tail (f, Stair (m));
  Series s= f - r;
  r= normalize (r, m);
  while (!nil (l)) {
    Series t= tail (f, Stair (car (l)));
    s= s - t;
    t= normalize (t, car (l));
    Monomial z= car (l) / m;
    if (B->decode (z) == 1) {
      r= r + t * (m / car (l));
      // FIXME: eliminate superfluous variable
    }
    else {
      Variable v= as_variable (z);
      B->z_coord [v]= B->decode (z) -> m;
      r= r + t * (m * Monomial (v, 1) / car (l));
    }
    l= cdr (l);
  }
  return regularize (Transseries (r));
}

/******************************************************************************
* Basic arithmetic
******************************************************************************/

TMPL Transseries
operator + (const Transseries& f, const Transseries& g) {
  return Transseries (f->f + g->f);
}

TMPL Transseries
operator + (const Transseries& f, const C& c) {
  return Transseries (f->f + Transseries (c));
}

TMPL Transseries
operator + (const C& c, const Transseries& f) {
  return Transseries (Transseries (c) + f->f);
}

TMPL Transseries
operator - (const Transseries& f) {
  return Transseries (-f->f);
}

TMPL Transseries
operator - (const Transseries& f, const Transseries& g) {
  return Transseries (f->f - g->f);
}

TMPL Transseries
operator - (const Transseries& f, const C& c) {
  return Transseries (f->f - Transseries (c));
}

TMPL Transseries
operator - (const C& c, const Transseries& f) {
  return Transseries (Transseries (c) - f->f);
}

TMPL Transseries
operator * (const C& c, const Transseries& f) {
  return Transseries (c * f->f);
}

TMPL Transseries
operator * (const Transseries& f, const C& c) {
  return Transseries (c * f->f);
}

TMPL Transseries
operator * (const Transseries& f, const Transseries& g) {
  return Transseries (f->f * g->f);
}

TMPL Transseries
operator / (const C& c, const Transseries& f) {
  return Transseries (c / regularize (f)->f);
}

TMPL Transseries
operator / (const Transseries& f, const C& c) {
  return Transseries (f->f / c);
}

TMPL Transseries
operator / (const Transseries& f, const Transseries& g) {
  return Transseries (f->f / regularize (g)->f);
}

ARITH_INT_SUGAR(TMPL,Transseries,C)

/******************************************************************************
* Ordering
******************************************************************************/

TMPL C
dominant_coefficient (const Transseries& f) {
  if (f->f->f == 0) return 0;
  Transseries g= regularize (f);
  return g->f->f [Monomial (1)];
}

TMPL int
sign (const Transseries& f) {
  if (f->f->f == 0) return 0;
  Transseries g= regularize (f);
  return sign (g->f->f [Monomial (1)]);
}

TMPL Transmonomial
dominant_monomial (const Transseries& f) {
  ASSERT (f->f->f != 0, "zero transseries has no dominant monomial");
  Transseries g= regularize (f);
  return f->B->decode (g->f->m);
}

COMPARE_SUGAR(TMPL,Transseries)
COMPARE_INT_SUGAR(TMPL,Transseries,Transseries)

TMPL inline bool asymptotic (const Transseries& f1, const Transseries& f2) {
  if (f2 == 0 || f2 == 0) return f1 == 0 && f2 == 0;
  return dominant_monomial (f1) == dominant_monomial (f2); }
TMPL inline bool operator << (const Transseries& f1, const Transseries& f2) {
  if (f1 == 0 || f2 == 0) return f1 == 0 && f2 != 0;
  return dominant_monomial (f1) << dominant_monomial (f2); }
TMPL inline bool operator <<= (const Transseries& f1, const Transseries& f2) {
  if (f1 == 0 || f2 == 0) return f1 == 0 || f2 != 0;
  return dominant_monomial (f1) <<= dominant_monomial (f2); }
TMPL inline bool operator >> (const Transseries& f1, const Transseries& f2) {
  if (f1 == 0 || f2 == 0) return f1 != 0 && f2 == 0;
  return dominant_monomial (f1) >> dominant_monomial (f2); }
TMPL inline bool operator >>= (const Transseries& f1, const Transseries& f2) {
  if (f1 == 0 || f2 == 0) return f1 != 0 || f2 == 0;
  return dominant_monomial (f1) >>= dominant_monomial (f2); }

/******************************************************************************
* Expansion
******************************************************************************/

TMPL C
sum_coefficients (const Transseries& f, const Stair& l) {
  if (nil (l)) return 0;
  else return f->f[car (l)] + sum_coefficients (f, cdr (l));
}

TMPL list<generic>
expand (const Transseries& f, nat order) {
  if (order == 0 || f == Transseries (0)) return list<generic> ();
  const Stair l= dominant_stair (f);
  C c= sum_coefficients (f, l);
  Transmonomial m= f->B->decode (car (l));
  Transseries nf (f->f - project (f->f, l));
  if (order == 1) return list<generic> (flatten (1), gen ("O", flatten (m)));
  if (c == 0) return expand (nf, order);
  return cons (flatten (c), cons (flatten (m), expand (nf, order-1)));
}

static generic
sum_in_order (const list<generic>& l) {
  if (nil (l)) return 0;
  else return flatten (car (l)) * flatten (car (cdr (l))) +
	      sum_in_order (cdr (cdr (l)));
}

TMPL generic
flatten (const Transseries& f) {
  nat number= 0;
  while (current_ev->contains ("simple_evaluator?")) {
    number++;
    restore_evaluator ();
  }
  list<generic> l= expand (f, 5);
  while (number > 0) {
    select_output_evaluator ();
    number--;
  }
  return sum_in_order (l);
}

//TMPL generic flatten (const Transseries& t) { return flatten (*t); }
PRINT_SUGAR(TMPL,Transseries)

/******************************************************************************
* Canonical decompositions
******************************************************************************/

TMPL Stair
bounded_stair (const Transbasis& B, const Monomial& m, const Stair& eps) {
  if (B->decode (m) <<= 1) return Stair (m);
  if (nil (eps)) return Stair ();
  Monomial z= car (eps);
  if (B->decode (m * pow (z, 10)) << 1) {
    // FIXME: should really be a test whether z is as least as flat as m
    return bounded_stair (B, m, cdr (eps)) * bounded_stair (B, m*z, eps);
  }
  else return bounded_stair (B, m, cdr (eps));
}

TMPL Stair
bounded_stair (const Transseries& f) {
  Stair eps;
  for (iterator<Variable > it= entries (variables (f->f->f)); busy (it); ++it)
    eps= cons (Monomial (*it, 1), eps);
  return bounded_stair (f->B, f->f->m, eps);
}

TMPL Stair
constant_stair (const Transseries& f) {
  Stair l= bounded_stair (f), r;
  for (; !nil (l); l= cdr (l))
    if (f->B->decode (car (l)) == 1)
      r= cons (car (l), r);
  return r;
}

TMPL inline Transseries infinite_part (const Transseries& f) {
  return head (f->f, bounded_stair (f)); }
TMPL inline Transseries bounded_part (const Transseries& f) {
  return tail (f->f, bounded_stair (f)); }
TMPL inline Transseries constant_part (const Transseries& f) {
  return project (f->f, constant_stair (f)); }
TMPL inline Transseries infinitesimal_part (const Transseries& f) {
  return bounded_part (f) - constant_part (f); }

/******************************************************************************
* Logarithm
******************************************************************************/

TMPL Series
log (const Transbasis& _B, const Variable& v) {
  Transbasis_rep* B= inside (_B);
  if (!B->log_table->contains (v)) {
    Variable lv= log (v);
    B->b_coord= B->b_coord * list<Variable > (lv);
    B->z_coord [1/lv]= Supermonomial (lv, -1);
    B->log_table [v]= Series (Monomial (1/lv, -1));
    //cout << "B= " << B->b_coord << "\n";
  }
  return read (B->log_table, v);
}

TMPL Transseries
log (const Transseries f) {
  Transseries g= regularize (f);
  Series s= log (g->f->f);
  Supermonomial m= f->B->decode (g->f->m) -> m;
  for (iterator<Variable > it= entries (*m); busy (it); ++it)
    if (m[*it] != 0)
      s += m[*it] * log (f->B, *it);
  return Transseries (s);
}

/******************************************************************************
* Exponentiation
******************************************************************************/

TMPL generic
as_expression (const Transseries& f) {
  bool old= get_series_formula_output ();
  set_series_formula_output (true);
  generic r= flatten (f->f->f) * flatten (f->B->decode (f->f->m));
  set_series_formula_output (old);
  return r;
}

TMPL Transseries
exp (const list<Variable >& l, const Transseries& ts) {
  ASSERT (!nil (l), "internal error in transseries exponentiation");

  Transbasis_rep* B= inside (ts->B);
  Transseries f= regularize (ts);
  if (f == Transseries (0)) return 1;
  if (f <<= Transseries (1)) {
    Monomial m= f->f->m;
    m= B->encode (B->decode (m));
    return Transseries (Series (exp (f->f->f * m), 1));
  }

  bool flag= B->log_table->contains (car (l));
  Transseries g= read (B->log_table, car (l));
  if ((!flag) || (f >> g)) {
    // Add new element to transbasis
    C c= dominant_coefficient (f);
    f= f / c;
    Variable bv= exp (as_expression (f));
    Variable zv= pow (bv, construct (-abs (c)));
    nat k= N(f->B->b_coord) - N(l);
    B->b_coord= range (B->b_coord, 0, k) * cons (bv, l);
    B->z_coord [zv]= Supermonomial (bv, -abs (c));
    B->log_table [bv]= f->f;
    //cout << "B= " << B->b_coord << "\n";
    return Transseries (Series (Monomial (zv, -sign (c))));
  }
  else if (asymptotic (f, g)) {
    // f is asymptotic to log of steepest element in l
    C c= dominant_coefficient (f);
    VERIFY (dominant_coefficient (g) == 1, "internal error, transseries exp");
    C a= abs (c);
    Variable zv= pow (car (l), construct (-a));
    B->z_coord [zv]= Supermonomial (car (l), -a);
    Monomial gm (zv, -sign (c));
    Transseries g= Series (gm);
    Transseries h= f - Transseries (Series (c, f->f->m));
    return g * exp (h);
  }
  else return exp (cdr (l), f);
}

TMPL Transseries
exp (const Transseries& f) {
  Transseries h= infinite_part (f);
  return exp (f->B->b_coord, h) * exp (f->B->b_coord, f-h);
}

TMPL Transseries
pow (const Transseries& f, const Transseries& g) {
  return exp (log(f) * g);
}

TMPL Transseries
pow (const Transseries& f, const C& g) {
  return exp (log (f) * g);
}

TMPL Transseries
pow (const C& f, const Transseries& g) {
  return exp (log (f) * g);
}

#undef TMPL_DEF
#undef TMPL
#undef Series
#undef Monomial
#undef Variable
#undef Stair
#undef Transbasis
#undef Transbasis_rep
#undef Transmonomial
#undef Transmonomial_rep
#undef Transseries
#undef Transseries_rep
} // namespace mmx
#endif // __MML_TRANSSERIES_HPP
