
/******************************************************************************
* MODULE     : mv_series_test.cc
* DESCRIPTION: Test multivariate series
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rational.hpp"
#include "mv_series.hpp"
#include "linear_series.hpp"
#include "laurent_series.hpp"
using namespace mmx;

#define Monomial monomial<int,generic>
#define Stair list<Monomial >
#define Series series<rational,multivariate<naive_multiply> >
#define Laurent_series laurent_series<rational,multivariate<naive_multiply> >

int
main () {
  Series x (generic ("x"));
  Series y (generic ("y"));
  Series z (generic ("z"));
  Series f= exp (x + y * exp (z));

  //cout << "x\t\t= " << x << "\n";
  //cout << "y\t\t= " << y << "\n";
  //cout << "x+y\t\t= " << x+y << "\n";
  //cout << "exp(x+y)\t= " << exp(x+y) << "\n";
  //cout << "x+y+exp(x+y)\t= " << (x+y)+exp(x+y) << "\n";

  cout << "x\t\t= " << x << " + ...\n";
  cout << "y\t\t= " << y << " + ...\n";
  cout << "exp(x)\t\t= " << exp (x) << " + ...\n";
  cout << "x+y\t\t= " << x + y << " + ...\n";
  cout << "log(1+x+y)\t= " << log (1 + x + y) << " + ...\n";
  cout << "exp(x+y)\t= " << exp (x + y) << " + ...\n";
  cout << "cos(x+y)\t= " << cos (x + y) << " + ...\n";
  cout << "sin(x+y)\t= " << sin (x + y) << " + ...\n";
  cout << "tan(x+y)\t= " << tan (x + y) << " + ...\n";
  cout << "exp(x+sin(y+z))\t= " << exp (x + sin (y + z)) << " + ...\n\n";

  cout << "f\t\t= " << f << " + ...\n";
  cout << "df/dx\t\t= " << derive (f, "x") << " + ...\n";
  cout << "df/dy\t\t= " << derive (f, "y") << " + ...\n";
  cout << "df/dz\t\t= " << derive (f, "z") << " + ...\n";
  cout << "I(f,x)\t\t= " << integrate (f, "x") << " + ...\n";
  cout << "I(f,y)\t\t= " << integrate (f, "y") << " + ...\n";
  cout << "I(f,z)\t\t= " << integrate (f, "z") << " + ...\n\n";

  Monomial m1 (generic ("x"), 1);
  Monomial m2 (generic ("y"), 1);
  Monomial m3 (generic ("z"), 2);
  Monomial m4= m1*m2;
  cout << "m1\t\t= " << m1 << "\n";
  cout << "m2\t\t= " << m2 << "\n";
  cout << "m3\t\t= " << m3 << "\n";
  cout << "m4\t\t= " << m4 << "\n";
  cout << "gcd(m3,m4)\t= " << gcd (m3, m4) << "\n";
  cout << "lcm(m3,m4)\t= " << lcm (m3, m4) << "\n";
  cout << "f\t\t= " << f << "\n";
  cout << "f[m4]\t\t= " << f[m4] << "\n";
  cout << "f+m4\t\t= " << f+Series(m4) << "\n";
  cout << "f*m4\t\t= " << f*m4 << "\n";
  cout << "H(f; m3)\t= " << head (f, Stair (m3)) << "\n";
  cout << "T(f; m3)\t= " << tail (f, Stair (m3)) << "\n";
  cout << "H(f; m3, m4)\t= " << head (f, Stair (m3, m4)) << "\n";
  cout << "T(f; m3, m4)\t= " << tail (f, Stair (m3, m4)) << "\n\n";

  Series g= tail (f, Stair (m3, m4));
  cout << "g\t\t= " << g << "\n";
  cout << "D(g)\t\t= " << dominant_monomials (g) << "\n";
  cout << "D(g-m4)\t\t= " << dominant_monomials (g-Series(m4)) << "\n\n";

  Laurent_series h1 (f, 1/m1);
  Laurent_series h2 (f, 1/m2);
  Laurent_series h3 (g, 1/m4);
  cout << "h1\t\t= " << h1 << "\n";
  cout << "h2\t\t= " << h2 << "\n";
  cout << "h3\t\t= " << h3 << "\n";
  cout << "h1+h2\t\t= " << h1+h2 << "\n";
  cout << "h1*h2\t\t= " << h1*h2 << "\n";
  cout << "h1/h2\t\t= " << h1/h2 << "\n";
  cout << "D(h3)\t\t= " << dominant_monomials (h3) << "\n";
  cout << "D(h2+h3)\t= " << dominant_monomials (h2+h3) << "\n";
  cout << "D(h3*h3)\t= " << dominant_monomials (h3*h3) << "\n";

  return 0;
}
