
/******************************************************************************
* MODULE     : vector_glue.hpp
* DESCRIPTION: Glue for vectors
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_VECTOR_GLUE_HPP
#define __MMX_VECTOR_GLUE_HPP
#include "dense_vector.hpp"
#include "category_meta.hpp"
#include "category_lift_glue.hpp"
#include "tuple.hpp"
#include "integer.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Vector vector<C>

TMPL Vector
make_vector (const tuple<C>& t) {
  return Vector (iterate (t));
}

TMPL
class vector_element_rep: public alias_rep<C> {
  alias<Vector > a;
  nat i;
public:
  inline vector_element_rep (const alias<Vector >& a2, nat i2):
    a (a2), i (i2) {}
  C get () const { return read (get_alias (a), i); }
  C& open () const { return open_alias (a) [i]; }
  void close () const { close_alias (a); }
};

TMPL alias<C>
vector_element (const alias<Vector >& a, const integer& i) {
  return new vector_element_rep<C> (a, as_int (i));
}

TMPL inline C
vector_access (const Vector& v, const integer& i) {
  return v[as_int (i)];
}

TMPL inline integer vector_size (const Vector& v) { return N(v); }

template<typename Cond, typename C> void
define_vector () {
  typedef both<different<C,generic>,Cond> Specialized;

  define_type<Cond,Vector > (gen ("Vector", type_name<C> ()));
  define<Cond> ("vector", make_vector<C>);
  define_upgrader<Cond,Vector,C> ();
  define_from_iterator<Cond,C> ("vector");
  define_downgrader<Cond,iterator<C>,Vector >
    (iterate<C>, PENALTY_HOMOMORPHISM);

  lift_field_ops<Cond,Vector,C> ();
  lift_scalar_ops<Cond,Vector,C> ();
  lift_ordering_ops<Cond,Vector,C> ();

  define<Cond> (GEN_SIZE, vector_size<C>);
  define<Cond> (GEN_ACCESS, vector_access<C>);
  define<Cond> (GEN_ACCESS, vector_element<C>);
}

#undef TMPL
#undef Vector
} // namespace mmx
#endif // __MMX_VECTOR_GLUE_HPP
