
/******************************************************************************
* MODULE     : vector_format.cpp
* DESCRIPTION: Vector formatted black box data
* COPYRIGHT  : (C) 2007  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "vector_format.hpp"
namespace mmx {
//namespace bbx {
#define Format const vector_format_rep*
#define INSTALL(l,r) (*((void**) ((void*) (&l)))) = ((void*) r)

void vector_init (data x, Format xf) {
  nat sz= xf->f->size;
  for (nat k=0; k<xf->n; k++, x+=sz) init (x, xf->f); }
void vector_init_int (data x, int i, Format xf) {
  nat sz= xf->f->size;
  for (nat k=0; k<xf->n; k++, x+=sz) init (x, i, xf->f); }
void vector_init_copy (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz) init (x, y, xyf->f); }
void vector_clear (data x, Format xf) {
  nat sz= xf->f->size;
  for (nat k=0; k<xf->n; k++, x+=sz) clear (x, xf->f); }
void vector_set (data x, Format xf) {
  nat sz= xf->f->size;
  for (nat k=0; k<xf->n; k++, x+=sz) set (x, xf->f); }
void vector_set_int (data x, int i, Format xf) {
  nat sz= xf->f->size;
  for (nat k=0; k<xf->n; k++, x+=sz) set (x, i, xf->f); }
void vector_set_copy (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz) set (x, y, xyf->f); }

generic vector_flatten (data x, Format xf) {
  nat sz= xf->f->size;
  generic args= comma ();
  for (nat k=0; k<xf->n; k++, x+=sz)
    args= comma (args, flatten (x, xf->f));
  return xsqtuple (args); }
nat vector_hash (data x, Format xf) {
  nat sz= xf->f->size, h= 12345;
  for (nat k=0; k<xf->n; k++, x+=sz)
    h= (h<<1) ^ (h<<5) ^ (h>>27) ^ hash (x, xf->f);
  return h; }
bool vector_equal (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    if (unequal (x, y, xyf->f)) return false;
  return true; }
bool vector_unequal (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    if (unequal (x, y, xyf->f)) return true;
  return false; }
nat vector_soft_hash (data x, Format xf) {
  nat sz= xf->f->size, h= 12345;
  for (nat k=0; k<xf->n; k++, x+=sz)
    h= (h<<1) ^ (h<<5) ^ (h>>27) ^ soft_hash (x, xf->f);
  return h; }
bool vector_eq (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    if (neq (x, y, xyf->f)) return false;
  return true; }
bool vector_neq (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    if (neq (x, y, xyf->f)) return true;
  return false; }

void vector_neg (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    neg (x, y, xyf->f); }
void vector_neg_set (data x, Format xf) {
  nat sz= xf->f->size;
  for (nat k=0; k<xf->n; k++, x+=sz)
    neg (x, xf->f); }
void vector_add (data x, data y, data z, Format xyzf) {
  nat sz= xyzf->f->size;
  for (nat k=0; k<xyzf->n; k++, x+=sz, y+=sz, z+=sz)
    add (x, y, z, xyzf->f); }
void vector_add_set (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    add (x, y, xyf->f); }
void vector_sub (data x, data y, data z, Format xyzf) {
  nat sz= xyzf->f->size;
  for (nat k=0; k<xyzf->n; k++, x+=sz, y+=sz, z+=sz)
    sub (x, y, z, xyzf->f); }
void vector_sub_set (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    sub (x, y, xyf->f); }
void vector_mul (data x, data y, data z, Format xyzf) {
  nat sz= xyzf->f->size;
  for (nat k=0; k<xyzf->n; k++, x+=sz, y+=sz, z+=sz)
    mul (x, y, z, xyzf->f); }
void vector_mul_bi (data x, data y, data z, Format yf, Format zf) {
  //VERIFY_FORMAT (of_vector_format (zf));
  VERIFY_FORMAT (yf->n == zf->n);
  nat sz= yf->f->size;
  for (nat k=0; k<yf->n; k++, x+=sz, y+=sz, z+=sz)
    mul (x, y, z, yf->f, zf->f); }
void vector_mul_set (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    mul (x, y, xyf->f); }
void vector_div (data x, data y, data z, Format xyzf) {
  nat sz= xyzf->f->size;
  for (nat k=0; k<xyzf->n; k++, x+=sz, y+=sz, z+=sz)
    div (x, y, z, xyzf->f); }
void vector_div_bi (data x, data y, data z, Format yf, Format zf) {
  //VERIFY_FORMAT (of_vector_format (zf));
  nat sz= yf->f->size;
  VERIFY_FORMAT (yf->n == zf->n);
  for (nat k=0; k<yf->n; k++, x+=sz, y+=sz, z+=sz)
    div (x, y, z, yf->f, zf->f); }
void vector_div_set (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    div (x, y, xyf->f); }
void vector_quo (data x, data y, data z, Format xyzf) {
  nat sz= xyzf->f->size;
  for (nat k=0; k<xyzf->n; k++, x+=sz, y+=sz, z+=sz)
    quo (x, y, z, xyzf->f); }
void vector_quo_bi (data x, data y, data z, Format yf, Format zf) {
  //VERIFY_FORMAT (of_vector_format (zf));
  VERIFY_FORMAT (yf->n == zf->n);
  nat sz= yf->f->size;
  for (nat k=0; k<yf->n; k++, x+=sz, y+=sz, z+=sz)
    quo (x, y, z, yf->f, zf->f); }
void vector_quo_set (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    quo (x, y, xyf->f); }
void vector_rem (data x, data y, data z, Format xyzf) {
  nat sz= xyzf->f->size;
  for (nat k=0; k<xyzf->n; k++, x+=sz, y+=sz, z+=sz)
    rem (x, y, z, xyzf->f); }
void vector_rem_bi (data x, data y, data z, Format yf, Format zf) {
  //VERIFY_FORMAT (of_vector_format (zf));
  VERIFY_FORMAT (yf->n == zf->n);
  nat sz= yf->f->size;
  for (nat k=0; k<yf->n; k++, x+=sz, y+=sz, z+=sz)
    rem (x, y, z, yf->f, zf->f); }
void vector_rem_set (data x, data y, Format xyf) {
  nat sz= xyf->f->size;
  for (nat k=0; k<xyf->n; k++, x+=sz, y+=sz)
    rem (x, y, xyf->f); }

vector_format_methods_rep::vector_format_methods_rep ():
  format_methods_rep ()
{
  INSTALL (init, vector_init);
  INSTALL (init_int, vector_init_int);
  INSTALL (init_copy, vector_init_copy);
  INSTALL (clear, vector_clear);
  INSTALL (set, vector_set);
  INSTALL (set_int, vector_set_int);
  INSTALL (set_copy, vector_set_copy);

  INSTALL (flatten, vector_flatten);
  INSTALL (hash, vector_hash);
  INSTALL (equal, vector_equal);
  INSTALL (unequal, vector_unequal);
  INSTALL (soft_hash, vector_hash);
  INSTALL (eq, vector_eq);
  INSTALL (neq, vector_neq);

  INSTALL (neg, vector_neg);
  INSTALL (neg_set, vector_neg_set);
  INSTALL (add, vector_add);
  INSTALL (add_set, vector_add_set);
  INSTALL (sub, vector_sub);
  INSTALL (sub_set, vector_sub_set);
  INSTALL (mul, vector_mul);
  INSTALL (mul_bi, vector_mul_bi);
  INSTALL (mul_set, vector_mul_set);
  INSTALL (div, vector_div);
  INSTALL (div_bi, vector_div_bi);
  INSTALL (div_set, vector_div_set);
  INSTALL (quo, vector_quo);
  INSTALL (quo_bi, vector_quo_bi);
  INSTALL (quo_set, vector_quo_set);
  INSTALL (rem, vector_rem);
  INSTALL (rem_bi, vector_rem_bi);
  INSTALL (rem_set, vector_rem_set);
}

vector_format_methods_rep vector_format_rep::mrep=
  vector_format_methods_rep ();

#undef Format
//} // namespace bbx
} // namespace mmx
