
/******************************************************************************
* MODULE     : holonomic_glue.hpp
* DESCRIPTION: Glue for holonomic functions
* COPYRIGHT  : (C) 2007  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_HOLONOMIC_GLUE_HPP
#define __MMX_HOLONOMIC_GLUE_HPP
#include "glue.hpp"
#include "holonomic.hpp"
#include "tuple.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Polynomial polynomial<C>
#define D_multiply skew_multiply<d_operator>
#define D_variant dense_polynomial<D_multiply >
#define Holonomic holonomic<C>

template<typename C,typename V> polynomial<C,V>
make_skew_polynomial (const tuple<C>& t) {
  nat i, n= N (t);
  C* coeffs= mmx_new<C> (n);
  for (i=0; i<n; i++)
    coeffs[n-1-i]= t[i];
  return polynomial<C,V> (coeffs, n);
}

TMPL Holonomic
make_holonomic (const tuple<Polynomial >& t) {
  return Holonomic (make_skew_polynomial<Polynomial,D_variant> (t));
}

template<typename Cond, typename C> void
define_holonomic () {
  define_type<Cond,Holonomic > (gen ("Holonomic", type_name<C> ()));
  define<Cond> ("holonomic", make_holonomic<C>);
  define_from_iterator<Cond,C> ("holonomic");

  define<Cond> ("D_equation", D_equation<C>);
  define<Cond> ("XD_equation", XD_equation<C>);
  define<Cond> ("S_equation", S_equation<C>);
  define<Cond> ("T_equation", T_equation<C>);

  define<Cond> ("move", move<C>);
  define<Cond> ("scale", scale<C>);

  define<Cond> ("step_relation", holonomic_step_relation<C>);
  define<Cond> ("cumul_relation", holonomic_cumul_relation<C>);

  //define<Cond> ("id_test", id_test<C>);
  //define<Cond> ("generic_sum", generic_sum<C>);
}

#undef TMPL
#undef Polynomial
#undef D_multiply
#undef D_variant
#undef Holonomic
} // namespace mmx
#endif // __MMX_HOLONOMIC_GLUE_HPP
