
/******************************************************************************
* MODULE     : holonomic.hpp
* DESCRIPTION: Holonomic functions
* COPYRIGHT  : (C) 2007  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_HOLONOMIC_HPP
#define __MMX_HOLONOMIC_HPP
#include "holonomic_operators.hpp"
#include "dense_matrix.hpp"
#include "pair.hpp"
#include "wrap.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Holonomic holonomic<C>
#define Polynomial polynomial<C>
#define Ratfunc quotient<Polynomial,Polynomial >
#define D_multiply skew_multiply<d_operator>
#define D_variant dense_polynomial<D_multiply >
#define D_operator polynomial<Polynomial,D_variant >
#define D_monic_operator polynomial<Ratfunc,D_variant >
#define XD_multiply skew_multiply<xd_operator>
#define XD_variant dense_polynomial<XD_multiply >
#define XD_operator polynomial<Polynomial,XD_variant >
#define XD_monic_operator polynomial<Ratfunc,XD_variant >
#define S_multiply skew_multiply<S_operator<C> >
#define S_variant dense_polynomial<S_multiply >
#define S_operator polynomial<Polynomial,S_variant >
#define S_monic_operator polynomial<Ratfunc,S_variant >
#define T_multiply skew_multiply<T_operator<C> >
#define T_variant dense_polynomial<T_multiply >
#define T_operator polynomial<Polynomial,T_variant >
#define T_monic_operator polynomial<Ratfunc,T_variant >
TMPL class holonomic;

/******************************************************************************
* Holonomic functions
******************************************************************************/

TMPL
class holonomic {
  D_operator L;
public:
  inline D_operator operator * () const { return L; }
  inline holonomic (const D_operator& L2):
    L (make_reduced (L2)) {}
  inline holonomic (const D_monic_operator& L2):
    L (make_fraction_free (L2)) {}
  inline holonomic (const XD_operator& L2):
    L (as_D_operator (make_reduced (L2))) {}
  inline holonomic (const XD_monic_operator& L2):
    L (as_D_operator (make_fraction_free (L2))) {}
  inline holonomic (const S_operator& L2):
    L (as_D_operator (make_reduced (L2))) {}
  //inline holonomic (const S_monic_operator& L2):
  //  L (as_D_operator (make_fraction_free (L2))) {}
  inline holonomic (const T_operator& L2):
    L (as_D_operator (make_reduced (L2))) {}
  //inline holonomic (const T_monic_operator& L2):
  //  L (as_D_operator (make_fraction_free (L2))) {}
};

WRAP_INDIRECT_IMPL(TMPL inline,Holonomic)

/******************************************************************************
* Different flavours of vanishing operators
******************************************************************************/

TMPL inline D_operator
D_vanishing_operator (const Holonomic& f) {
  return *f;
}

TMPL inline D_monic_operator
D_monic_vanishing_operator (const Holonomic& f) {
  return make_monic (D_vanishing_operator (f));
}

TMPL inline XD_operator
XD_vanishing_operator (const Holonomic& f) {
  return as_XD_operator (D_vanishing_operator (f));
}

TMPL inline XD_monic_operator
XD_monic_vanishing_operator (const Holonomic& f) {
  return make_monic (XD_vanishing_operator (f));
}

TMPL inline S_operator
S_vanishing_operator (const Holonomic& f) {
  return as_S_operator (XD_vanishing_operator (f));
}

/* FIXME: why does this not work?
TMPL inline S_monic_operator
S_monic_vanishing_operator (const Holonomic& f) {
  return make_monic (S_vanishing_operator (f));
}
*/

TMPL inline T_operator
T_vanishing_operator (const Holonomic& f) {
  return as_T_operator (XD_vanishing_operator (f));
}

/* FIXME: why does this not work?
TMPL inline T_monic_operator
T_monic_vanishing_operator (const Holonomic& f) {
  return make_monic (T_vanishing_operator (f));
}
*/

//TMPL inline Holonomic
//id_test (const Holonomic& f) {
//  return Holonomic (T_vanishing_operator (f));
//}

/******************************************************************************
* Output routines
******************************************************************************/

TMPL inline generic
flatten (const Holonomic& f) {
  return flatten (*f);
}

PRINT_SUGAR(TMPL,Holonomic)

template<typename C> generic
flattened_output (const C& x) {
  select_output_evaluator ();
  generic r= flatten (x);
  restore_evaluator ();
  return r;
}

TMPL inline generic
D_equation (const Holonomic& f) {
  return flattened_output (D_vanishing_operator (f));
}

TMPL inline generic
XD_equation (const Holonomic& f) {
  return flattened_output (XD_vanishing_operator (f));
}

TMPL inline generic
S_equation (const Holonomic& f) {
  return flattened_output (S_vanishing_operator (f));
}

TMPL inline generic
T_equation (const Holonomic& f) {
  return flattened_output (T_vanishing_operator (f));
}

/******************************************************************************
* Other operations on holonomic functions
******************************************************************************/

TMPL Holonomic
move (const Holonomic& f, const C& sh) {
  // Replace x -> x + sh
  D_operator L= D_vanishing_operator (f);
  Polynomial* c= mmx_new<Polynomial > (N(L));
  for (nat i=0; i<N(L); i++)
    c[i]= shift (L[i], sh);
  return Holonomic (D_operator (c, N(L)));
}

TMPL Holonomic
scale (const Holonomic& f, const C& sc) {
  // Replace x -> sc x
  D_operator L= D_vanishing_operator (f);
  C p= 1;
  Polynomial* c= mmx_new<Polynomial > (N(L));
  for (nat i=0; i<N(L); i++, p /= sc)
    c[i]= p * q_difference (L[i], sc);
  return Holonomic (D_operator (c, N(L)));
}

/* Requires implementation of skew lcm
TMPL Holonomic
generic_sum (const Holonomic& f, const Holonomic& g) {
  D_monic_operator K= D_monic_vanishing_operator (f);
  D_monic_operator L= D_monic_vanishing_operator (g);
  return Holonomic (lcm (K, L));
}
*/

/******************************************************************************
* Matrix recurrences
******************************************************************************/

#define Matrix matrix<C>
#define Pair   pair<Matrix,C>

TMPL Pair
step_relation (const S_operator& L, nat start, nat end) {
  ASSERT (deg (L) > 0, "no reccurence relation");
  nat n= (nat) deg (L);
  if (end <= start) {
    ASSERT (end == start, "only positive steps have been implemented");
    return Pair (Matrix (C(1), n, n), C(1));
  }
  else if (end == start+1) {
    C a= eval (L[n], C(start));
    C* m= mmx_new<C> (n*n, C(0));
    for (nat i=0; i<n-1; i++)
      m[i*n + (i+1)]= a;
    for (nat i=0; i<n; i++)
      m[(n-1)*n + i]= -eval (L[i], C(start));
    return Pair (Matrix (m, n, n), a);
  }
  else {
    nat mid= (start + end + 1) >> 1;
    return step_relation (L, mid, end) * step_relation (L, start, mid);
  }
}

TMPL Pair
cumul_relation (const S_operator& L, nat start, nat end) {
  ASSERT (deg (L) > 0, "no reccurence relation");
  nat d= (nat) deg (L), n= d+1;
  if (end <= start) {
    ASSERT (end == start, "only positive steps have been implemented");
    return Pair (Matrix (C(1), n, n), C(1));
  }
  else if (end == start+1) {
    C a= eval (L[d], C(start));
    C* m= mmx_new<C> (n*n, C(0));
    for (nat i=0; i<d-1; i++)
      m[i*n + (i+1)]= a;
    for (nat i=0; i<d; i++)
      m[(d-1)*n + i]= -eval (L[i], C(start));
    m[(n-1)*n] = a;
    m[n*n-1]   = a;
    return Pair (Matrix (m, n, n), a);
  }
  else {
    nat mid= (start + end + 1) >> 1;
    return cumul_relation (L, mid, end) * cumul_relation (L, start, mid);
  }
}

TMPL Matrix
holonomic_step_relation (const Holonomic& f, const int& i1, const int& i2) {
  S_operator K= S_vanishing_operator (f);
  S_operator L= make_coefficients_fraction_free (K);
  cout << "K= " << K << "\n";
  cout << "L= " << L << "\n";
  Pair p= step_relation (L, i1, i2);
  return p.x1 / p.x2;
}

TMPL Matrix
holonomic_cumul_relation (const Holonomic& f, const int& i1, const int& i2) {
  S_operator K= S_vanishing_operator (f);
  S_operator L= make_coefficients_fraction_free (K);
  cout << "K= " << K << "\n";
  cout << "L= " << L << "\n";
  Pair p= cumul_relation (L, i1, i2);
  return p.x1 / p.x2;
}

#undef Matrix
#undef Pair

#undef TMPL
#undef Holonomic
#undef Polynomial
#undef Ratfunc
#undef D_multiply
#undef D_variant
#undef D_operator
#undef D_monic_operator
#undef XD_multiply
#undef XD_variant
#undef XD_operator
#undef XD_monic_operator
#undef S_multiply
#undef S_variant
#undef S_operator
#undef S_monic_operator
#undef T_multiply
#undef T_variant
#undef T_operator
#undef T_monic_operator
} // namespace mmx
#endif // __MMX_HOLONOMIC_HPP
