
/******************************************************************************
* MODULE     : interpolate_test.cc
* DESCRIPTION: Test asymptotic interpolation
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real_ball.hpp"
#include "interpolate.hpp"
using namespace mmx;

const int order= 1000;

vector<real_ball>
fact_vector () {
  vector<real_ball> v (real_ball (0), order);
  v[0]= real_ball (1);
  for (int i=1; i<order; i++)
    v[i]= v[i-1] * real_ball (i);
  return v;
}

vector<real_ball>
test_vector () {
  vector<real_ball> v (real_ball (0), order);
  for (int nn=0; nn<order; nn++) {
    real_ball n (nn);
    //v[nn]= n * exp (square (n) / (log (n) + 1));
    //v[nn]= exp (square (square (n)));
    //v[nn]= exp (sqrt (n) * exp (1 / (n+1)));
    //v[nn]= log (exp (n * square (log (n))) * exp (1 / (n+1)));
    //v[nn]= n * exp (sqrt (n) / square (log (n + 1)));
    v[nn]= n * exp (sqrt (n) / square (1 - 1 / log (n+1)));
    //v[nn]= exp (sqrt (n) * log (n) / (1 + 1/n));
    //v[nn]= n / (n+1);
    //v[nn]= exp (exp (1 / n));
  }
  return v;
}

int
main () {
  (void) set_precision (192);
  vector<real_ball> v= test_vector ();
  vector<real_ball> r= interpolate (v, 0, order/3, 16);
  cout << "\n";
  show_changes (v, r, 0);
  return 0;
}
