
/******************************************************************************
* MODULE     : basix.cpp
* DESCRIPTION: Basic inclusions, definitions and routines
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "list.hpp"
#include "basix_utils.hpp"
namespace mmx {

void init_system ();
void init_parse_tools ();
void term_parse_tools ();

/******************************************************************************
* Initialization
******************************************************************************/

static list<void_function> glob_inits;
static list<void_function> glob_terms;

global_initializer::global_initializer (void_function initializer) {
  glob_inits= cons (initializer, glob_inits);
}

global_terminator::global_terminator (void_function terminator) {
  glob_terms= cons (terminator, glob_terms);
}

void
mmx_initialize () {
  glob_inits= reverse (glob_inits);
  while (!nil (glob_inits)) {
    car (glob_inits) ();
    glob_inits= cdr (glob_inits);
  }
}

void
mmx_terminate () {
  glob_terms= reverse (glob_terms);
  while (!nil (glob_terms)) {
    car (glob_terms) ();
    glob_terms= cdr (glob_terms);
  }
}

INITIALIZER(init_system)
INITIALIZER(init_parse_tools)
TERMINATOR (term_parse_tools)

/******************************************************************************
* Error management
******************************************************************************/

volatile void
fatal_error (char* message, char* routine, char* file) {
  cout << "\nFatal error: " << message
       << "\n             in '" << routine
       << "' in file '" << file << "'\n";
  exit (1);
}

exception
as_exception (const generic& x) {
  return exception (x);
}

exception
error_message (const char* msg) {
  return exception (generic (msg), generic ("C++ code"));
}

exception
error_message (const string& msg) {
  return exception (generic (msg), generic ("C++ code"));
}

/******************************************************************************
* Convenience routines
******************************************************************************/

static nat current_indentation;
static char indent_s[3]= "  ";
static char unindent_s[3]= "\b\b";
static char hrule_s[80]= "-------------------------------------------------------------------------------";

ostream&
operator << (ostream& out, print_format x) {
  nat i;
  switch (x) {
  case blank:
    indent_s[0]= ' ';
    indent_s[1]= ' ';
    break;
  case stroke:
    indent_s[0]= '|';
    indent_s[1]= ' ';
    break;
  case indent:
    current_indentation++;
    out << indent_s;
    break;
  case unindent:
    current_indentation--;
    out << unindent_s;
    break;
  case lf:
    out << "\n";
    for (i=0; i<current_indentation; i++)
      out << indent_s;
    break;
  case hrule:
    out << hrule_s;
    break;
  }
  return out;
}

} // namespace mmx
