
/******************************************************************************
* MODULE     : type_props.hpp
* DESCRIPTION: Associate types, variables and functions to types
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_TYPE_PROPS_HPP
#define __MMX_TYPE_PROPS_HPP
#include "basix.hpp"
namespace mmx {

/******************************************************************************
* Types associated to types
******************************************************************************/

#define TYPE_TO_TYPE(name,expr)			\
template<typename C>				\
struct name##_helper {				\
  typedef expr val;				\
};

#define STYPE_TO_TYPE(TMPL,name,C,expr)		\
TMPL						\
struct name##_helper<C > {			\
  typedef expr val;				\
};

class unknown_scalar_type {};
class unknown_real_type {};
//class unknown_numerator_type {};
//class unknown_denominator_type {};
class unknown_center_type {};
class unknown_radius_type {};
class unknown_as_vector_type {};
class unknown_as_matrix_type {};
template<typename C> class complexify;

TYPE_TO_TYPE(scalar_type,C);
TYPE_TO_TYPE(real_type,unknown_real_type);
TYPE_TO_TYPE(abs_type,C);
//TYPE_TO_TYPE(numerator_type,unknown_numerator_type);
//TYPE_TO_TYPE(denominator_type,unknown_denominator_type);
TYPE_TO_TYPE(numerator_type,C);
TYPE_TO_TYPE(denominator_type,C);
TYPE_TO_TYPE(center_type,unknown_center_type);
TYPE_TO_TYPE(radius_type,unknown_radius_type);
TYPE_TO_TYPE(complex_type,complexify<C>);
TYPE_TO_TYPE(as_vector_type,unknown_radius_type);
TYPE_TO_TYPE(as_matrix_type,unknown_radius_type);

#define Scalar_type(C) typename scalar_type_helper<C >::val
#define Real_type(C) typename real_type_helper<C >::val
#define Abs_type(C) typename abs_type_helper<C >::val
#define Numerator_type(C) typename numerator_type_helper<C >::val
#define Denominator_type(C) typename denominator_type_helper<C >::val
#define Center_type(C) typename center_type_helper<C >::val
#define Radius_type(C) typename radius_type_helper<C >::val
#define Complex_type(C) typename complex_type_helper<C >::val
#define As_vector_type(C) typename as_vector_type_helper<C >::val
#define As_matrix_type(C) typename as_matrix_type_helper<C >::val

/******************************************************************************
* Constants associated to types
******************************************************************************/

template<typename C>
struct constant_helper {
  static inline C pi () {
    ERROR ("not implemented (pi)"); return 0; }
  static inline C log2 () {
    ERROR ("not implemented (log_2)"); return 0; }
  static inline C euler () {
    ERROR ("not implemented (euler)"); return 0; }
  static inline C catalan () {
    ERROR ("not implemented (catalan)"); return 0; }
  static inline C imaginary () {
    ERROR ("not implemented (imaginary)"); return 0; }
  static inline C pi_U () {
    ERROR ("not implemented (pi_U)"); return 0; }
  static inline C log2_U () {
    ERROR ("not implemented (log2_U)"); return 0; }
  static inline C euler_U () {
    ERROR ("not implemented (euler_U)"); return 0; }
  static inline C catalan_U () {
    ERROR ("not implemented (catalan_U)"); return 0; }
  static inline C pi_D () {
    ERROR ("not implemented (pi_D)"); return 0; }
  static inline C log2_D () {
    ERROR ("not implemented (log2_D)"); return 0; }
  static inline C euler_D () {
    ERROR ("not implemented (euler_D)"); return 0; }
  static inline C catalan_D () {
    ERROR ("not implemented (catalan_D)"); return 0; }
  static inline C nan () {
    ERROR ("not implemented (nan)"); return 0; }
  static inline C fuzz () {
    ERROR ("not implemented (fuzz)"); return 0; }
  static inline C epsilon () {
    ERROR ("not implemented (epsilon)"); return 0; }
  static inline C infinity () {
    ERROR ("not implemented (infinity)"); return 0; }
  static inline C plus_infinity () {
    ERROR ("not implemented (plus_infinity)"); return 0; }
  static inline C minus_infinity () {
    ERROR ("not implemented (linus_infinity)"); return 0; }
};

#define Pi(C) constant_helper<C >::pi()
#define Log2(C) constant_helper<C >::log2()
#define Euler(C) constant_helper<C >::euler()
#define Catalan(C) constant_helper<C >::catalan()
#define Imaginary(C) constant_helper<C >::imaginary()
#define Pi_D(C) constant_helper<C >::pi_D()
#define Log2_D(C) constant_helper<C >::log2_D()
#define Euler_D(C) constant_helper<C >::euler_D()
#define Catalan_D(C) constant_helper<C >::catalan_D()
#define Pi_U(C) constant_helper<C >::pi_U()
#define Log2_U(C) constant_helper<C >::log2_U()
#define Euler_U(C) constant_helper<C >::euler_U()
#define Catalan_U(C) constant_helper<C >::catalan_U()
#define Nan(C) constant_helper<C >::nan()
#define Fuzz(C) constant_helper<C >::fuzz()
#define Epsilon(C) constant_helper<C >::epsilon()
#define Infinity(C) constant_helper<C >::infinity()
#define Plus_infinity(C) constant_helper<C >::plus_infinity()
#define Minus_infinity(C) constant_helper<C >::minus_infinity()

/******************************************************************************
* Helpers for conversions and casted arithmetic
******************************************************************************/

template<typename T,typename F>
struct cast_helper {
  static inline T cv (const F& x) { return x; }
};

template<typename T,typename F> inline T
as (const F& x) {
  return cast_helper<T,F>::cv (x);
}

template<typename Op,typename T,typename F>
struct cast_unary_helper {
  static inline T op (const F& x) {
    return Op::template cast_helper<T>::op (x); }
};

template<typename Op,typename T,typename F> T
cast_unary (const F& x) {
  return cast_unary_helper<Op,T,F>::op (x);
}

template<typename Op,typename T,typename F1,typename F2>
struct cast_binary_helper {
  static inline T op (const F1& x, const F2& y) {
    return Op::template cast_helper<T>::op (x, y); }
};

template<typename Op,typename T,typename F1,typename F2> T
cast_binary (const F1& x, const F2& y) {
  return cast_binary_helper<Op,T,F1,F2>::op (x, y);
}

/******************************************************************************
* Helpers for rounded constructors
******************************************************************************/

template<typename C>
struct rounded_constructor_helper {
  template<typename T> static C rnd_down (const T& x);
  template<typename T> static C rnd_up   (const T& x);
};

#define New_D(T) rounded_constructor_helper<T >::rnd_D
#define New_U(T) rounded_constructor_helper<T >::rnd_U

/******************************************************************************
* Helpers for interval/ball arithmetic
******************************************************************************/

template<typename C,typename A>
struct make_interval_helper {
  static inline C val (const A& l, const A& r) {
    A c= (l+r) >> 1;
    return C (c, add_U (as<Radius_type(C)> (sub_U (r, l)) >> 1,
			rnd_error (c)));
  }
};

template<typename C,typename A> C
make_interval (const A& l, const A& r) {
  return make_interval_helper<C,A>::val (l, r);
}

template<typename B,typename C,typename R>
struct make_ball_helper {
  static inline B val (const C& c, const R& r) {
    return B (sub_D (c, r), add_U (c, r));
  }
};

template<typename B,typename C,typename R> B
make_ball (const C& c, const R& r) {
  return make_ball_helper<B,C,R>::val (c, r);
}

} // namespace mmx
#endif // __MMX_TYPE_PROPS_HPP
