
/******************************************************************************
* MODULE     : list_glue.cpp
* DESCRIPTION: Standard glue for lists
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "list_glue.hpp"
#include "routine.hpp"
#include "tuple.hpp"
namespace mmx {

list<generic>
list_map_1 (const routine& fun, const list<generic>& l) {
  if (nil (l)) return l;
  return cons (fun->apply (car (l)), list_map_1 (fun, cdr (l)));
}

list<generic>
list_map_2 (const routine& fun,
	    const list<generic>& l1, const list<generic>& l2)
{
  ASSERT (nil (l1) == nil (l2), "lists of unequal lengths");
  if (nil (l1)) return l1;
  return cons (fun->apply (car (l1), car (l2)),
	       list_map_2 (fun, cdr (l1), cdr (l2)));
}

list<generic>
list_map_n (const routine& fun, const array<list<generic> >& a) {
  nat i, n= N(a);
  array<generic> cara= array_fill<generic> (n+1);
  cara[0]= fun->name;
  if (nil (a[0])) {
    for (i=0; i<n; i++)
      ASSERT (nil (a[i]), "lists of unequal lengths");
    return a[0];
  }
  for (i=0; i<n; i++) {
    ASSERT (!nil (a[i]), "lists of unequal lengths");
    cara[i+1]= car (a[i]);
  }
  array<list<generic> > cdra= array_fill<list<generic> > (n);
  for (i=0; i<n; i++) cdra[i]= cdr (a[i]);
  return cons (fun->xapply (array_to_compound (cara)),
	       list_map_n (fun, cdra));
}

list<generic>
list_map (const generic& f, const tuple<list<generic> >& t) {
  routine fun= is<routine> (f)? as<routine> (f): default_routine (f);
  switch (N(t)) {
  case 1: ASSERT (N(t)>0, "wrong number of arguments");
  case 2: return list_map_1 (fun, t[0]);
  case 3: return list_map_2 (fun, t[0], t[1]);
  default:
    {
      const array<generic> a= cdr (compound_to_array (*t));
      nat i, n= N(a);
      array<list<generic> > b= array_fill<list<generic> > (n);
      for (i=0; i<n; i++) b[i]= as<list<generic> > (a[i]);
      return list_map_n (fun, b);
    }
  }
}

generic
list_apply (const generic& f, const list<generic>& l2) {
  routine fun= is<routine> (f)? as<routine> (f): default_routine (f);
  list<generic> l= l2;
  nat i, n= N(l);
  array<generic> a= array_fill<generic> (n+1);
  a[0]= fun->name;
  for (i=0; !nil (l); i++, l= read_cdr(l))
    a[i+1]= read_car (l);
  return fun->xapply (array_to_compound (a));
}

void
define_standard_list () {
  define_list<always,generic> ();
  define<always> ("map", list_map);
  define<always> ("apply", list_apply);
}

} // namespace mmx
