
/******************************************************************************
* MODULE     : table_glue.cpp
* DESCRIPTION: Glue for tables
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_TABLE_GLUE_HPP
#define __MMX_TABLE_GLUE_HPP
#include "table_meta.hpp"
#include "category_lift_glue.hpp"
#include "tuple.hpp"
#include "alias.hpp"
namespace mmx {
#define TMPL template<typename C, typename T>
#define Table table<C,T>

TMPL
class table_element_rep: public alias_rep<C> {
  alias<Table > a;
  T key;
public:
  inline table_element_rep (const alias<Table >& a2, const T& key2):
    a (a2), key (key2) {}
  C get () const { return read (get_alias (a), key); }
  C& open () const { return open_alias (a) [key]; }
  void close () const { close_alias (a); }
};

TMPL alias<C>
table_element (const alias<Table >& a, const T& key) {
  return new table_element_rep<C,T> (a, key);
}

TMPL inline Table make_table (const C& defval) { return Table (defval); }
TMPL inline int table_size (const Table& t) { return (int) N (t); }
TMPL inline C table_access (const Table& t, const T& key) { return t[key]; }

template<typename Cond, typename C, typename T> void
define_table () {
  define_type<Cond,Table > (gen ("Table", type_name<T> (), type_name<C> ()));
  define<Cond> ("table", make_table<C,T>);
  define_upgrader<Cond,Table,C> (make_table<C,T>);

  lift_field_ops<Cond,Table,C> ();
  lift_scalar_ops<Cond,Table,C> ();
  lift_ordering_ops<Cond,Table,C> ();

  define<Cond> (GEN_SIZE, table_size<C,T>);
  define<Cond> (GEN_ACCESS, table_access<C,T>);
  define<Cond> (GEN_ACCESS, table_element<C,T>);

  define_downgrader<Cond,iterator<T>,Table > (entries<T>, PENALTY_CAST);
}

#undef TMPL
#undef Table
} // namespace mmx
#endif // __MMX_TABLE_GLUE_HPP
