
/******************************************************************************
* MODULE     : symbol.hpp
* DESCRIPTION: A symbol points to another object but is stored only once
*              unless explicitely specified
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_SYMBOL_HPP
#define __MMX_SYMBOL_HPP
#include "table.hpp"
namespace mmx {
#define TMPL_DEF template<typename C, typename V=eq_op>
#define TMPL template<typename C, typename V>
#define Symbol symbol<C,V>
#define Symbol_rep symbol_rep<C,V>
TMPL class symbol_rep;
TMPL class symbol;
TMPL inline nat hash (const Symbol& s);
TMPL inline bool operator == (const Symbol& s1, const Symbol& s2);
TMPL inline bool operator != (const Symbol& s1, const Symbol& s2);
TMPL inline nat soft_hash (const Symbol& s);
TMPL inline bool eq (const Symbol& s1, const Symbol& s2);
TMPL inline bool neq (const Symbol& s1, const Symbol& s2);
TMPL inline Symbol fuse (const Symbol& s);

/******************************************************************************
* Symbol class and its representation class
******************************************************************************/

TMPL_DEF
class symbol_rep REP_STRUCT {
public: // FIXME: should become private
  static table<Symbol_rep*,C,V> symbol_entries;
private:
  C ref;
public:
  inline Symbol_rep (const C& ref2): ref (ref2) { symbol_entries [ref]= this; }
  inline ~Symbol_rep () { reset (symbol_entries, ref); }
  friend Symbol fuse LESSGTR (const Symbol& s);
  friend class Symbol;
};

TMPL_DEF
class symbol {
INDIRECT_PROTO_2 (symbol, symbol_rep, C, V)
public:
  symbol (const C& ref) {
    rep= read (Symbol_rep::symbol_entries, ref);
    if (rep == (Symbol_rep*) NULL) rep= new Symbol_rep (ref);
    else INC_COUNT (rep);
  }
  inline const C& operator * () const { return rep->ref; };
  friend nat hash LESSGTR (const Symbol& s);
  friend bool operator == LESSGTR (const Symbol& s1, const Symbol& s2);
  friend bool operator != LESSGTR (const Symbol& s1, const Symbol& s2);
  friend nat soft_hash LESSGTR (const Symbol& s);
  friend bool eq LESSGTR (const Symbol& s1, const Symbol& s2);
  friend bool neq LESSGTR (const Symbol& s1, const Symbol& s2);
};
INDIRECT_IMPL_2 (symbol, symbol_rep, typename C, C, typename V, V)

/******************************************************************************
* Routines related to equality testing
******************************************************************************/

TMPL inline nat hash (const Symbol& s) {
  return hash (*s); }
TMPL inline bool operator == (const Symbol& s1, const Symbol& s2) {
  return (*s1) == (*s2); }
TMPL inline bool operator != (const Symbol& s1, const Symbol& s2) {
  return (*s1) != (*s2); }

template<typename C> inline nat hash (const symbol<C,equal_op>& s) {
  return hard_hash_impl (s.rep); }
template<typename C> inline bool
operator == (const symbol<C,equal_op>& s1, const symbol<C,equal_op>& s2) {
  return s1.rep == s2.rep; }
template<typename C> inline bool
operator != (const symbol<C,equal_op>& s1, const symbol<C,equal_op>& s2) {
  return s1.rep != s2.rep; }

TMPL inline nat soft_hash (const Symbol& s) {
  return hard_hash_impl (s.rep); }
TMPL inline bool eq (const Symbol& s1, const Symbol& s2) {
  return s1.rep == s2.rep; }
TMPL inline bool neq (const Symbol& s1, const Symbol& s2) {
  return s1.rep != s2.rep; }

TMPL inline Symbol melt (const Symbol& s) {
  // NOTE: melting has no effect on symbols contrary to mixed_symbols
  return s; }
TMPL inline Symbol fuse (const Symbol& s) {
  inside (s)->ref= fuse (s->ref); return s; }

/******************************************************************************
* Printing symbols
******************************************************************************/

TMPL table<Symbol_rep*,C,V>
Symbol_rep::symbol_entries ((Symbol_rep*) NULL);
TMPL void show_symbol_entries () {
  cout << "symbols\t= " << Symbol_rep::symbol_entries << "\n"; }
template<typename C> void show_symbol_entries () {
  show_symbol_entries<C,eq_op> (); }

TMPL inline generic flatten (const Symbol& s) {
  return generic ("symbol") (flatten (*s)); }

PRINT_SUGAR(TMPL,Symbol)

#undef TMPL_DEF
#undef TMPL
#undef Symbol
#undef Symbol_rep
} // namespace mmx
#endif // __MMX_SYMBOL_HPP
