
/******************************************************************************
* MODULE     : triple.hpp
* DESCRIPTION: Triples
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_TRIPLE_HPP
#define __MMX_TRIPLE_HPP
#include "basix.hpp"
namespace mmx {
#define TMPL template<typename C1, typename C2, typename C3>
#define Triple triple<C1,C2,C3>

TMPL
class triple {
MMX_ALLOCATORS;
public:
  C1 x1;
  C2 x2;
  C3 x3;
  inline triple () {}
  inline triple (const C1& x1b, const C2& x2b, const C3& x3b):
    x1 (x1b), x2 (x2b), x3 (x3b) {}
};

template<typename Op, typename C1, typename C2, typename C3> nat
unary_hash (const Triple& t) {
  nat h1= Op::op (t.x1), h2= Op::op (t.x2);
  return h1 ^ h2 ^ (h1<<3) ^ (h2<<5) ^ Op::op (t.x3);
}

template<typename Op, typename C1, typename C2, typename C3> bool
binary_test (const Triple& t1, const Triple& t2) {
  return Op::op (t1.x1,t2.x1) && Op::op(t1.x2, t2.x2) && Op::op(t1.x3, t2.x3);
}

TMPL inline generic
flatten (const Triple& t) {
  return xsqtuple (comma (flatten (t.x1),
			  comma (flatten (t.x2), flatten (t.x3))));
}

TRUE_IDENTITY_OP_SUGAR(TMPL,Triple)
SOFT_IDENTITY_OP_SUGAR(TMPL,Triple)
HARD_IDENTITY_OP_SUGAR(TMPL,Triple)
PRINT_SUGAR(TMPL,Triple)

#undef TMPL
#undef Triple
} // namespace mmx
#endif // __MMX_TRIPLE_HPP
