
/******************************************************************************
* MODULE     : environment.hpp
* DESCRIPTION: Environments for the base evaluator
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __ENVIRONMENT_HPP
#define __ENVIRONMENT_HPP
#include "string.hpp"
#include "list.hpp"
#include "table.hpp"

namespace mmx {

class environment_rep;
class environment {
INDIRECT_PROTO (environment, environment_rep)
  inline environment ();
  inline bool operator == (const environment& env) const;
  inline bool operator != (const environment& env) const;
  inline generic operator [] (const generic& x) const;
  inline generic& operator [] (const generic& x);
  friend inline list<string> strings_for_completion (const environment& env);
  // Return the list of all the litterals which are entries of the bindings
  friend inline void reset (const environment& env, const generic& x);
  friend inline bool nil (const environment& env);
  friend inline nat hash (const environment& env);
};

class environment_rep: public rep_struct {
public:
  table<generic,generic,eq_op> bindings;
  environment next;
  nat serial;       // time stamp for validity of converter cache
  nat next_serial;  // required time stamp for parent
public:
  inline environment_rep ();
  inline environment_rep (const environment& e);
  inline ~environment_rep ();

  // environment access
  generic name () const;
  inline generic& set (const generic& var);
  inline void set (const generic& var, const generic& val);
  inline void reset (const generic& var);
  bool contains (const generic& var) const;
  generic get (const generic& var) const;
  bool get (const generic& var, generic& val) const;
  list<string> strings_for_completion () const;

  // converters
  void ensure_up_to_date () const;
  void set_converter (nat src, nat dest, const generic& val, nat trv, nat pen);
  generic get_converter (nat src, nat dest, nat& pen) const;

  friend class environment;
};
INDIRECT_NULL_IMPL (environment, environment_rep)

inline environment_rep::environment_rep (): serial (0), next_serial (0) {}
inline environment_rep::environment_rep (const environment& e):
  next (e), serial (0), next_serial (next->serial) {}
inline environment_rep::~environment_rep () {}
inline generic& environment_rep::set (const generic& var) {
  return bindings [var]; }
inline void environment_rep::set (const generic& var, const generic& val) {
  bindings [var]= val; }
inline void environment_rep::reset (const generic& var) {
  mmx::reset (bindings, var); }

inline environment::environment (): rep (NULL) {}
inline bool environment::operator == (const environment& env) const {
  return rep == env.rep; }
inline bool environment::operator != (const environment& env) const {
  return rep != env.rep; }
inline generic environment::operator [] (const generic& x) const {
  return rep->get (x); }
inline generic& environment::operator [] (const generic& x) {
  return rep->set (x); }
inline generic read (const environment& env, const generic& x) {
  return env[x]; }
inline void reset (const environment& env, const generic& x) {
  return env.rep->reset (x); }

inline list<string> strings_for_completion (const environment& env) {
  return env.rep->strings_for_completion (); }
  
inline bool nil (const environment& env) {
  return env.rep == NULL; }
inline nat hash (const environment& env) {
  return ((nat) ((void*) env.rep)) >> 3; }
inline generic flatten (const environment& env) {
  if (nil (env)) return GEN_NIL; else return env->name (); }

PRINT_SUGAR(,environment);

inline environment global_environment () {
  return new environment_rep (); }
inline environment local_environment (const environment& next) {
  return new environment_rep (next); }

} // namespace mmx
#endif // __ENVIRONMENT_HPP
