
/******************************************************************************
* MODULE     : exception.hpp
* DESCRIPTION: The type for exceptional behaviour
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __EXCEPTION_HPP
#define __EXCEPTION_HPP
// NOTE: exception.hpp is included by basix.hpp

namespace mmx {

class exception {
MMX_ALLOCATORS
  generic rep;
public:
  inline generic operator * () const { return rep; }
  inline exception (const generic& g): rep (g) {}
  inline exception (const exception& t): rep (t.rep) {}
  exception (const generic& msg, const generic& where);
  exception (const generic& msg, const generic& a, const generic& where);
};

//WRAP_WRAPPED_IMPL(inline,exception);
//WRAP_PRINT_IMPL(inline,exception);

inline nat hash (const exception& c) { return hash (*c); }
inline nat soft_hash (const exception& c) { return soft_hash (*c); }
inline nat hard_hash (const exception& c) { return hard_hash (*c); }
inline bool operator == (const exception& c1, const exception& c2) {
  return (*c1) == (*c2); }
inline bool operator != (const exception& c1, const exception& c2) {
  return (*c1) != (*c2); }
inline bool eq (const exception& c1, const exception& c2) {
  return eq (*c1, *c2); }
inline bool neq (const exception& c1, const exception& c2) {
  return neq (*c1, *c2); }
inline bool hard_eq (const exception& c1, const exception& c2) {
  return hard_eq (*c1, *c2); }
inline bool hard_neq (const exception& c1, const exception& c2) {
  return hard_neq (*c1, *c2); }
inline generic flatten (const exception& c) { return flatten (*c); }
inline ostream& operator << (ostream& out, const exception& c) {
  return out << *c; }

/******************************************************************************
* Special exceptions
******************************************************************************/

generic wrong_nr_args (const generic& where);
generic type_mismatch (const generic& expect, const generic& where);

} // namespace mmx
#endif // __EXCEPTION_HPP
