
/******************************************************************************
* MODULE     : glue.hpp
* DESCRIPTION: Routines for defining glue
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __GLUE_HPP
#define __GLUE_HPP
#include "evaluator.hpp"
#include "routine.hpp"
#include "primitive.hpp"
#include "tuple.hpp"
#include "alias.hpp"
#include "category_meta.hpp"
namespace mmx {

/******************************************************************************
* Helpers for conditional definitions
******************************************************************************/

template<bool Cond>
struct define_helper {
  static inline void
  def_primitive (const generic& name, generic (*fun) (const generic&));
};

STMPL
struct define_helper<false> {
  static inline void
  def_primitive (const generic& name, generic (*fun) (const generic&)) {
    (void) name; (void) fun; }
};

template<bool Cond, typename D>
struct define_helper_0 {
  static inline void def_constant (const generic& name, const D& x);
  static inline void def_constructor (generic (*fun) (const D&));
  static inline void def (const generic& name, D (*fun) (), nat p);
  static inline void def_from_iterator (const generic& n, nat p);
  static inline void def_type (const generic& name);
};

template<typename D>
struct define_helper_0<false,D> {
  static inline void def_constant (const generic& name, const D& x) {
    (void) name; (void) x; }
  static inline void def_constructor (generic (*fun) (const D&)) {
    (void) fun; }
  static inline void def (const generic& name, D (*fun) (), nat p) {
    (void) name; (void) fun; (void) p; }
  static inline void def_from_iterator (const generic& name, nat p) {
    (void) name; (void) p; }
  static inline void def_type (const generic& name) {
    (void) name; }
};

template<bool Cond, typename D, typename S1>
struct define_helper_1 {
  typedef D (*fun_type) (const S1&);
  static inline void def (const generic& name, fun_type fun, nat p);
  static inline void def_caster (const generic& name, fun_type fun, nat p);
  static inline void def_upgrader (const generic& name, fun_type fun, nat p);
  static inline void def_downgrader (const generic& name, fun_type fun, nat p);
  static inline void def_converter (const generic& name, fun_type fun, nat p);
};

template<typename D, typename S1>
struct define_helper_1<false,D,S1> {
  typedef D (*fun_type) (const S1&);
  static inline void def (const generic& name, fun_type fun, nat p) {
    (void) name; (void) fun; (void) p; }
  static inline void def_caster (const generic& n, fun_type f, nat p) {
    (void) n; (void) f; (void) p; }
  static inline void def_upgrader (const generic& n, fun_type f, nat p) {
    (void) n; (void) f; (void) p; }
  static inline void def_downgrader (const generic& n, fun_type f, nat p) {
    (void) n; (void) f; (void) p; }
  static inline void def_converter (const generic& n, fun_type f, nat p) {
    (void) n; (void) f; (void) p; }
};

template<bool Cond, typename D, typename S1, typename S2>
struct define_helper_2 {
  static inline void
  def (const generic& name, D (*fun) (const S1&, const S2&), nat p);
};

template<typename D, typename S1, typename S2>
struct define_helper_2<false,D,S1,S2> {
  static inline void
  def (const generic& name, D (*fun) (const S1&, const S2&), nat p) {
    (void) name; (void) fun; (void) p; }
};

template<bool Cond, typename D, typename S1, typename S2, typename S3>
struct define_helper_3 {
  static inline void
  def (const generic& name, D (*fun) (const S1&, const S2&, const S3&), nat p);
};

template<typename D, typename S1, typename S2, typename S3>
struct define_helper_3<false,D,S1,S2,S3> {
  static inline void
  def (const generic& n, D (*fun) (const S1&, const S2&, const S3&), nat p) {
    (void) n; (void) fun; (void) p; }
};

template<typename C>
struct penalty_helper {
  static nat pen;
};

STMPL
struct penalty_helper<generic> {
  static nat pen;
};

/******************************************************************************
* Main functions for gluing routines to the current evaluator
******************************************************************************/

template<bool Cond, typename D> inline void
define_helper_0<Cond,D>::def_constant (const generic& name, const D& x) {
  current_ev->set (name, as<generic> (x));
}

template<typename Cond, typename D> inline void
define_constant (const generic& name, const D& x) {
  typedef typename Cond::Cond When;
  define_helper_0<When::value,D>::def_constant (name, x);
}

template<bool Cond, typename D> inline void
define_helper_0<Cond,D>::def_constructor (generic (*fun) (const D&)) {
  current_ev->overload (GEN_NEW, as<generic> (unary_routine (GEN_NEW, fun)));
}

template<typename Cond, typename D> inline void
define_constructor (generic (*fun) (const D&)) {
  typedef typename Cond::Cond When;
  define_helper_0<When::value,D>::def_constructor (fun);
}

template<bool Cond, typename D> inline void
define_helper_0<Cond,D>::def (const generic& name, D (*fun) (), nat p) {
  current_ev->overload (name, as<generic> (nullary_routine (name, fun)), p);
}

template<typename Cond, typename D> inline void
define (const generic& name, D (*fun) (), nat p= PENALTY_INCLUSION) {
  typedef typename Cond::Cond When;
  define_helper_0<When::value,D>::def (name, fun, p);
}

template<bool Cond, typename D, typename S1> inline void
define_helper_1<Cond,D,S1>::
def (const generic& name, D (*fun) (const S1&), nat p) {
  current_ev->overload (name, as<generic> (unary_routine (name, fun)), p);
}

template<typename Cond, typename D, typename S1> inline void
define (const generic& name, D (*fun) (const S1&), nat p= PENALTY_INCLUSION) {
  typedef typename Cond::Cond When;
  define_helper_1<When::value,D,S1>::def (name, fun, p);
}

template<bool Cond, typename D, typename S1, typename S2> inline void
define_helper_2<Cond,D,S1,S2>::
def (const generic& name, D (*fun) (const S1&, const S2&), nat p) {
  current_ev->overload (name, as<generic> (binary_routine (name, fun)), p);
}

template<typename Cond, typename D, typename S1, typename S2> inline void
define (const generic& name, D (*fun) (const S1&, const S2&),
	nat p= PENALTY_INCLUSION)
{
  typedef typename Cond::Cond When;
  define_helper_2<When::value,D,S1,S2>::def (name, fun, p);
}

template<bool Cond, typename D, typename S1, typename S2, typename S3>
inline void
define_helper_3<Cond,D,S1,S2,S3>::
def (const generic& name, D (*fun) (const S1&, const S2&, const S3&), nat p) {
  current_ev->overload (name, as<generic> (ternary_routine (name, fun)), p);
}

template<typename Cond, typename D, typename S1, typename S2, typename S3>
inline void
define (const generic& name, D (*fun) (const S1&, const S2&, const S3&),
	nat p= PENALTY_INCLUSION)
{
  typedef typename Cond::Cond When;
  define_helper_3<When::value,D,S1,S2,S3>::def (name, fun, p);
}

template<bool Cond, typename D> inline void
define_helper_0<Cond,D>::def_from_iterator (const generic& n, nat p) {
  current_ev->overload (n, as<generic> (from_iterator_routine<D> (n)), p);
}

template<typename Cond, typename D> inline void
define_from_iterator (const generic& name, nat p= PENALTY_INCLUSION) {
  typedef typename Cond::Cond When;
  define_helper_0<When::value,D>::def_from_iterator (name, p);
}

template<bool Cond> inline void
define_helper<Cond>::
def_primitive (const generic& name, generic (*fun) (const generic&)) {
  current_ev->set (name, as<generic> (primitive (name, fun)));
}

template<typename Cond> inline void
define_primitive (const generic& name, generic (*fun) (const generic&)) {
  typedef typename Cond::Cond When;
  define_helper<When::value>::def_primitive (name, fun);
}

/******************************************************************************
* Defining converters
******************************************************************************/

template<typename C> inline generic type_name ();
template<typename C> nat penalty_helper<C>::pen= PENALTY_INCLUSION;

template<bool Cond, typename D, typename S1> inline void
define_helper_1<Cond,D,S1>::
def_caster (const generic& s, D (*f) (const S1&), nat p) {
  routine fun= unary_routine (s, f);
  current_ev->overload (GEN_CAST, as<generic> (fun), p);
}

template<typename Cond, typename D, typename S1> inline void
define_caster (D (*f) (const S1&)= cast_helper<D,S1>::cv,
	       nat p= penalty_helper<S1>::pen) {
  typedef typename Cond::Cond When;
  generic name= gen (GEN_INTO, type_name<S1> (), type_name<D> ());
  define_helper_1<When::value,D,S1>::def_caster (name, f, p);
}

template<typename Cond, typename D, typename S1> inline void
define_caster (nat p) {
  define_caster<Cond,D,S1> (cast_helper<D,S1>::cv, p);
}

template<bool Cond, typename D, typename S1> inline void
define_helper_1<Cond,D,S1>::
def_upgrader (const generic& s, D (*f) (const S1&), nat p) {
  routine fun= unary_routine (s, f);
  current_ev->overload (GEN_UPGRADE, as<generic> (fun), p);
}

template<typename Cond, typename D, typename S1> inline void
define_upgrader (D (*f) (const S1&)= cast_helper<D,S1>::cv,
		 nat p= penalty_helper<S1>::pen) {
  typedef typename Cond::Cond When;
  generic name= gen (GEN_INTO, type_name<S1> (), type_name<D> ());
  define_helper_1<When::value,D,S1>::def_upgrader (name, f, p);
}

template<typename Cond, typename D, typename S1> inline void
define_upgrader (nat p) {
  define_upgrader<Cond,D,S1> (cast_helper<D,S1>::cv, p);
}

template<bool Cond, typename D, typename S1> inline void
define_helper_1<Cond,D,S1>::
def_downgrader (const generic& s, D (*f) (const S1&), nat p) {
  routine fun= unary_routine (s, f);
  current_ev->overload (GEN_DOWNGRADE, as<generic> (fun), p);
}

template<typename Cond, typename D, typename S1> inline void
define_downgrader (D (*f) (const S1&)= cast_helper<D,S1>::cv,
		   nat p= penalty_helper<S1>::pen) {
  typedef typename Cond::Cond When;
  generic name= gen (GEN_INTO, type_name<S1> (), type_name<D> ());
  define_helper_1<When::value,D,S1>::def_downgrader (name, f, p);
}

template<typename Cond, typename D, typename S1> inline void
define_downgrader (nat p) {
  define_downgrader<Cond,D,S1> (cast_helper<D,S1>::cv, p);
}

template<bool Cond, typename D, typename S1> inline void
define_helper_1<Cond,D,S1>::
def_converter (const generic& s, D (*f) (const S1&), nat p) {
  routine fun= unary_routine (s, f);
  current_ev->overload (GEN_CONVERT, as<generic> (fun), p);
}

template<typename Cond, typename D, typename S1> inline void
define_converter (D (*f) (const S1&)= cast_helper<D,S1>::cv,
		  nat p= penalty_helper<S1>::pen) {
  typedef typename Cond::Cond When;
  generic name= gen (GEN_INTO, type_name<S1> (), type_name<D> ());
  define_helper_1<When::value,D,S1>::def_converter (name, f, p);
}

template<typename Cond, typename D, typename S1> inline void
define_converter (nat p) {
  define_converter<Cond,D,S1> (cast_helper<D,S1>::cv, p);
}

/******************************************************************************
* Gluing new data types
******************************************************************************/

array<generic> all_type_names ();
void define_type_sub (const generic& name, nat id);
nat type_id (const generic& name);
generic type_name (nat id);

template<typename C> inline nat type_id () {
  return type_information<C>::id; }
template<typename C> inline generic type_name () {
  return type_name (type_id<C> ()); }

template<bool Cond, typename C>
struct define_type_helper {
  static bool equal (const C& x, const C& y) {
    return x==y; }
  static bool unequal (const C& x, const C& y) {
    return x!=y; }
  static inline void def_type (const generic& name);
};

template<typename C>
struct define_type_helper<false,C> {
  static inline void def_type (const generic& name) {
    (void) name; }
};

template<bool Cond, typename C> inline void
define_type_helper<Cond,C>::def_type (const generic& name) {
  define_type_sub (name, type_id<C> ());
  define_type_sub (gen (GEN_TUPLE_TYPE, name), type_id<tuple<C> > ());
  define_type_sub (gen (GEN_ALIAS_TYPE, name), type_id<alias<C> > ());
  define_type_sub (gen (GEN_GENERIC_ALIAS_TYPE, name),
		   type_id<generic_alias<C> > ());

  define<always> (GEN_ALIAS, new_alias<C>);
  routine specializer= unary_routine (GEN_SPECIALIZE, new_genalias<C>);
  alias_specializer (type_id<C> (), specializer);
  routine getter= unary_routine (GEN_UNALIAS, get_genalias<C>);
  alias_getter (type_id<generic_alias<C> > (), getter);  
  getter= unary_routine (GEN_UNALIAS, get_alias<C>);
  alias_getter (type_id<alias<C> > (), getter);  
  routine setter= binary_routine (GEN_UNALIAS, set_genalias<C>);
  alias_setter (type_id<generic_alias<C> > (), setter);  
  setter= binary_routine (GEN_UNALIAS, set_alias<C>);
  alias_setter (type_id<alias<C> > (), setter);

  define_converter<always,C,alias<C> >
    (get_alias<C>, PENALTY_AUTOMATIC);
  define_converter<always,C,generic_alias<C> >
    (get_genalias<C>, PENALTY_AUTOMATIC);
  define_converter<always,alias<generic>,generic_alias<C> >
    (generalize_genalias<C>, PENALTY_AUTOMATIC);
  define_converter<always,alias<C>,generic_alias<C> >
    (incarnate_genalias<C>, PENALTY_AUTOMATIC);
  define<different<C,generic>,bool,C,C> (GEN_EQUAL, equal);
  define<different<C,generic>,bool,C,C> (GEN_UNEQUAL, unequal);
}

template<typename Cond, typename C> inline void
define_type (const generic& name) {
  typedef typename Cond::Cond When;
  define_type_helper<When::value,C>::def_type (name);
}

template<typename Cond> inline void
define_no_flatten (const generic& name) {
  define_constant<Cond,bool> (gen (GEN_NO_FLATTEN, name), true);
}

/******************************************************************************
* Extern glue definitions
******************************************************************************/

void define_prerequisites ();

void define_simple ();
bool using_simple ();
evaluator simple_evaluator ();

} // namespace mmx
#endif // __GLUE_HPP
