
/******************************************************************************
* MODULE     : tuple.hpp
* DESCRIPTION: Syntactic tuple type constructor
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __TUPLE_HPP
#define __TUPLE_HPP
#include "array.hpp"
#include "table.hpp"
#include "wrap.hpp"
#include "compound.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Tuple tuple<C>

/******************************************************************************
* The tuple class
******************************************************************************/

template<typename C>
class tuple {
MMX_ALLOCATORS
  generic rep;
public:
  inline generic operator * () const { return rep; }
  inline tuple (const generic& g): rep (g) {}
  inline tuple (const Tuple& t): rep (t.rep) {}
  inline C operator[] (nat i) const { return as<C> (rep[i+1]); }
};

TMPL generic
flatten (const Tuple& t) {
  return flatten (gen (GEN_TUPLE, cdr (compound_to_array (*t))));
}

WRAP_INDIRECT_IMPL(TMPL inline,Tuple)
PRINT_SUGAR(TMPL,Tuple)

TMPL inline nat N (const Tuple& t) { return N (*t) - 1; }

/******************************************************************************
* Useful conversions
******************************************************************************/

TMPL Tuple
as_tuple (const array<C>& a) {
  nat i, n= N(a);
  array<generic> t= array_fill<generic> (n+1);
  t[0]= GEN_TUPLE;
  for (i=0; i<n; i++)
    t[i+1]= as<generic> (a[i]);
  return Tuple (array_to_compound (t));
}

TMPL array<C>
as_array (const Tuple& t) {
  nat i, n= N(t);
  array<C> a= array_fill<C> (n);
  for (i=0; i<n; i++)
    a[i]= t[i];
  return a;
}

TMPL iterator<C>
iterate (const Tuple& t) {
  return iterate (as_array (t));
}

/******************************************************************************
* Type information for tuple types
******************************************************************************/

TMPL struct type_information<Tuple > { static nat id; };
TMPL nat type_information<Tuple >::id=
  new_tuple_type_id (type_information<C>::id);

void tuple_type_info (nat& id, bool& mode);

inline bool
is_tuple_type (nat id) {
  bool mode= false;
  tuple_type_info (id, mode);
  return mode;
}

inline nat
tuple_to_scalar (nat id) {
  bool mode= false;
  tuple_type_info (id, mode);
  return id;
}

inline nat
new_tuple_type_id (nat id) {
  bool mode= true;
  tuple_type_info (id, mode);
  return id;
}

#undef TMPL
#undef Tuple
} // namespace mmx
#endif // __TUPLE_HPP
