
/******************************************************************************
* MODULE     : evaluator.cpp
* DESCRIPTION: Abstract evaluators for symbolic expressions
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "base_evaluator.hpp"
#include "glue.hpp"
namespace mmx {

/******************************************************************************
* Default evaluation methods do not do much...
******************************************************************************/

environment evaluator_rep::get_environment () const {
  return global_environment (); }
void evaluator_rep::set (const generic& var, const generic& val) const {
  (void) var; (void) val; }
void evaluator_rep::reset (const generic& var) const {
  (void) var; }
void evaluator_rep::overload
  (const generic& var, const generic& val, nat pen) const {
    (void) var; (void) val; (void) pen; }
bool evaluator_rep::contains (const generic& var) const {
  (void) var; return false; }
generic evaluator_rep::get (const generic& var) const {
  return var; }
bool evaluator_rep::get (const generic& var, generic& val) const {
  (void) var; (void) val; return false; }

generic evaluator_rep::eval (const generic& x) const {
  return x; }
generic evaluator_rep::construct (const generic& x) const {
  return x; }
generic evaluator_rep::xapply (const generic& x) const {
  return x; }
generic evaluator_rep::apply (const generic& fun) const {
  return gen (fun); }
generic evaluator_rep::apply (const generic& fun, const generic& x1) const {
  return gen (fun, x1); }
generic evaluator_rep::apply (const generic& fun,
			      const generic& x1, const generic& x2) const {
  return gen (fun, x1, x2); }

/******************************************************************************
* Current evaluator management
******************************************************************************/

evaluator current_ev;
list<evaluator> stack_ev;

void
select_evaluator (const evaluator& ev) {
  /*
  cout << "[" << N (stack_ev)
       << ":" << inside (ev)
       << "<" << inside (current_ev)
       << "]"; cout.flush ();
  */
  stack_ev  = cons (current_ev, stack_ev);
  current_ev= ev;
}

void
select_output_evaluator () {
  static evaluator output_ev= simple_evaluator ();
  select_evaluator (output_ev);
}

void
restore_evaluator () {
  ASSERT (!nil (stack_ev), "empty evaluator stack");
  /*
  cout << "[" << N (stack_ev)-1
       << ":" << inside (current_ev)
       << ">" << inside (car (stack_ev))
       << "]"; cout.flush ();
  */
  current_ev= read_car (stack_ev);
  stack_ev  = read_cdr (stack_ev);
}

} // namespace mmx
