
/******************************************************************************
* MODULE     : compound_glue.cpp
* DESCRIPTION: Standard glue for compounds
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "compound.hpp"
#include "category_glue.hpp"
#include "generic_internal.hpp"
#include "list.hpp"
namespace mmx {

inline compound make_compound (const tuple<generic>& v) {
  return compound (array<generic> (iterate (v))); }
inline compound
make_compound_cons (const generic& fun, const iterator<generic>& it) {
  return compound (cons (fun, array<generic> (it))); }
inline bool generic_compoundP (const generic& g) {
  return false; }
inline bool compound_compoundP (const compound& v) {
  return true; }
inline int compound_length (const compound& v) {
  return N (as_array (v)); }
inline generic compound_access (const compound& v, const int& i) {
  return read (as_array (v), (nat) i); }
inline generic compound_car (const compound& v) {
  return read (as_array (v), (nat) 0); }
inline compound compound_cdr (const compound& v) {
  return compound (range (as_array (v), 1, N (as_array (v)))); }
inline compound compound_cons (const generic& x, const compound& v) {
  return compound (cons (x, as_array (v))); }
inline compound compound_append (const compound& v, const compound& w) {
  return compound (append (as_array (v), as_array (w))); }

iterator<generic>
compound_to_iterator (const compound& c) {
  array<generic> a= as_array (c);
  return iterate (a);
}

list<generic>
compound_to_list (const compound& c) {
  const array<generic> a= as_array (c);
  nat i, n= N(a);
  list<generic> l;
  for (i=n-1; i!=((nat) (-1)); i--)
    l= cons (a[i], l);
  return l;
}

compound
list_to_compound (const list<generic>& l2) {
  list<generic> l= l2;
  nat i, n= N(l);
  array<generic> a= array_fill<generic> (n);
  for (i=0; i<n; i++, l= read_cdr (l)) a[i]= read_car (l);
  return compound (a);
}

compound
compound_map (const generic& f, const compound& c) {
  routine fun= is<routine> (f)? as<routine> (f): default_routine (f);
  const array<generic> a= as_array (c);
  nat i, n= N(a);
  array<generic> b= array_fill<generic> (n);
  b[0]= a[0];
  for (i=1; i<n; i++) b[i]= fun->apply (a[i]);
  return compound (b);
}

void
define_compound () {
  //define_type<always,compound > ("Compound");
  define<always> ("compound", make_compound);
  define_from_iterator<always,generic> ("compound");
  define<always> ("compound", make_compound_cons);
  define<always> ("compound?", generic_compoundP);
  define<always> ("compound?", compound_compoundP);

  define<always> (GEN_SIZE, compound_length);
  define<always> (GEN_ACCESS, compound_access);
  define<always> ("car", compound_car);
  define<always> ("cdr", compound_cdr);
  define<always> ("cons", compound_cons);
  define<always> ("append", compound_append);

  define<always> ("as_iterator", compound_to_iterator);
  define<always> ("as_list", compound_to_list);
  define<always> ("as_compound", list_to_compound);
  define<always> ("map", compound_map);
}

} // namespace mmx
