
/******************************************************************************
* MODULE     : generic_glue.cpp
* DESCRIPTION: Standard glue for generics
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "category_glue.hpp"
#include "generic_internal.hpp"
namespace mmx {

generic default_neg (const generic& x) {
  return gen (GEN_MINUS, x); }
generic default_add (const generic& x, const generic& y) {
  return gen (GEN_PLUS, x, y); }
generic default_sub (const generic& x, const generic& y) {
  return gen (GEN_MINUS, x, y); }
generic default_mul (const generic& x, const generic& y) {
  return gen (GEN_TIMES, x, y); }
generic default_div (const generic& x, const generic& y) {
  return gen (GEN_OVER, x, y); }
generic default_quo (const generic& x, const generic& y) {
  if (y == 0) return 0; else return x/y; }
generic default_rem (const generic& x, const generic& y) {
  if (y == 0) return x; else return 0; }
generic default_gcd (const generic& x, const generic& y) {
  return generic ((x == 0 && y == 0)? 0: 1); }
generic default_lcm (const generic& x, const generic& y) {
  return generic ((x == 0 || y == 0)? 0: x*y); }
generic default_pow (const generic& x, const generic& y) {
  return gen (GEN_POWER, x, y); }
generic default_compose (const generic& x, const generic& y) {
  return gen (GEN_COMPOSE, x, y); }
generic default_access (const generic& x, const generic& y) {
  return gen (GEN_ACCESS, x, y); }
generic default_tuple_access (const generic& x, const tuple<generic>& y) {
  return gen (GEN_ACCESS, cons (x, as_array (y))); }

void
define_generic () {
  define<always> (GEN_MINUS, default_neg);
  define<always> (GEN_PLUS , default_add);
  define<always> (GEN_MINUS, default_sub);
  define<always> (GEN_TIMES, default_mul);
  define<always> (GEN_OVER , default_div);
  define<always> (GEN_DIV  , default_quo);
  define<always> (GEN_MOD  , default_rem);
  define<always> (GEN_GCD  , default_gcd);
  define<always> (GEN_LCM  , default_lcm);
  define<always> (GEN_POWER, default_pow);
  define<always> (GEN_COMPOSE, default_compose);
  define<always> (GEN_ACCESS, default_access);
  define<always> (GEN_ACCESS, default_tuple_access);
}

} // namespace mmx
