
/******************************************************************************
* MODULE     : atomic_format.hpp
* DESCRIPTION: Atomic formatted black box data
* COPYRIGHT  : (C) 2007  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __ATOMIC_FORMAT_HPP
#define __ATOMIC_FORMAT_HPP
#include "format.hpp"
namespace mmx {
//namespace bbx {
#define TMPL template<typename C>
#define PER(x) (*((C*) ((void*) (const_cast<char*> (x)))))
#define Format const format_rep*

TMPL void atomic_init (data x, Format xf) {
  (void) xf; (void) new ((void*) x) C (); }
TMPL void atomic_init_int (data x, int i, Format xf) {
  (void) xf; (void) new ((void*) x) C (i); }
TMPL void atomic_init_copy (data x, data y, Format xf) {
  (void) xf; (void) new ((void*) x) C (PER(y)); }
TMPL void atomic_clear (data x, Format xf) {
  (void) xf; ((C*) ((void*) x)) ->~ C (); }
TMPL void atomic_set (data x, Format xf) {
  (void) xf; PER(x)= C(); }
TMPL void atomic_set_int (data x, int i, Format xf) {
  (void) xf; PER(x)= i; }
TMPL void atomic_set_copy (data x, const data y, Format xyf) {
  (void) xyf; PER(x)= PER(y); }

TMPL generic atomic_flatten (const data x, Format xf) {
  (void) xf; return flatten (PER(x)); }
TMPL nat atomic_hash (const data x, Format xf) {
  (void) xf; return hash (PER(x)); }
TMPL bool atomic_equal (const data x, const data y, Format xyf) {
  (void) xyf; return PER(x) == PER(y); }
TMPL bool atomic_unequal (const data x, const data y, Format xyf) {
  (void) xyf; return PER(x) != PER(y); }
TMPL nat atomic_soft_hash (const data x, Format xf) {
  (void) xf; return soft_hash (PER(x)); }
TMPL bool atomic_eq (const data x, const data y, Format xyf) {
  (void) xyf; return eq (PER(x), PER(y)); }
TMPL bool atomic_neq (const data x, const data y, Format xyf) {
  (void) xyf; return neq (PER(x), PER(y)); }

TMPL void atomic_neg (data x, const data y, Format xyf) {
  (void) xyf; PER(x) = -PER(y); }
TMPL void atomic_neg_set (data x, Format xf) {
  (void) xf; PER(x) = -PER(x); }
TMPL void atomic_add (data x, const data y, const data z, Format xyzf) {
  (void) xyzf; PER(x) = PER(y) + PER(z); }
TMPL void atomic_add_set (data x, const data y, Format xyf) {
  (void) xyf; PER(x) += PER(y); }
TMPL void atomic_sub (data x, const data y, const data z, Format xyzf) {
  (void) xyzf; PER(x) = PER(y) - PER(z); }
TMPL void atomic_sub_set (data x, const data y, Format xyf) {
  (void) xyf; PER(x) -= PER(y); }
TMPL void atomic_mul (data x, const data y, const data z, Format xyzf) {
  (void) xyzf; PER(x) = PER(y) * PER(z); }
TMPL void atomic_mul_bi (data x, const data y, const data z,
			 Format yf, Format zf) {
  VERIFY_FORMAT (yf == zf);
  (void) yf; (void) zf; PER(x) = PER(y) * PER(z); }
TMPL void atomic_mul_set (data x, const data y, Format xyf) {
  (void) xyf; PER(x) *= PER(y); }
TMPL void atomic_div (data x, const data y, const data z, Format xyzf) {
  (void) xyzf; PER(x) = PER(y) / PER(z); }
TMPL void atomic_div_bi (data x, const data y, const data z,
			 Format yf, Format zf) {
  VERIFY_FORMAT (yf == zf);
  (void) yf; (void) zf; PER(x) = PER(y) / PER(z); }
TMPL void atomic_div_set (data x, const data y, Format xyf) {
  (void) xyf; PER(x) /= PER(y); }
TMPL void atomic_quo (data x, const data y, const data z, Format xyzf) {
  (void) xyzf; PER(x) = quo (PER(y), PER(z)); }
TMPL void atomic_quo_bi (data x, const data y, const data z,
			 Format yf, Format zf) {
  VERIFY_FORMAT (yf == zf);
  (void) yf; (void) zf; PER(x) = quo (PER(y), PER(z)); }
TMPL void atomic_rem (data x, const data y, const data z, Format xyzf) {
  (void) xyzf; PER(x) = rem (PER(y), PER(z)); }
TMPL void atomic_rem_bi (data x, const data y, const data z,
			 Format yf, Format zf) {
  VERIFY_FORMAT (yf == zf);
  (void) yf; (void) zf; PER(x) = rem (PER(y), PER(z)); }

TMPL
struct atomic_format_methods_rep: public format_methods_rep {
  atomic_format_methods_rep<C> (): format_methods_rep () {
    init= atomic_init<C>;
    init_int= atomic_init_int<C>;
    init_copy= atomic_init_copy<C>;
    clear= atomic_clear<C>;
    set= atomic_set<C>;
    set_int= atomic_set_int<C>;
    set_copy= atomic_set_copy<C>;

    flatten= atomic_flatten<C>;
    hash= atomic_hash<C>;
    equal= atomic_equal<C>;
    unequal= atomic_unequal<C>;
    soft_hash= atomic_hash<C>;
    eq= atomic_eq<C>;
    neq= atomic_neq<C>;

    neg= atomic_neg<C>;
    neg_set= atomic_neg_set<C>;
    add= atomic_add<C>;
    add_set= atomic_add_set<C>;
    sub= atomic_sub<C>;
    sub_set= atomic_sub_set<C>;
    mul= atomic_mul<C>;
    mul_bi= atomic_mul_bi<C>;
    mul_set= atomic_mul_set<C>;
    div= atomic_div<C>;
    div_bi= atomic_div_bi<C>;
    div_set= atomic_div_set<C>;
    /*
    quo= atomic_quo<C>;
    quo_bi= atomic_quo_bi<C>;
    rem= atomic_rem<C>;
    rem_bi= atomic_rem_bi<C>;
    */
  }
};

TMPL
struct atomic_format_rep: public format_rep {
  static atomic_format_methods_rep<C> mrep;
  atomic_format_rep<C> (): format_rep (sizeof (C), &mrep) {}
  ~atomic_format_rep<C> () {}
};

TMPL atomic_format_methods_rep<C>
atomic_format_rep<C>::mrep= atomic_format_methods_rep<C> ();

TMPL
struct format_helper {
  static format f;
  static inline format get () { return f; }
};

TMPL format
format_helper<C>::f= format (new atomic_format_rep<C> ());

TMPL inline format atomic_format () { return format_helper<C>::get (); }

#undef PER
#undef TMPL
#undef Format
//} // namespace bbx
} // namespace mmx
#endif // __ATOMIC_FORMAT_HPP
