
/******************************************************************************
* MODULE     : compound.hpp
* DESCRIPTION: Compound generic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __COMPOUND_HPP
#define __COMPOUND_HPP
#include "array.hpp"
#include "mixed_symbol.hpp"
#include "wrap.hpp"
namespace mmx {

/******************************************************************************
* Compound expressions
******************************************************************************/

class compound {
MMX_ALLOCATORS
  array<generic> v;
  //symbol<array<generic> > v;
public:
  inline array<generic> operator * () const { return v; }
  //inline symbol<array<generic> > operator * () const { return v; }
  inline friend array<generic> as_array (const compound& v) { return v.v; }
  //inline friend array<generic> as_array (const compound& v) { return *v.v; }
  inline compound (const array<generic>& v2): v (v2) {}
  //inline compound (const array<generic>& v2):
  //v (symbol<array<generic> > (v2)) {}
  inline compound (const compound& v2): v (v2.v) {}
};

array<generic> compound_to_array (const generic& g);
generic array_to_compound (const array<generic>& v);
generic flatten (const compound& c);

WRAP_WRAPPED_IMPL(inline,compound);
PRINT_SUGAR(,compound)

inline compound melt (const compound& c) {
  return compound (melt (as_array (c))); }
inline compound fuse (const compound& c) {
  return compound (fuse (as_array (c))); }

/******************************************************************************
* Compound expressions as generics
******************************************************************************/

STMPL
struct internal<compound> {
  typedef mixed_symbol<compound> Repr;
  static inline Repr encode (const compound& x) { return Repr (x); }
  static inline Repr encode (const compound& x, bool flag) {
    return Repr (x, flag); }
  static inline compound decode (const Repr& x) { return *x; }
};

STMPL
struct inspector<compound> {
  static inline nat length (const compound& v) {
    return mmx::N (as_array (v)); }
  static inline generic access (const compound& v, nat i) {
    VERIFY (i < mmx::as_array (v), "out of range");
    return read (as_array (v), i); }
};

} // namespace mmx
#endif // __COMPOUND_HPP
