
/******************************************************************************
* MODULE     : format.hpp
* DESCRIPTION: Formatted black box data
* COPYRIGHT  : (C) 2007  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __FORMAT_HPP
#define __FORMAT_HPP
#include "basix.hpp"
namespace mmx {
//namespace bbx {
#define Format const format_rep*
#define VERIFY_FORMAT(cond) VERIFY(cond, "format mismatch")

struct format_rep;
struct format;
struct format_methods_rep;
typedef char* data;
typedef format_methods_rep* format_methods;

struct format_methods_rep {
MMX_ALLOCATORS

  void (*init) (data x, Format xf);
  void (*init_int) (data x, int i, Format xf);
  void (*init_copy) (data x, const data y, Format xyf);
  void (*init_array) (data x, nat n, Format xf);
  void (*init_array_int) (data x, nat n, int i, Format xf);
  void (*init_array_copy) (data x, nat n, const data y, Format xyf);
  void (*clear) (data x, Format xf);
  void (*clear_array) (data x, nat n, Format xf);
  void (*set) (data x, Format xf);
  void (*set_int) (data x, int i, Format xf);
  void (*set_copy) (data x, const data y, Format xyf);

  generic (*flatten) (const data x, Format xf);
  nat (*hash) (const data x, Format xf);
  bool (*equal) (const data x, const data y, Format xyf);
  bool (*unequal) (const data x, const data y, Format xyf);
  nat (*soft_hash) (const data x, Format xf);
  bool (*eq) (const data x, const data y, Format xyf);
  bool (*neq) (const data x, const data y, Format xyf);

  void (*neg) (data x, const data y, Format xyf);
  void (*neg_set) (data x, Format xf);
  void (*add) (data x, const data y, const data z, Format xyzf);
  void (*add_set) (data x, const data y, Format xyf);
  void (*sub) (data x, const data y, const data z, Format xyzf);
  void (*sub_set) (data x, const data y, Format xyf);
  void (*mul) (data x, const data y, const data z, Format yzf);
  void (*mul_bi) (data x, const data y, const data z, Format yf, Format zf);
  void (*mul_set) (data x, const data y, Format xyf);
  void (*div) (data x, const data y, const data z, Format yzf);
  void (*div_bi) (data x, const data y, const data z, Format yf, Format zf);
  void (*div_set) (data x, const data y, Format xyf);
  void (*quo) (data x, const data y, const data z, Format yzf);
  void (*quo_bi) (data x, const data y, const data z, Format yf, Format zf);
  void (*quo_set) (data x, const data y, Format xyf);
  void (*rem) (data x, const data y, const data z, Format yzf);
  void (*rem_bi) (data x, const data y, const data z, Format yf, Format zf);
  void (*rem_set) (data x, const data y, Format xyf);

  format_methods_rep ();
};

struct format_rep REP_STRUCT {
  nat size;
  format_methods m;
  inline format_rep (nat size2, format_methods m2): size (size2), m (m2) {}
  virtual inline ~format_rep () {}
};

struct format {
INDIRECT_PROTO (format, format_rep)
  //inline operator format_rep* () { return rep; }
  inline operator const format_rep* () const { return rep; }
};
INDIRECT_IMPL (format, format_rep)

inline void init (data x, Format xf) {
  xf->m->init (x, xf); }
inline void init (data x, int i, Format xf) {
  xf->m->init_int (x, i, xf); }
inline void init (data x, const data y, Format xyf) {
  xyf->m->init_copy (x, y, xyf); }
inline void init_array (data x, nat n, Format xf) {
  xf->m->init_array (x, n, xf); }
inline void init_array (data x, nat n, int i, Format xf) {
  xf->m->init_array_int (x, n, i, xf); }
inline void init_array (data x, nat n, const data y, Format xyf) {
  xyf->m->init_array_copy (x, n, y, xyf); }
inline void clear (data x, Format xf) {
  xf->m->clear (x, xf); }
inline void clear_array (data x, nat n, Format xf) {
  xf->m->clear_array (x, n, xf); }
inline void set (data x, Format xf) {
  xf->m->set (x, xf); }
inline void set (data x, int i, Format xf) {
  xf->m->set_int (x, i, xf); }
inline void set (data x, const data y, Format xyf) {
  xyf->m->set_copy (x, y, xyf); }

inline generic flatten (const data x, Format xf) {
  return xf->m->flatten (x, xf); }
inline nat hash (const data x, Format xf) {
  return xf->m->hash (x, xf); }
inline bool equal (const data x, const data y, Format xyf) {
  return xyf->m->equal (x, y, xyf); }
inline bool unequal (const data x, const data y, Format xyf) {
  return xyf->m->unequal (x, y, xyf); }
inline nat soft_hash (const data x, Format xf) {
  return xf->m->soft_hash (x, xf); }
inline bool eq (const data x, const data y, Format xyf) {
  return xyf->m->eq (x, y, xyf); }
inline bool neq (const data x, const data y, Format xyf) {
  return xyf->m->neq (x, y, xyf); }

inline void neg (data x, const data y, Format xyf) {
  xyf->m->neg (x, y, xyf); }
inline void neg (data x, Format xf) {
  xf->m->neg_set (x, xf); }
inline void add (data x, const data y, const data z, Format xyzf) {
  xyzf->m->add (x, y, z, xyzf); }
inline void add (data x, const data y, Format xyf) {
  xyf->m->add_set (x, y, xyf); }
inline void sub (data x, const data y, const data z, Format xyzf) {
  xyzf->m->sub (x, y, z, xyzf); }
inline void sub (data x, const data y, Format xyf) {
  xyf->m->sub_set (x, y, xyf); }

inline void mul (data x, const data y, const data z, Format yzf) {
  yzf->m->mul (x, y, z, yzf); }
inline void mul (data x, const data y, const data z, Format yf, Format zf) {
  yf->m->mul_bi (x, y, z, yf, zf); }
inline void mul (data x, const data y, Format xyf) {
  xyf->m->mul_set (x, y, xyf); }
inline void div (data x, const data y, const data z, Format yzf) {
  yzf->m->div (x, y, z, yzf); }
inline void div (data x, const data y, const data z, Format yf, Format zf) {
  yf->m->div_bi (x, y, z, yf, zf); }
inline void div (data x, const data y, Format xyf) {
  xyf->m->div_set (x, y, xyf); }
inline void quo (data x, const data y, const data z, Format yzf) {
  yzf->m->quo (x, y, z, yzf); }
inline void quo (data x, const data y, const data z, Format yf, Format zf) {
  yf->m->quo_bi (x, y, z, yf, zf); }
inline void quo (data x, const data y, Format xyf) {
  xyf->m->quo_set (x, y, xyf); }
inline void rem (data x, const data y, const data z, Format yzf) {
  yzf->m->rem (x, y, z, yzf); }
inline void rem (data x, const data y, const data z, Format yf, Format zf) {
  yf->m->rem_bi (x, y, z, yf, zf); }
inline void rem (data x, const data y, Format xyf) {
  xyf->m->rem_set (x, y, xyf); }

#undef Format
//} // namespace bbx
} // namespace mmx
#endif // __FORMAT_HPP
