
/******************************************************************************
* MODULE     : literal.hpp
* DESCRIPTION: Atomic literals
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __LITERAL_HPP
#define __LITERAL_HPP
#include "string.hpp"
#include "mixed_symbol.hpp"
#include "wrap.hpp"
namespace mmx {

/******************************************************************************
* Literals
******************************************************************************/

class literal {
MMX_ALLOCATORS
  string s;
  //symbol<string> s;
public:
  inline string operator * () const { return s; }
  //inline symbol<string> operator * () const { return s; }
  inline friend string as_string (const literal& s) { return *s; }
  //inline string as_string (const literal& s) { return *s; }
  inline literal (char *s2): s (string (s2)) {}
  inline literal (const string& s2): s (s2) {}
  inline literal (const literal& s2): s (s2.s) {}
};

generic flatten (const literal& s);
string literal_to_string (const generic& g);

WRAP_WRAPPED_IMPL(inline,literal);
PRINT_SUGAR(,literal)

template<typename C> C
make_literal (const literal& lit) {
  return C (as_string (lit));
}

/******************************************************************************
* Literals as generics
******************************************************************************/

STMPL
struct internal<literal> {
  typedef mixed_symbol<literal> Repr;
  static inline Repr encode (const literal& x) { return Repr (x); }
  static inline Repr encode (const literal& x, bool flag) {
    return Repr (x, flag); }
  static inline literal decode (const Repr& x) { return *x; }
};

} // namespace mmx
#endif // __LITERAL_HPP
