
/******************************************************************************
* MODULE     : format.cpp
* DESCRIPTION: Formatted black box data
* COPYRIGHT  : (C) 2007  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "format.hpp"
namespace mmx {
//namespace bbx {
#define Format const format_rep*

void default_init (data x, Format xf) {
  ERROR ("init not implemented"); }
void default_init_int (data x, int i, Format xf) {
  ERROR ("init_int not implemented"); }
void default_init_copy (data x, const data y, Format xyf) {
  ERROR ("init_copy not implemented"); }
void default_init_array (data x, nat n, Format xf) {
  const nat xs= xf->size;
  for (nat k=0; k<n; k++) default_init (x+k*xs, xf); }
void default_init_array_int (data x, nat n, int i, Format xf) {
  const nat xs= xf->size;
  for (nat k=0; k<n; k++) default_init_int (x+k*xs, i, xf); }
void default_init_array_copy (data x, nat n, const data y, Format xyf) {
  const nat xs= xyf->size;
  for (nat k=0; k<n; k++) default_init_copy (x+k*xs, y, xyf); }
void default_clear (data x, Format xf) {
  ERROR ("init_clear not implemented"); }
void default_clear_array (data x, nat n, Format xf) {
  const nat xs= xf->size;
  for (nat k=0; k<n; k++) default_clear (x+k*xs, xf); }
void default_set (data x, Format xf) {
  ERROR ("init_set not implemented"); }
void default_set_int (data x, int i, Format xf) {
  ERROR ("init_set_int not implemented"); }
void default_set_copy (data x, const data y, Format xyf) {
  ERROR ("init_set_copy not implemented"); }

generic default_flatten (const data x, Format xf) {
  ERROR ("flatten not implemented"); }
nat default_hash (const data x, Format xf) {
  ERROR ("hash not implemented"); }
bool default_equal (const data x, const data y, Format xyf) {
  ERROR ("equal not implemented"); }
bool default_unequal (const data x, const data y, Format xyf) {
  return equal (x, y, xyf); }
nat default_soft_hash (const data x, Format xf) {
  ERROR ("soft_hash not implemented"); }
bool default_eq (const data x, const data y, Format xyf) {
  ERROR ("eq not implemented"); }
bool default_neq (const data x, const data y, Format xyf) {
  return !eq (x, y, xyf); }

void default_neg (data x, const data y, Format xyf) {
  ERROR ("neg not implemented"); }
void default_neg_set (data x, Format xf) {
  neg (x, x, xf); }
void default_add (data x, const data y, const data z, Format xyzf) {
  ERROR ("add not implemented"); }
void default_add_set (data x, const data y, Format xyf) {
  add (x, x, y, xyf); }
void default_sub (data x, const data y, const data z, Format xyzf) {
  ERROR ("sub not implemented"); }
void default_sub_set (data x, const data y, Format xyf) {
  sub (x, x, y, xyf); }
void default_mul (data x, const data y, const data z, Format yzf) {
  ERROR ("mul not implemented"); }
void default_mul_bi (data x, const data y, const data z, Format yf, Format zf){
  ERROR ("mul_bi not implemented"); }
void default_mul_set (data x, const data y, Format xyf) {
  mul (x, x, y, xyf); }
void default_div (data x, const data y, const data z, Format yzf) {
  ERROR ("div not implemented"); }
void default_div_bi (data x, const data y, const data z, Format yf, Format zf){
  ERROR ("div_bi not implemented"); }
void default_div_set (data x, const data y, Format xyf) {
  div (x, x, y, xyf); }
void default_quo (data x, const data y, const data z, Format yzf) {
  ERROR ("quo not implemented"); }
void default_quo_bi (data x, const data y, const data z, Format yf, Format zf){
  ERROR ("quo_bi not implemented"); }
void default_quo_set (data x, const data y, Format xyf) {
  quo (x, x, y, xyf); }
void default_rem (data x, const data y, const data z, Format yzf) {
  ERROR ("rem not implemented"); }
void default_rem_bi (data x, const data y, const data z, Format yf, Format zf){
  ERROR ("rem_bi not implemented"); }
void default_rem_set (data x, const data y, Format xyf) {
  rem (x, x, y, xyf); }

format_methods_rep::format_methods_rep () {
  init= default_init;
  init_int= default_init_int;
  init_copy= default_init_copy;
  init_array= default_init_array;
  init_array_int= default_init_array_int;
  init_array_copy= default_init_array_copy;
  clear= default_clear;
  clear_array= default_clear_array;
  set= default_set;
  set_int= default_set_int;
  set_copy= default_set_copy;

  flatten= default_flatten;
  hash= default_hash;
  equal= default_equal;
  unequal= default_unequal;
  soft_hash= default_soft_hash;
  eq= default_eq;
  neq= default_neq;

  neg= default_neg;
  neg_set= default_neg_set;
  add= default_add;
  add_set= default_add_set;
  sub= default_sub;
  sub_set= default_sub_set;
  mul= default_mul;
  mul_bi= default_mul_bi;
  mul_set= default_mul_set;
  div= default_div;
  div_bi= default_div_bi;
  div_set= default_div_set;
  quo= default_quo;
  quo_bi= default_quo_bi;
  quo_set= default_quo_set;
  rem= default_rem;
  rem_bi= default_rem_bi;
  rem_set= default_rem_set;
}

#undef Format
//} // namespace bbx
} // namespace mmx
