/******************************************************************************
* MODULE     : math_printer.cpp
* DESCRIPTION: Printing math expressions for TeXmacs
* COPYRIGHT  : (C) 2007  Gregoire Lecerf
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "string.hpp"
#include "table.hpp"
#include "generic.hpp"
#include "literal.hpp"
#include "compound.hpp"

namespace mmx {

static void
init_math_symbol_table (table<generic, generic, eq_op>& t) {
  t[GEN_ACCESS] = "_"; // or "%sqaccess"
  t[GEN_AND] = "/\\";
  t[GEN_CATALAN] = "%mathcatalan";
  t[GEN_COMPOSE] = "%circ";
  t[GEN_DELTA] = "%delta";
  t[GEN_DERIVATIVE] = "D";
  t[GEN_DOT] = "%dotaccess";
  t[GEN_DIV] = "div";
  t[GEN_E] = "%mathe";
  t[GEN_EQUAL] = "=";
  t[GEN_EQUIV] = "<=>";
  t[GEN_EULER] = "%matheuler";
  t[GEN_FACTORIAL] = "factorial";
  t[GEN_GTR] = ">";
  t[GEN_GTREQ] = ">=";
  t[GEN_GTRGTR] = ">>";
  t[GEN_I] = "%mathi";
  t[GEN_IMPLIES] = "=>";
  t[GEN_IN] = "%in";
  t[GEN_INFINITY] = "%infty";
  t[GEN_INTO] = "->";
  t[GEN_LESS] = "<";
  t[GEN_LESSEQ] = "<=";
  t[GEN_LESSLESS] = "<<";
  t[GEN_MAPSTO] = ":->";
  t[GEN_MINUS] = "-";
  t[GEN_MOD] = "mod";
  t[GEN_NOT] = "!";
  t[GEN_OR] = "\\/";
  t[GEN_OVER] = "/";
  t[GEN_PARTIAL] = "%partial";
  t[GEN_PI] = "%mathpi";
  t[GEN_PLUS] = "+&";
  t[GEN_POWER] = "^";
  t[GEN_PRIME] = "%prime";
  t[GEN_RANGE] = "%colons"; // problem with iterator flattening
  t[GEN_RANGEEQ] = "%ldots";
  t[GEN_ROW] = "row";
  t[GEN_SEQAND] = "and";
  t[GEN_SEQOR] = "or";
  t[GEN_SIZE] = "%card";
  t[GEN_SQTUPLE] = "list";
  t[GEN_TIMES] = "*&";
  t[GEN_TRANSTYPE] = ":>";
  t[GEN_TUPLE] = "tuple";
  //t[GEN_TYPE] = ":";
  t[GEN_UNEQUAL] = "!=";
  t[GEN_VWHERE] = "|";
  t[GEN_WHERE] = "||";
  t[GEN_XOR] = "%veebar";

  t["alpha"] = "%alpha";
  t["beta"] = "%beta";
  t["gamma"] = "%gamma";
  t["delta"] = "%delta";
  t["epsilon"] = "%varepsilon";
  t["zeta"] = "%zeta";
  t["eta"] = "%eta";
  t["theta"] = "%theta";
  t["iota"] = "%iota";
  t["kappa"] = "%kappa";
  t["lambda"] = "%lambda";
  t["mu"] = "%mu";
  t["nu"] = "%nu";
  t["xi"] = "%xi";
  t["omicron"] = "%omicron";
  t["pi"] = "%pi";
  t["rho"] = "%rho";
  t["sigma"] = "%sigma";
  t["tau"] = "%tau";
  t["upsilon"] = "%upsilon";
  t["phi"] = "%varphi";
  t["chi"] = "%chi";
  t["psi"] = "%psi";
  t["omega"] = "%omega";

  t["Alpha"] = "%Alpha";
  t["Beta"] = "%Beta";
  t["Gamma"] = "%Gamma";
  t["Delta"] = "%Delta";
  t["Epsilon"] = "%Epsilon";
  t["Zeta"] = "%Zeta";
  t["Eta"] = "%Eta";
  t["Theta"] = "%Theta";
  t["Iota"] = "%Iota";
  t["Kappa"] = "%Kappa";
  t["Lambda"] = "%Lambda";
  t["Mu"] = "%Mu";
  t["Nu"] = "%Nu";
  t["Xi"] = "%Xi";
  t["Omicron"] = "%Omicron";
  t["Pi"] = "%Pi";
  t["Rho"] = "%Rho";
  t["Sigma"] = "%Sigma";
  t["Tau"] = "%Tau";
  t["Upsilon"] = "%Upsilon";
  t["Phi"] = "%Phi";
  t["Chi"] = "%Chi";
  t["Psi"] = "%Psi";
  t["Omega"] = "%Omega";
}

void
math_print (ostream& out, const generic& g) {
  static table<generic, generic, eq_op> math_symbol_table;
  if (N(math_symbol_table)==0)
    init_math_symbol_table (math_symbol_table);

  generic tmp;
  if (math_symbol_table->get (g, tmp)) out << literal_to_string (tmp);
  else if (is<literal> (g)) out << literal_to_string (g);
  else if (is<compound> (g)) {
    nat i, n= N (g);
    out << "(";
    if (n>0) {
      if (eq (g[0], GEN_SQTUPLE) && n>1 &&
	  is<compound> (g[1]) && eq (g[1][0], GEN_ROW))
	out << "matrix";
      else math_print (out, g[0]);
    }
    for (i=1; i<n; i++) {
      out << " ";
      math_print (out, g[i]);
    }
    out << ")";
  }
  else {
    select_output_evaluator ();
    generic f= flatten (g);
    restore_evaluator ();
    math_print (out, f);
  }
}

} // namespace mmx
