
/******************************************************************************
* MODULE     : parse_tools.cpp
* DESCRIPTION: Tools for parsing
* COPYRIGHT  : (C) 2007  Gregoire Lecerf
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "parse_tools.hpp"
#include "literal.hpp"

namespace mmx {

parse_instance::parse_instance (const string& f, nat i, const string& s) {
  lex_file_name= f;
  lex_input_number= i;
  lex_string= const_cast<char*> (S(s));
  lex_length= N(s);
  lex_start = 0;
  lex_start_line = 0;
  lex_start_begin_line = 0;
  lex_prev = 0;
  lex_prev_line = 0;
  lex_prev_begin_line = 0;
  lex_pos = 0;
  lex_line = 0;
  lex_begin_line = 0;
}

/******************************************************************************
* Keep track of locations in source code
******************************************************************************/

// NOTE: carefully force the table with literals to be deleted
// after *source_locs

static literal* dummy;

static table<source_location, generic>* source_locs;

void init_parse_tools () {
  dummy= new literal ("dummy");
  source_locs= new table<source_location, generic> (); }

void term_parse_tools () {
  delete source_locs;
  delete dummy; }

// l = (*source_locs) location[g] means that the
// location of the generic g in the source code is stored in l.

void
source_insert (const generic& g, const source_location& l) {
  (*source_locs) [g] = l;
}

void
source_delete (const generic& g) {
  reset ((*source_locs), g);
}
  
void
source_locate (const generic& g, source_location& l) {
  if ((*source_locs) -> contains (g)) {
    source_location l1 = read (*source_locs, g);
    if (nil (l))
      l = l1;
    else 
      if (!nil (l1)) {
	l.obj = l1.obj;
	if (l.file_name == l1.file_name && l.input_number == l1.input_number) {
	  l.begin = min (l.begin, l1.begin);
	  l.end = max (l.end, l1.end);
	}
	else {
	  VERIFY (l.file_name == l1.file_name, "wrong file names");
	  VERIFY (l.input_number == l1.input_number, "wrong line numbers");
	}
      }
  }
  if (is<compound> (g)) {
    iterator<generic> it = iterate (compound_to_array (g));
    for (; busy(it); ++it)
      source_locate (*it, l);
  }
  l.obj = g;
}

generic
source_extend (const generic& g1, const generic& g2) {
  source_location l;
  source_locate (g1, l);
  source_locate (g2, l);
  source_delete (g1);
  source_delete (g2);
  l.obj = g1;
  source_insert (g1, l);
  return g1;
}

void
print_source_location_table (ostream& out) {
  out << (*source_locs);
}

} // namespace mmx
