<TeXmacs|1.0.6.10>

<style|mmxdoc>

<\body>
  <tmdoc-title|The <value|mmx> syntax>

  This document describes the syntax of the <value|mmx> interpreter, provided
  by the executable <verbatim|mmx-light>, whose usage can be obtained with
  the <verbatim|--help> option. Input is read on the standard input by
  default. If <name|Readline> has been enabled then newline insertion in
  interactive mode is possible by typing Meta-Return.

  <section|Variables and constants>

  <subsection|Identifiers>

  Identifiers are formed using letters, digits and the special characters
  <verbatim|_> and <verbatim|?>. Identifiers must start with a letter or
  <verbatim|_>. Identifiers match the <verbatim|[a-zA-Z_]+[a-zA-Z0-9_?]*>.
  Examples of identifiers are <verbatim|x>, <verbatim|y2>, <verbatim|my_var>
  and <verbatim|test?>.

  Some special predefined constants are:

  <\description>
    <item*|<verbatim|true>>The boolean constant <verbatim|true>.

    <item*|<verbatim|false>>The boolean constant <verbatim|false>.

    <item*|<verbatim|mmout>>The standard output stream.

    <item*|<verbatim|mmerr>>The standard error stream.
  </description>

  Notice that streams can be used with the operators
  <verbatim|\<less\>\<less\>> as in C++:

  <\mmx-fragment>
    mmout \<less\>\<less\> 3 \<less\>\<less\> "\\n";
  </mmx-fragment>

  <subsection|Literals>

  In addition to identifiers, <value|mmx> provides three types of literal
  constants:

  <\description>
    <item*|String literals>Strings can be braced into double quotes " ...
    "<verbatim|>. Inside such a string a double quote must be blackslashed.
    In order to avoid blackslashing one can use the stronger string
    delimiters /" ... "/.

    <\mmx-fragment>
      s1 := "This is a string";

      s2 := "Print \\"foo\\"";

      s2 := /"Print "foo"/;
    </mmx-fragment>

    <item*|Integer literals>An integer literal is a sequence of digits,
    possible preceded by a minus sign. It matches the regular expression
    <verbatim|[-]?[0-9]+>. Examples: <verbatim|123456789123456789>,
    <verbatim|-123>.

    <item*|Floating literals>A floating literal is a sequence of digits with
    a decimal point inside and an optional exponent. It matches the regular
    expression <verbatim|[-]?[0-9]+[.][0-9]+[[eE][-]?[0-9]+]?>. Some
    examples:

    <\mmx-fragment>
      z := 0.0;

      w := -3.14159;

      x := 1.11e2007
    </mmx-fragment>

    Note that <verbatim|0.> is not permited, one must write <verbatim|0.0>.
  </description>

  <section|Operators>

  Here we list the operators in increasing priority order, together with
  their internal names:

  \;

  DEFINE "==", ASSIGN ":=", DEFINE_MACRO ``==\<gtr\>'', ASSIGN_MACRO
  ``:=\<gtr\>'', PLUS_ASSIGN "+=", MINUS_ASSIGN "-=", TIMES_ASSIGN "*=",
  OVER_ASSIGN "/="

  \;

  LEFT_FLUX "\<less\>\<less\>", LEFT_FLUX_VAR "\<less\>\<less\>*",
  LEFT_FLUX_STR "\<less\>\<less\>%", RIGHT_FLUX "\<gtr\>\<gtr\>"

  \;

  IMPLIES "=\<gtr\>", EQUIVALENT "\<less\>=\<gtr\>"

  \;

  SEQOR "or", OR "\\/", XOR "xor"

  \;

  SEQAND "and", AND "/\\"

  \;

  EQUAL "=", LESS "\<less\>", LEQ "\<less\>=", GREATER "\<gtr\>", GEQ
  "\<gtr\>=", NOT_EQUAL "!=", NOT_LESS "!\<less\>", NOT_LEQ "!\<less\>=",
  NOT_GREATER "!\<gtr\>", NOT_GEQ "!\<gtr\>="

  \;

  INTO "-\<gtr\>", MAPSTO ":-\<gtr\>"

  \;

  RANGE "::", RANGEEQ ".."

  \;

  PLUS "+", MINUS "-" (binary), OPLUS "@+", OMINUS "@-"

  \;

  TIMES "*", OVER "/", OTIMES "@*", OOVER "@/", DIV "div", MOD "mod", COMPOSE
  "@"

  \;

  NOT "!" (infix unitary), \ MINUS "-" (infix unitary), OMINUS "@-" (infix
  unitary),\ 

  INC "++" (infix unitary), DEC "--" (infix unitary), SIZE "#" (infix
  unitary)

  \;

  POWER "^"

  \;

  INC "++" (postfix unitary), DEC "--" (postfix unitary), NOT "!" (postfix
  unitary), QUOTE "'" (postfix unitary), BACKQUOTE ```'' (postfix unitary),

  TILDA ``~'' (postifix unitary), ACCESS ``.''

  <subsection|Ranges>

  <verbatim|a..b> means the range [a,b], while <verbatim|a::b> stands for the
  half open range [a,b).

  <section|Sequences>

  Tuples are written inside <verbatim|(>...<verbatim|)>. Elements are
  separated by <verbatim|,>. Row-tuples rows are separated by <verbatim|;>.

  Lists are written inside <verbatim|[>...<verbatim|]>.

  <\cpp-fragment>
    v := (1, 2, 3);

    x := ("a", 3, (7, 0.4));

    M := (1, 2; 3, 4; 5, 6);

    l := [2, 3, 5, 7, 11];
  </cpp-fragment>

  Automatic constructions are possible through the <verbatim|\|> <abbr|resp.>
  <verbatim|\|\|> notation:

  <\cpp-fragment>
    v := ( i^2 \| i in 1..3 );

    l := [ i * j \| i in 1..3, j in 1..4 ];

    m := matrix (i*j \| i in 1..5 \|\| j in 1..5)
  </cpp-fragment>

  <\itemize>
    <item>Note associativity: <verbatim|(1, (2, 3))> is the same as
    <verbatim|(1, 2, 3)>.

    <item>Note that <verbatim|1..10> is not a tuple but a generator.
  </itemize>

  <section|Functions and macros>

  <subsection|Function definitions>

  Any function definition can be preceded by quantifiers
  <verbatim|forall(>...<verbatim|)> or <verbatim|exists(>...<verbatim|)>.

  A function definition is done as follows:

  <\mmx-fragment>
    name <verbatim|(>arg1: type1, arg2: type2, ...<verbatim|)>: returned_type
    == ...
  </mmx-fragment>

  <value|mmx> also provides a syntax for lambda expressions:

  <\mmx-fragment>
    <verbatim|lambda> (arg1: type1, arg2: type2, ...): returned_type
    <verbatim|do> ...
  </mmx-fragment>

  Notice that any of the type specifications can be omitted, in which case
  the type is assumed to be <verbatim|Generic>. Expression macros can be
  defined by using <verbatim|macro> instead of <verbatim|lambda>.

  When function or macro have only one argument, <verbatim|()> can be omited.

  <\mmx-fragment>
    square x ==\<gtr\> x^2; // macro for squaring

    discr (a, b, c) == b^2 - 4*a*c;

    \;

    forall (R)

    gcd (p: Polynomial(R), q: Polynomial(R)): Polynomial(R) == {...}
  </mmx-fragment>

  <\itemize>
    <item>Nested declarations <verbatim|shift(x)(y) == x+y;>

    <item>How macros are evaluated.

    <item>More explanations and examples...
  </itemize>

  <subsection|Function Calls>

  A function or a macro <verbatim|foo> is called in the usual way:
  <verbatim|foo(arg1, arg2,...)>.

  If <verbatim|foo> is unitary then <verbatim|()> can be omited, but note
  that <verbatim|foo a b c> is equivalent to <verbatim|foo(a(b(c)))>.
  Function call is always by value.

  <section|Control flow>

  <subsection|Sequence of Instructions>

  Instructions are separated with <verbatim|;>.

  <\cpp-fragment>
    x := 1;

    y := 3;

    z := x*y;
  </cpp-fragment>

  <subsection|Conditionals>

  The construction is as follows: <verbatim|if> condition <verbatim|then>
  block1 <verbatim|else> block2.

  condition is any expression that evaluates to a boolean. block1 and block2
  are either one instruction or a block of instructions braced into
  <verbatim|{>...<verbatim|}>. The <verbatim|else> part is optional.

  <\mmx-fragment>
    if true then 1;

    if a = b then 1 else 2;

    if i = 0 then { x := 1; y := 2} else { z := 3; mmerr \<less\>\<less\>
    "error" }
  </mmx-fragment>

  <subsection|Case separation>

  <subsection|Loops>

  Loops are constructed as follows: [<verbatim|for> E1] [<verbatim|while> E2]
  [<verbatim|until> E3] [<verbatim|step> E4] <verbatim|do> block.

  Here <verbatim|[>...<verbatim|]> means that the expression is optional.
  block is an instruction or a sequence of instruction delimited by
  <verbatim|{>...<verbatim|}>. <verbatim|break> exits the loop, and
  <verbatim|continue> goes to the next step of the loop.

  <\mmx-fragment>
    do mmout \<less\>\<less\> "no end "; // This loop has no end

    for i in 1..3 do mmout \<less\>\<less\> i \<less\>\<less\> " ";

    i := 0; while i \<less\> 0 do { mmout \<less\>\<less\> i \<less\>\<less\>
    " "; i := i + 1 }
  </mmx-fragment>

  <section|Comments>

  Comments starting with <verbatim|//> extend to the end of the physical
  line. Such a comment may appear at the start of a line or following
  whitespace or code, but not within a string literal. Multi-line comments
  must be braced into <verbatim|/{ ... }/ >and can be nested.

  <\mmx-fragment>
    x := 1; // Assign 1 to x.

    \;

    y := 2;

    /{ This

    \ \ \ is multi-line

    \ \ \ comment about y.

    }/
  </mmx-fragment>

  <\with|font|roman>
    \;
  </with>

  <section|Technical Notes>

  The Mathemagix grammar is specified in the file <verbatim|mmx-parser.ypp>
  of the <verbatim|basix> module.
</body>