
/******************************************************************************
* MODULE     : complex_ball_glue.cpp
* DESCRIPTION: Standard glue for complex balls
* COPYRIGHT  : (C) 2007  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "complex_ball.hpp"
#include "floating_meta.hpp"
#include "complexify_meta.hpp"
#include "category_glue.hpp"
namespace mmx {
#define Real floating<>
#define Complex complexify<Real >

complex_ball make_ball_1 (const Complex& c) {
  return complex_ball (c); }
complex_ball make_ball_2 (const Complex& c, const Real& r) {
  return complex_ball (c, as<floating<single_floating> > (r)); }
complex_ball make_complex (const real_ball& x, const real_ball& y) {
  return complex_ball (x, y); }
complexify<generic> as_complex_generic (const complex_ball& z) {
  return complexify<generic> (as<generic> (Re (z)), as<generic> (Im (z))); }

void
define_complex_ball () {
  define_type<always,complex_ball > ("Complex_ball");
  define<always> ("ball", make_ball_1);
  define<always> ("ball", make_ball_2);
  define<always> ("complex", make_complex);
  define_upgrader<always,complex_ball,Complex > (PENALTY_HOMOMORPHISM);
  define_upgrader<always,complex_ball,real_ball> (PENALTY_INCLUSION);
  define_caster<always,complexify<generic>,complex_ball>
    (as_complex_generic, PENALTY_FALL_BACK);

  define_field<always,complex_ball > ();
  define_elementary<always,complex_ball > ();
}

} // namespace mmx
