
/******************************************************************************
* MODULE     : complex_ball.hpp
* DESCRIPTION: Standard complex balls
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_COMPLEX_BALL_HPP
#define __MMX_COMPLEX_BALL_HPP
#include "real_ball.hpp"
#include "complexify.hpp"
#include "type_props.hpp"
namespace mmx {
typedef complexify<real_ball_center> complex_ball_center;
typedef real_ball_radius complex_ball_radius;

class complex_ball {
MMX_ALLOCATORS
private:
  // Define fields of same size as "Ball" type (see implementation class)
  // IMPORTANT: The sizes MUST match for binary compatibility
  void* x;
  void* y;
  void* r;

public:
  complex_ball (const complex_ball& x);
  ~complex_ball ();
  complex_ball& operator = (const complex_ball& x);
  complex_ball ();
  complex_ball (int i);
  complex_ball (nat i);
  complex_ball (double x);
  complex_ball (const integer& i);
  complex_ball (const rational& x);
  complex_ball (const real_ball_center& x);
  complex_ball (const real_ball& x, const real_ball& y= 0);
  complex_ball (const complex_ball_center& z);
  complex_ball (const complex_ball_center& c, const complex_ball_radius& r);
};

STYPE_TO_TYPE(STMPL,real_type,complex_ball,real_ball);
STYPE_TO_TYPE(STMPL,abs_type,complex_ball,real_ball);
STYPE_TO_TYPE(STMPL,complex_type,real_ball,complex_ball);
STYPE_TO_TYPE(STMPL,center_type,complex_ball,complex_ball_center);
STYPE_TO_TYPE(STMPL,radius_type,complex_ball,complex_ball_radius);

// Definitions as plain routines fixes compilation problem with gcc 2.95.3
complex_ball complex_ball_nan ();
complex_ball complex_ball_infinity ();
complex_ball complex_ball_fuzz ();
complex_ball complex_ball_log2 ();
complex_ball complex_ball_pi ();
complex_ball complex_ball_euler ();
complex_ball complex_ball_imaginary ();

STMPL
struct constant_helper<complex_ball > {
  inline static complex_ball nan () { return complex_ball_nan (); }
  inline static complex_ball infinity () { return complex_ball_infinity (); }
  inline static complex_ball fuzz () { return complex_ball_fuzz (); }
  inline static complex_ball log2 () { return complex_ball_log2 (); }
  inline static complex_ball pi () { return complex_ball_pi (); }
  inline static complex_ball euler () { return complex_ball_euler (); }
  inline static complex_ball imaginary () { return complex_ball_imaginary (); }
};

STMPL
struct make_ball_helper<complex_ball,complex_ball_center,complex_ball_radius> {
  static inline complex_ball
  val (const complex_ball_center& c, const complex_ball_radius& r) {
    return complex_ball (c, r); }
};

WRAP_WRAPPED_PROTO(,complex_ball);
WRAP_PRINT_PROTO(,complex_ball);
WRAP_FIELD_PROTO(,complex_ball);
WRAP_SHIFT_PROTO(,complex_ball);
WRAP_RING_EQUAL_PROTO(,complex_ball);
WRAP_NUMBER_CHECK_PROTO(,complex_ball);
WRAP_ELEMENTARY_PROTO(,complex_ball);
WRAP_COMPLEX_PROTO(,complex_ball,real_ball);

generic flatten (const complex_ball& x, nat dd);
complex_ball sharpen (const complex_ball& z);
complex_ball_center center (const complex_ball& z);
complex_ball_radius radius (const complex_ball& z);
complex_ball_radius abs_D (const complex_ball& x);
complex_ball_radius abs_U (const complex_ball& x);
template<> inline int var_sign (const complex_ball& z) {
  if (z == 0) return 0; else return 1; }

} // namespace mmx
#endif // __MMX_COMPLEX_BALL_HPP
