
/******************************************************************************
* MODULE     : complex_ball.cpp
* DESCRIPTION: Standard complex balls
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "complex_ball_impl.hpp"
#include "floating.hpp"
#include "ball.hpp"
namespace mmx {

complex_ball::complex_ball (const complex_ball& x): rep (x.rep) {}
complex_ball::~complex_ball () {}
complex_ball& complex_ball::operator = (const complex_ball& x) {
  rep= x.rep; return *this; }
complex_ball::complex_ball (): rep () {}
complex_ball::complex_ball (int i): rep (i) {}
complex_ball::complex_ball (nat i): rep (i) {}
complex_ball::complex_ball (double x): rep (x) {}
complex_ball::complex_ball (const integer& i): rep (i) {}
complex_ball::complex_ball (const rational& x): rep (x) {}
complex_ball::complex_ball (const real_ball_center& x):
  rep (gaussian (copy (x))) {}
complex_ball::complex_ball (const real_ball& x, const real_ball& y):
  rep (x.rep, y.rep) {}
complex_ball::complex_ball (const complex_ball_center& z):
  rep (gaussian (copy (Re (z)), copy (Im (z)))) {}
complex_ball::complex_ball (const complex_ball_center& c,
			    const complex_ball_radius& r):
  rep (c, r) {}

WRAP_WRAPPED_IMPL(,complex_ball);
WRAP_PRINT_IMPL(,complex_ball);
WRAP_NUMBER_CHECK_IMPL(,complex_ball);

generic flatten (const complex_ball& z, nat dd) { return flatten (z.rep, dd); }
complex_ball sharpen (const complex_ball& z) { return sharpen (z.rep); }
complex_ball_center center (const complex_ball& z) { return center (z.rep); }
complex_ball_radius radius (const complex_ball& z) { return radius (z.rep); }
complex_ball_radius abs_D (const complex_ball& x) { return abs_D (x.rep); }
complex_ball_radius abs_U (const complex_ball& x) { return abs_U (x.rep); }

} // namespace mmx
