
/******************************************************************************
* MODULE     : complexify_glue.hpp
* DESCRIPTION: Glue for complexified numbers
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_COMPLEXIFY_GLUE_HPP
#define __MMX_COMPLEXIFY_GLUE_HPP
#include "complexify.hpp"
#include "category_meta.hpp"
#include "category_lift_glue.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Complexify complexify<C>

TMPL Complexify construct_complexify (const C& x, const C& y) {
  return Complexify (x, y); }
TMPL C complex_re (const Complexify& z) { return Re (z); }
TMPL C complex_im (const Complexify& z) { return Im (z); }

template<typename Hyp, typename C> void
define_complexify () {
  typedef both<has<C,ring_cat>,Hyp> Cond;
  typedef both<different<C,generic>,Cond> Specialized;

  define_type<Cond,Complexify > (gen ("Complexify", type_name<C> ()));
  define<Cond> ("complex", construct_complexify<C>);
  define_upgrader<Cond,Complexify,C> ();
  define<Cond> ("Re", complex_re<C>);
  define<Cond> ("Im", complex_im<C>);

  define_ring<Cond,Complexify> ();
  define_field<both<has<C,field_cat>,Cond>, Complexify> ();
  lift_scalar_ring_ops<Cond,Complexify,C> ();
  define_left <both<has<C,field_cat>,Cond>, Complexify, C, ldiv_op> ();
  define_right<both<has<C,field_cat>,Cond>, Complexify, C, rdiv_op> ();
  define_elementary<both<has<C,elementary_cat>, Specialized>, Complexify> ();
}

#undef TMPL
#undef Complexify
} // namespace mmx
#endif // __MMX_COMPLEXIFY_GLUE_HPP
