
/******************************************************************************
* MODULE     : integer_glue.cpp
* DESCRIPTION: Standard glue for integers
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "integer_meta.hpp"
#include "category_glue.hpp"
#include "literal.hpp"
#include "list.hpp"
//#include "jet_integer.hpp"
namespace mmx {

static integer
int_as_integer (const int& i) {
  return i;
}

static int
integer_as_int (const integer& i) {
  return as_int (i);
}

static generic
integer_constructor_int (const int& i) {
  return as<generic> (integer (i));
}

static integer integer_and (const integer& i, const integer& j) {
  return i & j; }
static integer integer_or (const integer& i, const integer& j) {
  return i | j; }
static integer integer_xor (const integer& i, const integer& j) {
  return i ^ j; }
static integer integer_not (const integer& i) {
  return ~i; }
static integer integer_bit_size (const integer& i) {
  return bit_size (i); }
static bool integer_access (const integer& i, const integer& j) {
  return i[as_int (j)]; }
static integer integer_hamming_norm (const integer& i) {
  return hamming_norm (i); }
static integer integer_hamming_distance (const integer& i, const integer& j) {
  return hamming_distance (i, j); }

/*
static integer*
decode_list (const list<generic>& l) {
  integer* v= mmx_new<integer> (N(l));
  nat i= 0;
  list<generic> counter= l;
  while (!nil (counter)) {
    ASSERT (is<integer> (car (counter)), "integer expected");
    v[i]= as<integer> (car (counter));
    counter= cdr (counter);
    i++;
  }
  return v;
}

static list<generic>
encode_list (integer* v, nat n) {
  list<generic> l;
  for (nat i= n-1; i != ((nat) -1); i--)
    l= cons (as<generic> (v[i]), l);
  return l;
}

static integer
encode_kronecker (const list<generic>& l, const int& bits) {
  integer* v= decode_list (l);
  integer result;
  kronecker_encode (result, v, N(l), bits);
  mmx_delete<integer> (v, N(l));
  return result;
}

static list<generic>
decode_kronecker (const integer& i, const int& n, const int& bits) {
  integer* v= mmx_new<integer> (n);
  kronecker_decode (v, n, bits, i);
  list<generic> l= encode_list (v, n);
  mmx_delete<integer> (v, N(l));
  return l;
}

static list<generic>
multiply_kronecker (const list<generic>& l1, const list<generic>& l2) {
  integer* v1= decode_list (l1);
  integer* v2= decode_list (l2);
  integer* w = mmx_new<integer> (N(l1)+N(l2)-1);
  kronecker_mul (w, v1, N(l1), v2, N(l2));
  list<generic> m= encode_list (w, N(l1)+N(l2)-1);
  mmx_delete<integer> (v1, N(l1));
  mmx_delete<integer> (v2, N(l2));
  mmx_delete<integer> (w,  N(l1)+N(l2)-1);
  return m;
}
*/

void
define_integer () {
  define_type<always,integer> ("Integer");
  define_no_flatten<always> ("Integer");
  define<always> ("literal_integer", make_literal<integer>);
  define_constructor<always,int> (integer_constructor_int);
  define_upgrader<always,int,integer> (integer_as_int, PENALTY_HOMOMORPHISM);
  define_caster<always,integer,int> (int_as_integer, PENALTY_CAST);

  define_euclidean_ring<always,integer> ();
  define_ordering<always,integer> ();
  define_factorial<always,integer> ();

  define<always> (GEN_AND, integer_and);
  define<always> (GEN_OR, integer_or);
  define<always> (GEN_XOR, integer_xor);
  define<always> (GEN_NOT, integer_not);
  define<always> (GEN_SIZE, integer_bit_size);
  define<always> (GEN_ACCESS, integer_access);
  define<always> ("hamming_norm", integer_hamming_norm);
  define<always> ("hamming_distance", integer_hamming_distance);
  /*
  define<always> ("encode_kronecker", encode_kronecker);
  define<always> ("decode_kronecker", decode_kronecker);
  define<always> ("multiply_kronecker", multiply_kronecker);
  */
}

} // namespace mmx
