
/******************************************************************************
* MODULE     : mmx_gmp.cpp
* DESCRIPTION: Additional code for interface to gmp and mpfr
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rational.hpp"
#include "evaluator.hpp"
#include "generic_internal.hpp"
namespace mmx {

string
as_string (const integer& i) {
  char* s= mpz_get_str (NULL, 10, *i);
  string r= s;
  mmx_free (s, strlen (s) + 1);
  /* FIXME: for upcoming version of GMP
  void (*free_func) (void *, size_t);
  mp_get_memory_functions (NULL, NULL, &free_func);
  free_func ((void*) s, strlen (s) + 1);
  */
  return r;
}

generic
construct (const integer& i) {
  return construct (as<generic> (i));
}

generic
construct (const rational& x) {
  return construct (as<generic> (x));
}

generic
flatten (const integer& i) {
  if (current_ev->contains (gen (GEN_NO_FLATTEN, GEN_INTEGER_TYPE)))
    return construct (as<generic> (i));
  return as_string (i);
}

generic
flatten (const rational& x) {
  if (current_ev->contains (gen (GEN_NO_FLATTEN, GEN_RATIONAL_TYPE)))
    return construct (as<generic> (x));
  if (denominator (x) == 1) return generic (as_string (numerator (x)));
  return generic (as_string (numerator (x))) /
         generic (as_string (denominator (x)));
}

} // namespace mmx
