
/******************************************************************************
* MODULE     : floating_glue.cpp
* DESCRIPTION: Standard glue for floating point numbers
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "floating_meta.hpp"
#include "category_glue.hpp"
#include "literal.hpp"
#include "jet_floating.hpp"
namespace mmx {
#define Floating floating<>

class bit_precision_rep: public alias_rep<int> {
MMX_ALLOCATORS
  int prec;
public:
  inline bit_precision_rep () {}
  int get () const { return get_precision (); }
  int& open () const { return *(const_cast<int*> (&prec)); }
  void close () const { set_precision (prec); }
};

static Floating*
decode_list (const list<generic>& l) {
  Floating* v= mmx_new<Floating > (N(l));
  nat i= 0;
  list<generic> counter= l;
  while (!nil (counter)) {
    if (is<integer> (car (counter)))
      v[i]= Floating (as<integer> (car (counter)));
    else if (is<rational> (car (counter)))
      v[i]= Floating (as<rational> (car (counter)));
    else if (is<Floating > (car (counter)))
      v[i]= as<Floating > (car (counter));
    else ASSERT (false, "floating expected");
    counter= cdr (counter);
    i++;
  }
  return v;
}

static list<generic>
encode_list (Floating* v, nat n) {
  list<generic> l;
  for (nat i= n-1; i != ((nat) -1); i--)
    l= cons (as<generic> (v[i]), l);
  return l;
}

static list<generic>
multiply_kronecker (const list<generic>& l1, const list<generic>& l2) {
  Floating* v1= decode_list (l1);
  Floating* v2= decode_list (l2);
  Floating* w = mmx_new<Floating > (N(l1)+N(l2)-1);
  kronecker_mul (w, v1, N(l1), v2, N(l2));
  list<generic> m= encode_list (w, N(l1)+N(l2)-1);
  mmx_delete<Floating > (v1, N(l1));
  mmx_delete<Floating > (v2, N(l2));
  mmx_delete<Floating > (w , N(l1)+N(l2)-1);
  return m;
}

void
define_floating () {
  static alias<int> prec= new bit_precision_rep ();
  define_constant<always,alias<int> > ("bit_precision", prec);

  define_type<always,Floating > ("Floating");
  define<always> ("literal_floating", make_literal<Floating >);
  define_upgrader<always,Floating,rational> (PENALTY_HOMOMORPHISM);

  define_field<always,Floating > ();
  define_elementary<always,Floating > ();
  define_ordering<always,Floating > ();

  define<always> ("multiply_kronecker", multiply_kronecker);
}

} // namespace mmx
