
/******************************************************************************
* MODULE     : mmx_mpfr.hpp
* DESCRIPTION: Interface to mpfr
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_MPFR_HPP
#define __MMX_MPFR_HPP
#include <gmp.h>
#include <mpfr.h>
#include "mmx_gmp.hpp"

namespace mmx {

/******************************************************************************
* Global variables
******************************************************************************/

extern nat        mmx_precision;     // current bit precision
extern mpfr_rnd_t mmx_rounding_mode; // current rounding mode

inline nat set_precision (nat prec) {
  nat old= mmx_precision; mmx_precision= prec; return old; }
inline void reset_precision (nat prec) { mmx_precision= prec; }
inline nat get_precision () { return mmx_precision; }

inline mpfr_rnd_t set_rounding_mode (mpfr_rnd_t mode) {
  mpfr_rnd_t old= mmx_rounding_mode; mmx_rounding_mode= mode; return old; }
inline void reset_rounding_mode (mpfr_rnd_t mode) { mmx_rounding_mode= mode; }
inline mpfr_rnd_t get_rounding_mode () { return mmx_rounding_mode; }

/******************************************************************************
* Extra routines
******************************************************************************/

class string;
extern string mpfr_to_string (const mpz_t x);
extern string mpfr_to_string (const mpfr_t x, nat bits= 0);
extern string zero_to_string (const mpfr_t err);
extern ostream& operator << (ostream& out, const mpfr_t x);

inline nat mpfr_prec_round_up (nat prec) {
  return BITS_PER_LIMB * ((prec + BITS_PER_LIMB - 1) / BITS_PER_LIMB); }
inline mp_exp_t mpfr_get_step (const mpfr_t arg) {
  return mpfr_get_exp (arg) - mpfr_get_prec (arg); }
inline mp_exp_t mpfr_safe_get_step (const mpfr_t arg) {
  if (mpfr_sgn (arg) == 0) return mpfr_get_emin ();
  else return mpfr_get_exp (arg) - mpfr_get_prec (arg); }
inline mp_exp_t mpfr_safe_get_exp (const mpfr_t arg) {
  if (mpfr_sgn (arg) == 0) return mpfr_get_emin ();
  else return mpfr_get_exp (arg); }
double mpfr_get_magnitude (const mpfr_t arg);

}
#endif // __MMX_MPFR_HPP
