
/******************************************************************************
* MODULE     : mmx_gmp.cpp
* DESCRIPTION: Additional code for interface to gmp and mpfr
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "mmx_mpfr.hpp"
#include "integer.hpp"
#include "string.hpp"
namespace mmx {
nat        mmx_precision    = 64;
mpfr_rnd_t mmx_rounding_mode= GMP_RNDN;

double
mpfr_get_magnitude (const mpfr_t arg) {
  if (mpfr_inf_p (arg) || !mpfr_number_p (arg))
    return (double) mpfr_get_emax ();
  if (mpfr_sgn (arg) == 0)
    return (double) mpfr_get_emin ();
  mp_exp_t expo= mpfr_get_exp (arg);
  mpfr_t aux;
  mpfr_init2 (aux, mpfr_get_prec (arg));
  mpfr_div_2si (aux, arg, expo, GMP_RNDU);
  return ((double) expo) + log (mpfr_get_d (aux, GMP_RNDU)) / log (2.0);
  //return ((double) expo) + log2 (mpfr_get_d (aux, GMP_RNDU));
}

string
mpfr_to_string (const mpz_t x) {
  char* s= mpz_get_str (NULL, 10, x);
  string r= s;
  mpfr_free_str (s);
  return r;
}

string
mpfr_to_string (const mpfr_t x, nat bits) {
  string r;
  if (mpfr_nan_p (x) != 0) return "NaN";
  if (mpfr_inf_p (x) != 0) {
    if (mpfr_sgn (x) < 0) return "-Infty";
    return "Infty";
  }
  nat digs= bits==0? 0: max (((int) (0.301029995 * bits)) - 1, 2);
  mp_exp_t exponent;
  char* mantissa_s= mpfr_get_str (NULL, &exponent, 10, digs, x, GMP_RNDN);
  int first= 0;
  if (mantissa_s[0] == '-') { r << "-"; first++; }
  r << mantissa_s[first];
  r << ".";
  r << (mantissa_s + (first + 1));
  r << "e" << as_string (exponent-1);
  mpfr_free_str (mantissa_s);
  return r;
}

string
zero_to_string (const mpfr_t err) {
  string r;
  mp_exp_t exponent;
  char* mantissa_s= mpfr_get_str (NULL, &exponent, 10, 2, err, GMP_RNDN);
  r << "0.0e" << as_string (exponent-1);
  mpfr_free_str (mantissa_s);
  return r;
}

ostream&
operator << (ostream& out, const mpfr_t x) {
  return out << mpfr_to_string (x, 0);
}

} // namespace mmx
