
/******************************************************************************
* MODULE     : polynomial_generic_glue.cpp
* DESCRIPTION: Generic polynomials
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "glue.hpp"
#include "dense_polynomial.hpp"
#include "integer.hpp"
#include "literal.hpp"
#include "compound.hpp"
#include "big_sum.hpp"
#include "big_product.hpp"
#include "generic_internal.hpp"
namespace mmx {
#define Polynomial polynomial<generic>
#define Iterator iterator<generic>
#define Table table<generic,generic>

generic as_polynomial (const generic& x, const generic& v) {
  return current_ev->apply ("as_polynomial", x, v); }

Polynomial
generic_to_polynomial (const generic& x, const generic& v) {
  if (x == v) return Polynomial (generic (1), (nat) 1);
  else return Polynomial (x);
}

Polynomial
sum_to_polynomial (const big_sum& s, const generic& v) {
  const Table t= *s;
  array<Polynomial > a;
  for (Iterator it= entries (t); busy (it); ++it)
    a << t[*it] * as_polynomial (*it, v);
  return big_add (a);
}

Polynomial
product_to_polynomial (const big_product& p, const generic& v) {
  const Table t= *p;
  array<Polynomial > a;
  for (Iterator it= entries (t); busy (it); ++it) {
    generic expo= t[*it];
    if (is<integer> (expo)) {
      integer n= as<integer> (expo);
      if (n >= 0 && n < 1000000) {
	a << binpow (as_polynomial (*it, v), (nat) as_int (n));
	continue;
      }
    }
    a << Polynomial (pow (*it, expo));
  }
  return big_mul (a);
}

generic
polynomial_to_generic (const Polynomial& p, const generic& v) {
  nat i, n= N(p);
  array<generic> a= array_fill<generic> (n);
  for (i=0; i<n; i++) a[i]= p[i] * pow (v, generic (i));
  return big_add (a);
}

void
define_polynomial_generic () {
  define<always> ("as_polynomial", generic_to_polynomial);
  define<always> ("as_polynomial", sum_to_polynomial);
  define<always> ("as_polynomial", product_to_polynomial);
  define<always> ("as_generic", polynomial_to_generic);
}

#undef Polynomial
#undef Iterator
#undef Table
} // namespace mmx
