
/******************************************************************************
* MODULE     : big_product.hpp
* DESCRIPTION: Big commutative products
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __BIG_PRODUCT_HPP
#define __BIG_PRODUCT_HPP
#include "mixed_symbol.hpp"
#include "wrap.hpp"
namespace mmx {
#define Table table<generic,generic>

/******************************************************************************
* Big products (in reality power products)
******************************************************************************/

class big_product {
MMX_ALLOCATORS
  Table prod;
public:
  inline Table operator * () const { return prod; }
  inline big_product (const Table& t): prod (t) {}
  inline big_product (const big_product& p): prod (p.prod) {}
};

generic flatten (const big_product& p);
Table product_to_table (const generic& x);
generic table_to_product (const Table& t);

inline big_product melt (const big_product& p) {
  return big_product (melt (*p)); }
inline big_product fuse (const big_product& p) {
  return big_product (fuse (*p)); }

WRAP_INDIRECT_IMPL(inline,big_product)
PRINT_SUGAR(,big_product)

/******************************************************************************
* Big products as generics
******************************************************************************/

STMPL
struct internal<big_product> {
  typedef mixed_symbol<big_product> Repr;
  // Using mixed_symbols instead of symbols allows for melting
  static inline Repr encode (const big_product& x) { return Repr (x); }
  static inline big_product decode (const Repr& x) { return *x; }
};

#undef Table
} // namespace mmx
#endif // __BIG_PRODUCT_HPP
