
/******************************************************************************
* MODULE     : evaluator_test.cpp
* DESCRIPTION: Test evaluation of generic expressions
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolix_glue.hpp"
#include "rational.hpp"
using namespace mmx;

int
main () {
  select_evaluator (symbolix_evaluator<rational> ());

  generic f ("f");
  generic x ("x");
  generic y ("y");
  generic z= f (x, y);
  generic t= xsqtuple (comma (comma (x, x), comma (y, z)));
  generic u= from_lisp ("(+ (f x x) (g (^ x x)))");
  generic one = as<generic,rational> (1);
  generic two = gen (GEN_PLUS, one, one);
  generic half= gen (GEN_OVER, one, two);

  cout << "x\t= " << x << "\n";
  cout << "y\t= " << y << "\n";
  cout << "z\t= " << z << "\n";
  cout << "t\t= " << t << "\n";
  cout << "u\t= " << u << "\n";
  cout << "one\t= " << one << "\n";
  cout << "two\t= " << two << "\n";
  cout << "half\t= " << half << "\n\n";

  cout << "*x\t= " << current_ev->eval (x) << "\n";
  cout << "*y\t= " << current_ev->eval (y) << "\n";
  cout << "*z\t= " << current_ev->eval (z) << "\n";
  cout << "*t\t= " << current_ev->eval (t) << "\n";
  cout << "*u\t= " << current_ev->eval (u) << "\n";
  cout << "*one\t= " << current_ev->eval (one) << "\n";
  cout << "*two\t= " << current_ev->eval (two) << "\n";
  cout << "*half\t= " << current_ev->eval (half) << "\n\n";

  cout << "2\t= " << one + one << "\n\n";


  {
  generic f ("f");
  generic x ("x");
  generic y ("y");
  generic z= f (x, y);
  generic p (49);

  cout << "x\t= " << x << "\n";
  cout << "y\t= " << y << "\n";
  cout << "z\t= " << z << "\n";
  cout << "p\t= " << p << "\n\n";

  cout << "x = y\t= " << (x==y) << "\n";
  cout << "x != y\t= " << (x!=y) << "\n";
  cout << "z = z\t= " << (z==z) << "\n";
  cout << "x = z\t= " << (x==z) << "\n";
  cout << "x y\t= " << (x*y) << "\n";
  cout << "x + 1\t= " << (x+1) << "\n";
  cout << "x + 0\t= " << (x+0) << "\n";
  cout << "z z\t= " << z*z << "\n";
  cout << "z z z\t= " << z*z*z << "\n";
  cout << "z / 7\t= " << z/7 << "\n";
  cout << "z / 111\t= " << z/111 << "\n";
  cout << "p p\t= " << p*p << "\n";
  cout << "p / p\t= " << p/p << "\n";
  cout << "21 / p\t= " << 21/p << "\n\n";

  cout << "x x\t\t\t= " << x*x << "\n";
  cout << "2 x y\t\t\t= " << 2*x*y << "\n";
  cout << "x 2 y\t\t\t= " << x*2*y << "\n";
  cout << "x y + x y\t\t= " << x*y + x*y << "\n";
  cout << "2 x y + y x\t\t= " << 2*x*y + y*x << "\n";
  cout << "2 x y + x y\t\t= " << 2*x*y + x*y << "\n";
  cout << "2 x y - y x\t\t= " << 2*x*y - y*x << "\n";
  cout << "2 x y - y x - x y\t= " << 2*x*y - y*x - x*y << "\n";
  cout << "x (x + y)\t\t= " << x*(x+y) << "\n";
  cout << "(x + y) (x - y)\t\t= " << (x+y) * (x-y) << "\n";
  cout << "2 (x + y) - x\t\t= " << 2*(x+y) - x << "\n\n";

  cout << "x x / x\t\t\t= " << x*x/x << "\n";
  cout << "x / (2 y)\t\t= " << x/(2*y) << "\n";
  cout << "x / (2 y) x\t\t= " << (x/(2*y))*x << "\n";
  cout << "x / (2 y) 2 x\t\t= " << (x/(2*y))*(2*x) << "\n";
  cout << "x / (2 y) 4 x\t\t= " << (x/(2*y))*(4*x) << "\n";
  cout << "(x^2)^3\t\t\t= " << pow (pow (x, 2), 3) << "\n";
  cout << "(x^x)^y\t\t\t= " << pow (pow (x, x), y) << "\n";
  cout << "(x y)^y\t\t\t= " << pow (x*y, y) << "\n";
  cout << "(x y)^y / x\t\t= " << pow (x*y, y) / x << "\n";
  cout << "(x y)^y / (x y)\t\t= " << pow (x*y, y) / (x*y) << "\n";
  cout << "(x y)^y / (x y)^2\t= " << pow (x*y, y) / pow (x*y, 2) << "\n";
  cout << "((x + y)^x)^y\t\t= " << pow (pow (x+y, x), y) << "\n";
  cout << "2^x 2^(1-x)\t\t= " << pow (2, x) * pow (2, 1-x) << "\n";
  cout << "3 2^x 2^(1-x)\t\t= " << 3 * pow (2, x) * pow (2, 1-x) << "\n";
  cout << "(x y)^3 / (y z)^5\t= " << pow (x*y, 3) / pow (y*z, 5) << "\n";
  cout << "(x y)^5 / (y z)^3\t= " << pow (x*y, 5) / pow (y*z, 3) << "\n\n";

  cout << "sum (x, y, z)\t\t= "
       << big_add (array<generic> (x, y, z)) << "\n";
  cout << "sum (x, y, x + z)\t= "
       << big_add (array<generic> (x, y, x+z)) << "\n";
  cout << "prod (x, y, z)\t\t= "
       << big_mul (array<generic> (x, y, z)) << "\n";
  cout << "prod (x, y, x + z)\t= "
       << big_mul (array<generic> (x, y, x+z)) << "\n";
  cout << "prod (x, x y, x + z)\t= "
       << big_mul (array<generic> (x, x*y, x+z)) << "\n";
  cout << "prod (x + z, x, x y)\t= "
       << big_mul (array<generic> (x+z, x, x*y)) << "\n\n";

  cout << "exp (x)\t\t\t= " << exp (x) << "\n";
  cout << "exp (x) exp (y)\t\t= " << exp (x) * exp (y) << "\n";
  cout << "log (2 x y)\t\t= " << log (2*x*y) << "\n";
  cout << "log (2 x / y)\t\t= " << log (2*x/y) << "\n";
  cout << "sqrt (x)\t\t= " << sqrt (x) << "\n";
  cout << "sqrt (x)'\t\t= " << derive (sqrt (x), x) << "\n";
  cout << "sqrt (x)''\t\t= " << derive (sqrt (x), x, 2) << "\n";
  cout << "(x^x)'\t\t\t= " << derive (pow (x, x), x) << "\n";
  cout << "(x^x)''\t\t\t= " << derive (pow (x, x), x, 2) << "\n";
  cout << "(x^(x^x))'\t\t= " << derive (pow (x, pow (x, x)), x) << "\n";
  cout << "((x y) / (y x))'\t= " << derive ((x*y) / (y*x), x) << "\n";
  cout << "((x y) / (y x))''\t= " << derive ((x*y) / (y*x), x, 2) << "\n";
  cout << "(x y^y / x)'\t\t= " << derive ((x * pow (y, y)) / x, x) << "\n";
  cout << "f (x)'\t\t\t= " << derive (f(x), x) << "\n";
  cout << "f (x sin (x))'\t\t= " << derive (f (x * sin(x)), x) << "\n";
  cout << "f (x, sin (x y), y)'\t= " << derive (f (x, sin(x*y), y), x) << "\n";
  cout << "tan (tan (x))'\t\t= " << derive (tan (tan (x)), x) << "\n";
  cout << "tan (tan (x))''\t\t= " << derive (tan (tan (x)), x, 2) << "\n";
  cout << "arcsin (arcsin (x))'\t= " << derive (asin (asin (x)), x) << "\n";
  cout << "arcsin (x + y)'\t\t= " << derive (asin (x + y), x) << "\n";
  //cout << "f(x*sin(x))''\t\t= "
  //<< derive (derive (f (x * sin(x)), x), x) << "\n";
  //cout << "(x^(x^x))''\t\t= "
  //<< derive (derive (pow (x, pow (x, x)), x), x) << "\n";

  table<generic,generic> t;
  t[x]= x+y;
  t[y]= sin(y);
  generic a= derive (pow (sin (x), y), x, 2);
  cout << "a\t\t\t= " << a << "\n";
  cout << "s(a)\t\t\t= " << substitute (a, as<generic> (t)) << "\n";

  cout << "solve (sin(x)+3=y^2-3)\t= "
       << solve (gen (GEN_EQUAL, sin (x) + 3, y*y - 3), x) << "\n";
  cout << "solve (x-y,x+y-3)\t= "
       << solve (as<generic> (list<generic> (x-y,x+y-3)),
		 as<generic> (list<generic> (x, y))) << "\n";
  cout << "solve (x-sin(y),x+y-3)\t= "
       << solve (as<generic> (list<generic> (x-sin(y),x+y-3)),
		 as<generic> (list<generic> (x, y))) << "\n";
  cout << "solve (x+y,x+y-3)\t= "
       << solve (as<generic> (list<generic> (x+y,x+y-3)),
		 as<generic> (list<generic> (x, y))) << "\n";
  cout << "solve (x+y,x+y)\t\t= "
       << solve (as<generic> (list<generic> (x+y,x+y)),
		 as<generic> (list<generic> (x, y))) << "\n\n";
  }

  /*
  {
    define_standard_vector ();
    generic v= current_ev->apply ("vector", x, y);
    generic w= current_ev->apply ("vector", x+y, x*y);
    cout << "v\t= " << v << "\n";
    cout << "w\t= " << w << "\n";
    cout << "v+w\t= " << v+w << "\n";
  }
  */

  restore_evaluator ();
  return 0;
}
